Package: RMeDPower2
Type: Package
Title: Design and Modeling for Repeated Measures Studies
Version: 1.0.2
Authors@R: c(
    person("Min-Gyoung", "Shin",
           email = "mingyoung.shin@gladstone.ucsf.edu",
           role = c("aut")),
    person("Reuben", "Thomas",
           email = "reuben.thomas@gladstone.ucsf.edu",
           role = c("aut", "cre"))
    )
Description: Provides complete functionality to analyse data from repeated
    measures experiments with hierarchical or crossed experimental designs.
    Supports testing modeling assumptions, identifying outlier observations
    and experimental units, estimating statistical power, and performing
    sample size calculations. Uses linear mixed effects models via 'lme4'
    and simulation-based power analysis via 'simr'. Handles both normal and
    non-normal error distributions including binomial and Poisson families. For more details see Shin et al. (2022) <doi:10.1101/2022.07.18.500490>, Bates et al. (2015) <doi:10.18637/jss.v067.i01>, Green and MacLeod (2016) <doi:10.1111/2041-210X.12504>, Hartig (2024) <doi:10.32614/CRAN.package.DHARMa>, Nieuwenhuis et al. (2012) <doi:10.32614/RJ-2012-011>, Millard  (2013) <doi:10.1007/978-1-4614-8456-1> and Kuznetsova et al. (2017) <doi:10.18637/jss.v082.i13>.
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Depends: R (>= 4.0)
Imports: lme4, dplyr, simr, magrittr, ggplot2, ggtext, quantreg,
        tibble, lmerTest, DHARMa, influence.ME, EnvStats, jsonlite,
        methods, stats, grDevices, graphics
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
NeedsCompilation: no
Packaged: 2026-06-06 15:59:21 UTC; rthomas
Author: Min-Gyoung Shin [aut],
  Reuben Thomas [aut, cre]
Maintainer: Reuben Thomas <reuben.thomas@gladstone.ucsf.edu>
Repository: CRAN
Date/Publication: 2026-06-06 16:50:02 UTC
Built: R 4.5.2; ; 2026-06-06 20:42:50 UTC; unix

Package: RMeDPower2
Type: Package
Title: Design and Modeling for Repeated Measures Studies
Version: 1.0.2
Authors@R: c(
    person("Min-Gyoung", "Shin",
           email = "mingyoung.shin@gladstone.ucsf.edu",
           role = c("aut")),
    person("Reuben", "Thomas",
           email = "reuben.thomas@gladstone.ucsf.edu",
           role = c("aut", "cre"))
    )
Description: Provides complete functionality to analyse data from repeated
    measures experiments with hierarchical or crossed experimental designs.
    Supports testing modeling assumptions, identifying outlier observations
    and experimental units, estimating statistical power, and performing
    sample size calculations. Uses linear mixed effects models via 'lme4'
    and simulation-based power analysis via 'simr'. Handles both normal and
    non-normal error distributions including binomial and Poisson families. For more details see Shin et al. (2022) <doi:10.1101/2022.07.18.500490>, Bates et al. (2015) <doi:10.18637/jss.v067.i01>, Green and MacLeod (2016) <doi:10.1111/2041-210X.12504>, Hartig (2024) <doi:10.32614/CRAN.package.DHARMa>, Nieuwenhuis et al. (2012) <doi:10.32614/RJ-2012-011>, Millard  (2013) <doi:10.1007/978-1-4614-8456-1> and Kuznetsova et al. (2017) <doi:10.18637/jss.v082.i13>.
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Depends: R (>= 4.0)
Imports: lme4, dplyr, simr, magrittr, ggplot2, ggtext, quantreg,
        tibble, lmerTest, DHARMa, influence.ME, EnvStats, jsonlite,
        methods, stats, grDevices, graphics
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
NeedsCompilation: no
Packaged: 2026-06-06 15:59:21 UTC; rthomas
Author: Min-Gyoung Shin [aut],
  Reuben Thomas [aut, cre]
Maintainer: Reuben Thomas <reuben.thomas@gladstone.ucsf.edu>
Repository: CRAN
Date/Publication: 2026-06-06 16:50:02 UTC
Built: R 4.6.0; ; 2026-06-06 21:19:11 UTC; unix

# Generated by roxygen2: do not edit by hand

export(calculatePower)
export(diagnoseDataModel)
export(getEstimatesOfInterest)
export(readDesign)
export(readPowerParams)
export(readProbabilityModel)
exportClasses(PowerParams)
exportClasses(ProbabilityModel)
exportClasses(RMeDesign)
import(dplyr)
import(ggplot2)
import(magrittr)
import(methods)
importFrom(DHARMa,outliers)
importFrom(DHARMa,simulateResiduals)
importFrom(EnvStats,rosnerTest)
importFrom(ggtext,element_textbox_simple)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)
importFrom(graphics,mtext)
importFrom(influence.ME,cooks.distance.estex)
importFrom(influence.ME,grouping.levels)
importFrom(influence.ME,se.fixef)
importFrom(jsonlite,fromJSON)
importFrom(lme4,VarCorr)
importFrom(lme4,fixef)
importFrom(lme4,getME)
importFrom(lme4,glmer)
importFrom(lme4,glmer.nb)
importFrom(lme4,ranef)
importFrom(lmerTest,lmer)
importFrom(simr,"fixef<-")
importFrom(simr,extend)
importFrom(simr,fcompare)
importFrom(simr,fixed)
importFrom(simr,makeLmer)
importFrom(simr,powerCurve)
importFrom(simr,powerSim)
importFrom(stats,Gamma)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,fitted)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,poisson)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,qqline)
importFrom(stats,qqnorm)
importFrom(stats,residuals)
importFrom(stats,update)
importFrom(stats,vcov)
importFrom(tibble,rownames_to_column)

# RMeDPower
![image](https://user-images.githubusercontent.com/18338399/186777298-189fb773-d89b-4557-85c4-b22546f566e5.png)

## Description
Biomedical research very often involves data generated from repeated measures experiments. RMeDPower2 is an R package that provides complete functionality to analyse data coming from repeated measures experiments, i.e., where one has repeated measures from the same biological/independent units or samples.

RMeDPower2 helps test the modeling assumptions one makes, identify outlier observations, outlier units at different levels of the design, estimates statistical power or perform sample size calculations, estimate parameters of interest and also to visualize the association being tested. The functionality is limited to testing associations of one predictor (continuous or categorical, e.g., disease status or brain pathology) along with one another covariate (e.g., gender status) in the context of hierarchical or crossed experimental designs.

RMeDPower2 defines the experimental design for the data, probability model of the data generating distribution and necessary parameters required for sample size calculation using convenient S4 class objects. It uses these objects in one framework that brings together the functionality implemented in multiple R packages - lme4 (implementation of linear mixed effects models), influence.ME (identification of outlier units), EnvStats EnvStats (identification of outlier observations), DHARMa (testing of modeling assumptions for non-normal distributions), simr (sample-size calculations) and tidyverse (data manipulation and visualization).

## How to install
```
install.packages("devtools")
library(devtools)
install_github('gladstone-institutes/RMeDPower2', build_vignettes=TRUE)
library(RMeDPower2)
```

## Website
Please visit https://gladstone-institutes.github.io/RMeDPower2/index.html for a detailed description of the package and tutorials.
1. [Quick Start Guide](https://gladstone-institutes.github.io/RMeDPower2/articles/getting-started.html)
2. [RMeDPower2 Class Configuration Guide](https://gladstone-institutes.github.io/RMeDPower2/articles/RMeDPower2_Class_Configuration_Guide.html)
3. [Create the objects of required classes using an **Rshiny app**](https://gladstone-institutes.github.io/RMeDPower2/articles/RMeDPower2_Class_Configuration_Guide.html#interactive-configuration-with-shiny-app)
4. [Complete tutorial with multiple use cases in biomedical research](https://gladstone-institutes.github.io/RMeDPower2/articles/Tutorial.html)

## AI Disclosure Statement

Generative AI tools (Claude Code, Anthropic) were used as coding assistants during the development of this package. The authors maintain full responsibility for the accuracy, reproducibility, and scientific validity of all code. AI-assisted outputs were reviewed and validated against expected behavior before integration. The research questions, analytical approaches, parameter selections, and scientific interpretations were determined independently by the authors without AI input.

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(warning = FALSE, message = FALSE)

## ----echo=FALSE---------------------------------------------------------------
template_dir <- system.file("input_templates/cell_assay_data",
                            package = "RMeDPower2")

## ----eval=FALSE---------------------------------------------------------------
# install.packages("devtools")
# library(devtools)
# install_github('gladstone-institutes/RMeDPower2', build_vignettes=TRUE, force = TRUE)

## ----results='hide'-----------------------------------------------------------
suppressPackageStartupMessages(library(RMeDPower2))
suppressPackageStartupMessages(library(ggplot2))
suppressPackageStartupMessages(library(dplyr))
suppressPackageStartupMessages(library(magrittr))

data <- plate_assay_pilot_data

## -----------------------------------------------------------------------------
knitr::kable(data[1:5,] )

## -----------------------------------------------------------------------------
design <- readDesign(file.path(template_dir,"design_cell_assay.json"))

## -----------------------------------------------------------------------------
model <- readProbabilityModel(file.path(template_dir,"prob_model.json"))

## -----------------------------------------------------------------------------
power_param <- readPowerParams(file.path(template_dir,"power_param.json"))

## ----eval=FALSE---------------------------------------------------------------
# diagnose_res <- diagnoseDataModel(data, design, model)
# data_update = diagnose_res$Data_updated

## ----eval=FALSE---------------------------------------------------------------
# diagnose_res_update <- diagnoseDataModel(data_update, design_update, model_update)

## ----eval=FALSE---------------------------------------------------------------
# power_res <- calculatePower(data, design, model, power_param)

## ----eval=FALSE---------------------------------------------------------------
# estimate_res <- getEstimatesOfInterest(data, design, model, print_plots = F)

## ----echo=FALSE---------------------------------------------------------------
suppressPackageStartupMessages(require(dplyr))
suppressPackageStartupMessages(library(RMeDPower2))
template_dir <- system.file("input_templates/cell_assay_data",
                            package = "RMeDPower2")

data <- plate_assay_pilot_data %>%
  select(experiment, line, classification, cell_size2) %>%
  mutate(experiment = as.factor(experiment), line = as.factor(line), classification = as.factor(classification))

str(data)

## -----------------------------------------------------------------------------
design <- new("RMeDesign")
print(design)

## ----echo=FALSE---------------------------------------------------------------
template_dir <- system.file("input_templates/cell_assay_data",
                            package = "RMeDPower2")

## -----------------------------------------------------------------------------
design <- readDesign(file.path(template_dir, "design_cell_assay.json"))
print(design)

## -----------------------------------------------------------------------------
model <- readProbabilityModel(file.path(template_dir, "prob_model.json"))
print(model)

## -----------------------------------------------------------------------------
power_param <- readPowerParams(file.path(template_dir, "power_param.json"))
print(power_param)

## ----flowchart, echo=FALSE, fig.align = "center", fig.cap="Flowchart illustrating the functionality of the package", fig.small=TRUE, echo=FALSE----
knitr::include_graphics(system.file("figures/package_structure.png", package = "RMeDPower2"))

## ----applications, echo=FALSE, fig.align = "center", fig.cap="Flowchart illustrating the functionality of the package", fig.small=TRUE, echo=FALSE----
knitr::include_graphics(system.file("figures/application_domains.png", package = "RMeDPower2"))

## -----------------------------------------------------------------------------
template_dir <- system.file("input_templates/cell_assay_data", package = "RMeDPower2")
data <- plate_assay_pilot_data
design <- readDesign(file.path(template_dir,"design_cell_assay.json"))
model <- readProbabilityModel(file.path(template_dir,"prob_model.json"))
power_param <- readPowerParams(file.path(template_dir,"power_param.json"))

## ----echo=FALSE---------------------------------------------------------------
print(ggplot(data, aes(x=experiment, y=cell_size2, color = line)) +
  geom_boxplot() +
  theme_bw() +
  theme(text = element_text(size = 16)))

print(ggplot(data, aes(x=cell_size2, color = experiment)) +
  stat_ecdf() +
  theme_bw() +
  theme(text = element_text(size = 16)))

## -----------------------------------------------------------------------------
data %<>% filter(cell_size2 < quantile(cell_size2, 0.95))

## -----------------------------------------------------------------------------
diagnose_res <- diagnoseDataModel(data, design, model)

## -----------------------------------------------------------------------------
print(diagnose_res$models)

## ----output-plots1, results='asis'--------------------------------------------
plot_no <- 1
plots <- diagnose_res$diagnostic_plots$natural_scale$plots
captions <- diagnose_res$diagnostic_plots$natural_scale$captions
for (i in seq_along(plots)) {
  print(plots[[i]])
  cat(sprintf("\n\n\n**Figure %d:** %s. %s\n\n", plot_no,names(plots)[i], captions[i]))
  plot_no <- plot_no + 1
}

cat("\n\n\n")
cooks_plots <- diagnose_res$cooks_plots$natural_scale
for(i in 1:length(cooks_plots)) {
  print(cooks_plots[[i]])
  cat(sprintf("\n\n\n**Figure %d:**\n\n", plot_no))
  plot_no <- plot_no + 1
}

print("Inferred outlier groups are")
print(diagnose_res$inferred_outlier_groups$natural_scale)

## ----output-plots2, results='asis'--------------------------------------------
plots <- diagnose_res$diagnostic_plots$log_scale$plots
captions <- diagnose_res$diagnostic_plots$log_scale$captions
for (i in seq_along(plots)) {
  print(plots[[i]])
  cat(sprintf("\n\n\n**Figure %d:** %s. %s\n\n", plot_no,names(plots)[i], captions[i]))
  plot_no <- plot_no + 1
}


cooks_plots <- diagnose_res$cooks_plots$log_scale

cat("\n\n\n")
for(i in 1:length(cooks_plots)) {
  print(cooks_plots[[i]])
  cat(sprintf("\n\n\n**Figure %d:**\n\n", plot_no))
  plot_no <- plot_no + 1
}

print("Inferred outlier groups are")
print(diagnose_res$inferred_outlier_groups$natural_scale)

## -----------------------------------------------------------------------------
design_update <- design
data_update <- diagnose_res$Data_updated
print(colnames(data_update))
design_update@response_column <-  "cell_size2_logTransformed"
estimate_res <- getEstimatesOfInterest(data_update, design_update, model,print_plots = FALSE)

## ----results="asis", echo = FALSE---------------------------------------------
print(estimate_res[[2]]$plots)
knitr::asis_output(sprintf("\n\n\n**Figure %d:**\n\n", plot_no))
plot_no <- plot_no + 1


## -----------------------------------------------------------------------------
##print model summary output
print(estimate_res[[1]])

## -----------------------------------------------------------------------------
power_param@max_size <- 15
power_res <- calculatePower(data_update, design_update, model, power_param)

## ----results="asis", echo = FALSE---------------------------------------------
print(power_res$plots)
knitr::asis_output(sprintf("\n\n\n**Figure %d:**\n\n", plot_no))
plot_no <- plot_no + 1

## -----------------------------------------------------------------------------
full_data <- plate_assay_full_data

##filter the top 5% of the observations
full_data %<>% filter(perim_2th_effect < quantile(perim_2th_effect, 0.95))

design@response_column <- "perim_2th_effect"
design@condition_column <- "classif"
full_diag_res <- diagnoseDataModel(full_data, design, model)
full_data_update <- full_diag_res$Data_updated
full_design <- design
full_design@response_column <- "perim_2th_effect_logTransformed"

## ----results='asis'-----------------------------------------------------------
cooks_plots <- full_diag_res$cooks_plots$log_scale
cat("\n\n\n")
for(i in 1:length(cooks_plots)) {
  print(cooks_plots[[i]])
  cat(sprintf("\n\n\n**Figure %d:**\n\n", plot_no))
  plot_no <- plot_no + 1
}

print("Inferred outlier groups are")
print(diagnose_res$inferred_outlier_groups$natural_scale)


## -----------------------------------------------------------------------------
full_res <- getEstimatesOfInterest(full_data_update, full_design, model, print_plots = FALSE)
print(full_res[[1]])

## ----results="asis", echo = FALSE---------------------------------------------
print(full_res[[2]]$plots)
knitr::asis_output(sprintf("\n\n\n**Figure %d:**\n\n", plot_no))
plot_no <- plot_no + 1
cat("\n\n\n")

## ----sessionInfo, echo=FALSE--------------------------------------------------
sessionInfo()

#' @title get_residuals_covariate
#'
#' @description This function retrieve residual values from an lmerFit summary object and plot residual values by condition_column
#'
#' Note: The current version does not accept categorical response variables, sample size parameters smaller than the observed samples size
#'
#'
#' @param data Input data
#' @param condition_column Name of the condition variable (ex variable with values such as control/case). The input file has to have a corresponding column name
#' @param experimental_columns Name of variables related to experimental design such as "experiment", "plate", and "cell_line". They should be in order, for example, "experiment" should always come first .
#' @param response_column Name of the variable observed by performing the experiment. ex) intensity.
#' @param total_column Set this column only when family_p="binomial" and it is equal to the total number of observations (number of cases plus number of controls) for a given number of cases
#' @param condition_is_categorical Specify whether the condition variable is categorical. TRUE: Categorical, FALSE: Continuous.
#' @param covariate The name of the covariate to control in the regression model
#' @param crossed_columns Name of experimental variables that may appear repeatedly with the same ID. For example, cell_line C1 may appear in multiple experiments, but plate P1 cannot appear in more than one experiment
#' @param error_is_non_normal Default: the observed variable is continuous Categorical response variable will be implemented in the future. TRUE: Categorical , FALSE: Continuous (default).
#' @param family_p The type of distribution family to specify when the response is categorical. If family is "binary" then binary(link="log") is used, if family is "poisson" then poisson(link="logit") is used, if family is "poisson_log" then poisson(link=") log") is used.
#' @param na.action "complete": missing data is not allowed in all columns (default), "unique": missing data is not allowed only in condition, experimental, and response columns. Selecting "complete" removes an entire row when there is one or more missing values, which may affect the distribution of other features.
#' @param include_interaction Whether to include condition * covariate interaction
#' @param random_slope_variable Variable for random slopes (typically one of "condition_column" or "covariate" and assuming that they are numeric variables). A random slope term is added for each of the variables specified in the experimental columns in addition to their corresponding random intercept terms. The random slope and intercept terms for each experimental_columns variable are assumed to be uncorrelated.
#' @param covariate_is_categorical Specify whether the covariate variable is categorical. TRUE: Categorical, FALSE: Continuous.
#' @param print_plots Whether or not to print the plots, irrespective of this argument ggplot versions of evaluated association between the response_column and the condition_column. TRUE - print the plot, FALSE - do not print the plot
#'
#' @return A list with 3 elements. 1. The updated input data with an additional column with the model residuals of the individual observations. 2. A plot representing the purported association between the response column and the condition column. 3. The corresponding caption for this figure.
#'
#' @keywords internal
#' @noRd


get_residuals <- function(data, condition_column, experimental_columns, response_column,  condition_is_categorical, covariate=NULL,
                          crossed_columns=NULL, total_column=FALSE, error_is_non_normal=FALSE, family_p=NULL, na.action="complete", include_interaction=FALSE, random_slope_variable=NULL, covariate_is_categorical = TRUE, print_plots = TRUE){



  ######input error handler
  if(!is.null(covariate) )
    if(!covariate%in%colnames(data))
      stop("covariate should be null or one of the column names")
  if(!condition_column%in%colnames(data)) stop("condition_column should be one of the column names")
  if(sum(experimental_columns%in%colnames(data))!=length(experimental_columns)) stop("experimental_columns must match column names")
  if(!response_column%in%colnames(data)) stop("response_column should be one of the column names")
  if(!is.na(condition_is_categorical) && !condition_is_categorical%in%c(TRUE,FALSE)) stop("condition_is_categorical must be TRUE or FALSE")
  if(!is.null(crossed_columns)){if(sum(crossed_columns%in%colnames(data))!=length(crossed_columns)) stop("crossed_columns must match column names")}
  if(!is.na(covariate_is_categorical) && !covariate_is_categorical%in%c(TRUE,FALSE)) stop("covariate_is_categorical must be TRUE or FALSE")

  # Validation for new parameters
  if(!is.na(include_interaction)) {
    if (include_interaction && is.null(covariate)) {
      stop("Cannot include interaction when covariate is NULL")
    }
  }

  if (!is.null(random_slope_variable) &&
      !random_slope_variable %in% c("condition_column", condition_column, "covariate", covariate)) {
    stop("random_slope_variable should be 'condition_column', 'covariate' or the actual condition column name or covariate column name")
  }

  if(error_is_non_normal==TRUE){
    if(family_p != "negative_binomial")
      family_p=switch(family_p, "poisson" = poisson(link="log"), "binomial" = binomial(link="logit"), "bionomial_log" = binomial(link="log") )
    else
      family_p = list(family = "negative_binomial")
  }


  if(na.action=="complete"){

    notNAindex=which( rowSums(is.na(data)) == 0 )

  }else if(na.action=="unique"){

    if(is.null(covariate)) notNAindex=which( rowSums(is.na(data[,c(condition_column, experimental_columns, response_column, covariate)])) == 0 )
    else notNAindex=which( rowSums(is.na(data[,c(condition_column, experimental_columns, response_column)])) == 0 )

  }

  Data=data[notNAindex,]




  # cat("\n")
  # print("__________________________________________________________________Summary of data:")
  # print(summary(Data))
  # cat("\n")

  colnames_original=colnames(Data)
  experimental_columns_index=NULL
  ####### assign categorical variables
  if(condition_is_categorical==TRUE)
    Data[,condition_column]=as.factor(Data[,condition_column])
  else
    Data[,condition_column]=as.numeric(Data[,condition_column])

  if(!is.null(covariate)) {
    if(covariate_is_categorical==TRUE)
      Data[,covariate]=as.factor(Data[,covariate])
    else
      Data[,covariate]=as.numeric(Data[,covariate])
  }

  # random slope should be allowed only with a continuous variable
  if(!is.null(random_slope_variable)) {
    if(!inherits(Data[,random_slope_variable], "numeric")) {
      stop("random_slope_variable should be a numeric variable")
    }
  }

  # cat("\n")


  noncrossed_columns=NULL

  for(i in 1:length(experimental_columns)){
    Data[,experimental_columns[i]]=as.factor(Data[,experimental_columns[i]])
    experimental_columns_index=c(experimental_columns_index,which(colnames(Data)==experimental_columns[i]))
    colnames(Data)[experimental_columns_index[i]]=paste("experimental_column",i,sep="")

    if(i!=1&&!experimental_columns[i]%in%crossed_columns){
      noncrossed_columns=c(noncrossed_columns, paste("experimental_column",i,sep=""))
    }


    # cat("\n")
    # print(paste("_________________________________",experimental_columns[i]," is assigned to experimental_column",i,sep=""))
    # cat("\n")
  }


  if(length(experimental_columns)>=2){
    for(r in 2:length(experimental_columns)){
      if(colnames(Data)[experimental_columns_index[r]]%in%noncrossed_columns){
        Data[,experimental_columns_index[r]]=paste(Data[,experimental_columns_index[r-1]], Data[,experimental_columns_index[r]],sep="_")
      }
    }
  }




  colnames(Data)[which(colnames(Data)==condition_column)]="condition_column"
  colnames(Data)[which(colnames(Data)==response_column)]="response_column"
  if(!is.null(covariate)) colnames(Data)[which(colnames(Data)==covariate)]="covariate"
  if(!is.null(random_slope_variable)) {
    if(random_slope_variable == condition_column)
      random_slope_variable = "condition_column"
    else if(random_slope_variable == covariate)
      random_slope_variable = "covariate"
    else{
      stop("random_slope_variable can only be 'condition_column', 'covariate' or the actual condition column name or covariate column name")
    }
  }

  if(!is.null(total_column))
    colnames(Data)[which(colnames(Data)==total_column)]="total_column"

  # Build formula components
  random_formula <- build_random_formula0(experimental_columns)
  lmerFit <- generate_model_fit0(data=Data,
                                random_formula,
                                error_is_non_normal,
                                family_p,
                                total_column)

  # ####### run the formula
  #
  # if(is.null(covariate)){
  #   if(error_is_non_normal==FALSE){
  #     if(length(experimental_columns)==1){
  #       lmerFit <- lmerTest::lmer(response_column ~  (1 | experimental_column1), data=Data)
  #     }else if(length(experimental_columns)==2){
  #       lmerFit <- lmerTest::lmer(response_column ~  (1 | experimental_column1) + (1 | experimental_column2), data=Data)
  #     }else if(length(experimental_columns)==3){
  #       lmerFit <- lmerTest::lmer(response_column ~  (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3), data=Data)
  #     }else if(length(experimental_columns)==4){
  #       lmerFit <- lmerTest::lmer(response_column ~  (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4), data=Data)
  #     }else if(length(experimental_columns)==5){
  #       lmerFit <- lmerTest::lmer(response_column ~  (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4) + (1 | experimental_column5), data=Data)
  #     }
  #   }else if(family_p$family == "binomial" & !is.null(total_column)){
  #     if(length(experimental_columns)==1){
  #       lmerFit <- lme4::glmer(cbind(response_column, (total_column - response_column)) ~ (1 | experimental_column1), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==2){
  #       lmerFit <- lme4::glmer(cbind(response_column, total_column - response_column) ~ (1 | experimental_column1) + (1 | experimental_column2), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==3){
  #       lmerFit <- lme4::glmer(cbind(response_column, total_column - response_column) ~ (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==4){
  #       lmerFit <- lme4::glmer(cbind(response_column, total_column - response_column) ~ (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==5){
  #       lmerFit <- lme4::glmer(cbind(response_column, total_column - response_column) ~ (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4) + (1 | experimental_column5), data=Data, family=family_p)
  #     }
  #   }else if(family_p$family == "negative_binomial" & !is.null(total_column)){
  #     if(length(experimental_columns)==1){
  #       lmerFit <- lme4::glmer.nb(response_column ~  (1 | experimental_column1) + offset(log(total_column)), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==2){
  #       lmerFit <- lme4::glmer.nb(response_column ~  (1 | experimental_column1) + (1 | experimental_column2) + offset(log(total_column)), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==3){
  #       lmerFit <- lme4::glmer.nb(response_column ~  (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + offset(log(total_column)) , data=Data, family=family_p)
  #     }else if(length(experimental_columns)==4){
  #       lmerFit <- lme4::glmer.nb(response_column ~  (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4) + offset(log(total_column)), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==5){
  #       lmerFit <- lme4::glmer.nb(response_column ~ condition_column + (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4) + (1 | experimental_column5) + offset(log(total_column)) , data=Data, family=family_p)
  #     }
  #   }else if(family_p$family == "poisson" & !is.null(total_column)){
  #     if(length(experimental_columns)==1){
  #       lmerFit <- lme4::glmer(response_column ~ (1 | experimental_column1) + offset(log(total_column)), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==2){
  #       lmerFit <- lme4::glmer(response_column ~ (1 | experimental_column1) + (1 | experimental_column2) + offset(log(total_column)), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==3){
  #       lmerFit <- lme4::glmer(response_column ~ (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + offset(log(total_column)) , data=Data, family=family_p)
  #     }else if(length(experimental_columns)==4){
  #       lmerFit <- lme4::glmer(response_column ~ (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4) + offset(log(total_column)), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==5){
  #       lmerFit <- lme4::glmer(response_column ~ (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4) + (1 | experimental_column5) + offset(log(total_column)) , data=Data, family=family_p)
  #     }
  #   }else{
  #     if(length(experimental_columns)==1){
  #       lmerFit <- lme4::glmer(response_column ~  (1 | experimental_column1), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==2){
  #       lmerFit <- lme4::glmer(response_column ~  (1 | experimental_column1) + (1 | experimental_column2), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==3){
  #       lmerFit <- lme4::glmer(response_column ~  (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==4){
  #       lmerFit <- lme4::glmer(response_column ~  (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==5){
  #       lmerFit <- lme4::glmer(response_column ~  (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4) + (1 | experimental_column5), data=Data, family=family_p)
  #     }
  #   }
  # }else{
  #   if(error_is_non_normal==FALSE){
  #     if(length(experimental_columns)==1){
  #       lmerFit <- lmerTest::lmer(response_column ~  covariate + (1 | experimental_column1), data=Data)
  #     }else if(length(experimental_columns)==2){
  #       lmerFit <- lmerTest::lmer(response_column ~  covariate + (1 | experimental_column1) + (1 | experimental_column2), data=Data)
  #     }else if(length(experimental_columns)==3){
  #       lmerFit <- lmerTest::lmer(response_column ~  covariate + (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3), data=Data)
  #     }else if(length(experimental_columns)==4){
  #       lmerFit <- lmerTest::lmer(response_column ~  covariate + (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4), data=Data)
  #     }else if(length(experimental_columns)==5){
  #       lmerFit <- lmerTest::lmer(response_column ~  covariate + (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4) + (1 | experimental_column5), data=Data)
  #     }
  #   }else if(family_p$family == "binomial" & !is.null(total_column)){
  #     if(length(experimental_columns)==1){
  #       lmerFit <- lme4::glmer(cbind(response_column, (total_column - response_column)) ~ covariate + (1 | experimental_column1), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==2){
  #       lmerFit <- lme4::glmer(cbind(response_column, total_column - response_column) ~ covariate + (1 | experimental_column1) + (1 | experimental_column2), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==3){
  #       lmerFit <- lme4::glmer(cbind(response_column, total_column - response_column) ~ covariate + (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==4){
  #       lmerFit <- lme4::glmer(cbind(response_column, total_column - response_column) ~ covariate + (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==5){
  #       lmerFit <- lme4::glmer(cbind(response_column, total_column - response_column) ~ covariate + (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4) + (1 | experimental_column5), data=Data, family=family_p)
  #     }
  #   }else if(family_p$family == "negative_binomial" & !is.null(total_column)){
  #     if(length(experimental_columns)==1){
  #       lmerFit <- lme4::glmer.nb(response_column ~ covariate + (1 | experimental_column1) + offset(log(total_column)), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==2){
  #       lmerFit <- lme4::glmer.nb(response_column ~ covariate + (1 | experimental_column1) + (1 | experimental_column2) + offset(log(total_column)), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==3){
  #       lmerFit <- lme4::glmer.nb(response_column ~ covariate + (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + offset(log(total_column)) , data=Data, family=family_p)
  #     }else if(length(experimental_columns)==4){
  #       lmerFit <- lme4::glmer.nb(response_column ~ covariate + (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4) + offset(log(total_column)), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==5){
  #       lmerFit <- lme4::glmer.nb(response_column ~ covariate + (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4) + (1 | experimental_column5) + offset(log(total_column)) , data=Data, family=family_p)
  #     }
  #   }else if(family_p$family == "poisson" & !is.null(total_column)){
  #     if(length(experimental_columns)==1){
  #       lmerFit <- lme4::glmer(response_column ~ covariate + (1 | experimental_column1) + offset(log(total_column)), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==2){
  #       lmerFit <- lme4::glmer(response_column ~ covariate + (1 | experimental_column1) + (1 | experimental_column2) + offset(log(total_column)), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==3){
  #       lmerFit <- lme4::glmer(response_column ~ covariate + (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + offset(log(total_column)) , data=Data, family=family_p)
  #     }else if(length(experimental_columns)==4){
  #       lmerFit <- lme4::glmer(response_column ~ covariate + (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4) + offset(log(total_column)), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==5){
  #       lmerFit <- lme4::glmer(response_column ~ covariate + (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4) + (1 | experimental_column5) + offset(log(total_column)) , data=Data, family=family_p)
  #     }
  #   }else{
  #     if(length(experimental_columns)==1){
  #       lmerFit <- lme4::glmer(response_column ~  covariate + (1 | experimental_column1), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==2){
  #       lmerFit <- lme4::glmer(response_column ~  covariate + (1 | experimental_column1) + (1 | experimental_column2), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==3){
  #       lmerFit <- lme4::glmer(response_column ~  covariate + (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==4){
  #       lmerFit <- lme4::glmer(response_column ~  covariate + (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4), data=Data, family=family_p)
  #     }else if(length(experimental_columns)==5){
  #       lmerFit <- lme4::glmer(response_column ~  covariate + (1 | experimental_column1) + (1 | experimental_column2) + (1 | experimental_column3) + (1 | experimental_column4) + (1 | experimental_column5), data=Data, family=family_p)
  #     }
  #   }
  # }







  slmerFit <- summary(lmerFit)
  # cat("\n")
  # print("__________________________________________________________________Model statistics:")
  # print(slmerFit)
  # cat("\n")

  residuals=slmerFit$residuals


  Data = cbind(residuals, Data)


  colnames(Data)[1] = "residual"
  #Data[,"condition_column"]=as.numeric(as.character(Data[,"condition_column"]))

  if(is.null(covariate)) {
    Data_sum <- Data %>%
      dplyr::group_by(condition_column, experimental_column1) %>%
      dplyr::summarise(mean_residual1 = mean(residual), med_residual1 = median(residual))
  }
  if(!is.null(covariate)){
    Data_sum <- Data %>%
      dplyr::group_by(covariate, condition_column, experimental_column1) %>%
      dplyr::summarise(mean_residual1 = mean(residual), med_residual1 = median(residual))
  }


  condition_var <- condition_column
  response_var <- response_column
  covariate_var <- covariate

  if(condition_is_categorical==TRUE){

    if(!is.null(covariate)) {
       if(inherits(Data[["covariate"]], "factor") | inherits(Data[["covariate"]], "character")) {
        gp=ggplot2::ggplot(Data_sum, aes(x=condition_column, y=med_residual1, color = covariate)) +
          geom_boxplot(position = position_dodge(width = 0.7), outlier.shape = NA) +
          geom_point(position = position_jitterdodge(jitter.width = 0.1,dodge.width = 0.7)) +
          labs(title = "Association of interest",
               subtitle = paste0("Box plot of the median residuals across all observations within each level of experimental column = ", experimental_columns[1], " as a function of ", condition_column, " and separated by levels of ", covariate),
               y = paste(response_var, ": median residual"),
               x = condition_var) +
          theme_bw() +
          theme(axis.title   = element_text(face  = "bold"))+
          theme(plot.subtitle  = element_textbox_simple())
        captions = paste0("Box plot of the median residuals across all observations within each level of experimental column = ", experimental_columns[1], " as a function of ", condition_column, " and separated by levels of ", covariate)
      }
      else if(inherits(Data[["covariate"]], "numeric")) {
        gp=ggplot2::ggplot(Data_sum, aes(x=covariate, y=med_residual1, color = condition_column)) +
          geom_smooth(method = "lm", se = T) +
          geom_point() +
          labs(title = "Association of interest",
               subtitle = paste0("Best linear fits of the median residuals across all observations within each level of experimental column = ", experimental_columns[1], " as a function of ", covariate, " for each level of ", condition_column),
               y = paste(response_var, ": median residual"),
               x = covariate_var) +
          theme_minimal() +
          theme(plot.title = element_text(size = 12, face = "bold")) +
          theme(plot.subtitle  = element_textbox_simple())
        captions = paste0("Best linear fits of the median residuals across all observations within each level of experimental column = ", experimental_columns[1], " as a function of ", covariate, " for each level of ", condition_column)

      }
    }else{
      gp=ggplot2::ggplot(Data_sum, aes(x=condition_column, y=med_residual1)) +
        geom_boxplot(position = position_dodge(width = 0.5), outlier.shape = NA) +
        geom_point() +
        labs(title = "Association of interest",
             subtitle = paste0("Boxplot of the median residuals across all observations within each level of experimental column = ", experimental_columns[1], " as a function of ", condition_column),
             y = paste(response_var, ": median residual"),
             x = condition_var) +
        theme_minimal() +
        theme(plot.title = element_text(size = 12, face = "bold")) +
        theme(plot.subtitle  = element_textbox_simple())
      captions = paste0("Boxplot of the median residuals across all observations within each level of experimental column = ", experimental_columns[1], " as a function of ", condition_column)

    }

  }else{


    # plot(Data[,"condition_column"], Data[,"residual"], xlab=condition_column, ylab=paste0(response_column, " Residual Value"), main=NULL)
    # abline(lm( as.formula( paste0( "residual ~  condition_column") ), data=Data),col='blue')

    gp <- ggplot2::ggplot(Data, aes(x=condition_column, y=residual, color=experimental_column1)) +
      geom_point() +
      geom_smooth(method = "lm", se = T) +
      labs(title = "Association of interest",
           subtitle = paste0("Best linear fits of the residuals across all observations separated for each level of experimental column = ", experimental_columns[1], " as a function of ", condition_column),
           y = paste(response_var, ": residual"),
           x = condition_var) +
      theme_minimal() +
      theme(plot.title = element_text(size = 12, face = "bold")) +
      theme(plot.subtitle  = element_textbox_simple())

    captions = paste0("Best linear fits of the residuals across all observations separated for each level of experimental column = ", experimental_columns[1], " as a function of ", condition_column)



  }

  if(print_plots)
    print(gp)

  return(list(Data = Data, plots = gp, captions = captions))
}

#' @title RMeDesign-class
#'
#' @description Objects of RMeDesign class store information on the relevant repeated measures design for the given data
#'
#'
#' @param data Input data
#' @param condition_column Name of the condition variable (ex variable with values such as control/case). The input file has to have a corresponding column name
#' @param experimental_columns Name of the variable related to experimental design such as "experiment", "plate", and "cell_line". They should be in order, for example, "experiment" should always come first .
#' @param response_column Name of the variable observed by performing the experiment. ex) intensity.
#' @param total_column Set this column only when family_p="binomial" and it is equal to the total number of observations (number of cases plus number of controls) for a given number of cases
#' @param outlier_alpha numeric scalar between 0 and 1 indicating the Type I error associated with the test of outliers
#' @param condition_is_categorical Specify whether the condition variable is categorical. TRUE: Categorical, FALSE: Continuous.
#' @param covariate The name of the covariate to control in the regression model
#' @param method The method used to detect outliers. "rosner" (default) runs Rosner's test and "cook" runs Cook's distance.
#' @param crossed_columns Name of experimental variables that may appear repeatedly with the same ID. For example, cell_line C1 may appear in multiple experiments, but plate P1 cannot appear in more than one experiment
#' @param error_is_non_normal Default: the observed variable is continuous Categorical response variable will be implemented in the future. TRUE: Categorical , FALSE: Continuous (default).
#' @param family_p The type of distribution family to specify when the response is categorical. If family is "binary" then binary(link="log") is used, if family is "poisson" then poisson(link="logit") is used, if family is "poisson_log" then poisson(link=") log") is used.
#' @param na.action "complete": missing data is not allowed in all columns (default), "unique": missing data is not allowed only in condition, experimental, and response columns. Selecting "complete" removes an entire row when there is one or more missing values, which may affect the distribution of other features.
#' @param include_interaction logical - TRUE or FALSE - Whether to include condition * covariate interaction
#' @param random_slope_variable Variable for random slopes (typically one of "condition_column" or "covariate" and assuming that they are numeric variables). A random slope term is added for each of the variables specified in the experimental columns in addition to their corresponding random intercept terms. The random slope and intercept terms for each experimental_columns variable are assumed to be uncorrelated.
#' @param covariate_is_categorical Specify whether the covariate variable is categorical. TRUE: Categorical, FALSE: Continuous.
#'
#' @return an object of class RMeDesign
#'
#' @export
#'
#' @examples design=new("RMeDesign")

setClass("RMeDesign",
         slots = list(
           response_column = "character",
           covariate = "ANY",
           condition_column = "character",
           condition_is_categorical = "logical",
           experimental_columns = "character",
           crossed_columns = "ANY",
           total_column = "ANY",
           outlier_alpha = "numeric",
           na_action = "character",
           # NEW SLOTS FOR ENHANCED FUNCTIONALITY
           include_interaction = "logical",    # Whether to include condition * covariate interaction
           random_slope_variable = "ANY", # Variable for random slopes (typically condition_column)
           covariate_is_categorical = "logical" # Specify whether the covariate variable is categorical. TRUE: Categorical, FALSE: Continuous.
         ),
         prototype = list(
           response_column = "response_column",
           covariate = NULL,
           condition_column = "condition_column",
           condition_is_categorical = TRUE,
           experimental_columns = "experimental_column",
           crossed_columns = NULL,
           total_column = NULL,
           outlier_alpha = 0.05,
           na_action = "complete",
           # NEW DEFAULTS
           include_interaction = NA,        # Default: no interaction
           random_slope_variable = NULL,        # Default: no random slopes
           covariate_is_categorical = NA
         ))

#' @title ProbabilityModel-class
#'
#' @description Objects of ProbabilityModel class store information on the assumed probability distribution for the model
#'
#'
#' @param error_is_non_normal Default: the observed variable is continuous Categorical response variable will be implemented in the future. TRUE: Categorical , FALSE: Continuous (default).
#' @param family_p The type of distribution family to specify when the response is categorical. If family is "binary" then binary(link="log") is used, if family is "poisson" then poisson(link="logit") is used, if family is "poisson_log" then poisson(link=") log") is used.
#'
#' @return an object of class ProbabilityModel
#'
#' @export
#'
#' @examples model=new("ProbabilityModel")


setClass("ProbabilityModel",
         slots = list(
           error_is_non_normal = "logical",
           family_p = "ANY"
         ),
         prototype = list(
           error_is_non_normal = FALSE,
           family_p = NULL
         ))

#' @title PowerParams-class
#'
#' @description Objects of PowerParams class store information required for sample size estimation for given data
#'
#'
#' @param power_curve 1: Power simulation over a range of sample sizes or levels. 0: Power calculation over a single sample size or a level.
#' @param nsimn The number of simulations to run. Default=1000
#' @param target_columns Name of the experimental parameters to use for the power calculation.
#' @param levels 1: Amplify the number of corresponding target parameter. 0: Amplify the number of samples from the corresponding target parameter, ex) If target_columns = c("experiment","cell_line") and if you want to expand the number of experiment and sample more cells from each cell line, set levels = c(1,0).
#' @param max_size Maximum levels or sample sizes to test. Default: the current level or the current sample size x 5. ex) If max_levels = c(10,5), it will test upto 10 experiments and 5 cell lines.
#' @param breaks Levels /sample sizes of the variable to be specified along the power curve. Default: max(1, round( the number of current levels / 5 ))
#' @param  effect_size If you know the effect size of your condition variable, the effect size can be provided as a parameter. If the effect size is not provided, it will be estimated from your data
#' @param  alpha Threshold for Type I error
#' @param  ICC Intra-Class Coefficients (ICC) for each parameter
#'
#' @return an object of class ProbabilityModel
#'
#' @export
#'
#' @examples power_param=new("PowerParams")

setClass("PowerParams",
         slots = list(
           target_columns = "character",
           power_curve = "numeric",
           nsimn = "numeric",
           levels = "numeric",
           max_size = "ANY",
           alpha = "numeric",
           breaks = "ANY",
           effect_size = "ANY",
           icc = "ANY"
         ),
         prototype = list(
           target_columns = "experimental_column",
           power_curve = 1,
           nsimn = 20,
           levels = 1,
           max_size = NULL,
           alpha = 0.05,
           breaks = NULL,
           effect_size = NULL,
           icc = NULL
         ))

#' @title readProbabilityModel
#'
#' @description This functions reads the underlying design for the data
#'
#'
#' @param jsonfile the jsonfile with the necessary parameters for probability model: error_is_non_normal, family_p
#'
#' @return an object of class ProbabilityModel
#' @export
#'
#' @examples
#' template_dir <- system.file("input_templates/cell_assay_data", package = "RMeDPower2")
#' model <- readProbabilityModel(file.path(template_dir,"prob_model.json"))

readProbabilityModel <- function(jsonfile) {
  Prob_Model_data <- jsonlite::fromJSON(jsonfile)

  prob_model <- new("ProbabilityModel")

  prob_model@error_is_non_normal = Prob_Model_data$error_is_non_normal
  if(!is.null(Prob_Model_data$family_p)) prob_model@family_p = Prob_Model_data$family_p

  return(prob_model)
}

# Tests for utility functions in RMeDPower2

test_that("readDesign function works correctly", {
  temp_file <- tempfile(fileext = ".json")

  test_json <- list(
    response_column = "outcome",
    condition_column = "treatment",
    condition_is_categorical = TRUE,
    experimental_columns = c("batch", "subject"),
    covariate = "age",
    covariate_is_categorical = FALSE,
    crossed_columns = NULL,
    include_interaction = FALSE,
    random_slope_variable = NULL,
    total_column = NULL,
    na_action = "complete"
  )

  jsonlite::write_json(test_json, temp_file, auto_unbox = TRUE)

  design <- readDesign(temp_file)

  expect_s4_class(design, "RMeDesign")
  expect_equal(design@response_column, "outcome")
  expect_equal(design@condition_column, "treatment")
  expect_true(design@condition_is_categorical)
  expect_equal(design@experimental_columns, c("batch", "subject"))
  expect_equal(design@covariate, "age")
  expect_false(design@covariate_is_categorical)

  unlink(temp_file)
})

test_that("readProbabilityModel function works correctly", {
  temp_file <- tempfile(fileext = ".json")

  test_json <- list(error_is_non_normal = FALSE, family_p = NULL)
  write(jsonlite::toJSON(test_json, pretty = TRUE, auto_unbox = TRUE, null = "null"), file = temp_file)

  model <- readProbabilityModel(temp_file)
  expect_s4_class(model, "ProbabilityModel")
  expect_false(model@error_is_non_normal)
  expect_null(model@family_p)

  test_json2 <- list(error_is_non_normal = TRUE, family_p = "poisson")
  write(jsonlite::toJSON(test_json2, pretty = TRUE, auto_unbox = TRUE, null = "null"), file = temp_file)

  model2 <- readProbabilityModel(temp_file)
  expect_true(model2@error_is_non_normal)
  expect_equal(model2@family_p, "poisson")

  unlink(temp_file)
})

test_that("readPowerParams function works correctly", {
  temp_file <- tempfile(fileext = ".json")

  test_json <- list(
    target_columns = "experiment",
    levels = 1,
    power_curve = 1,
    nsimn = 1000,
    alpha = 0.05,
    max_size = NULL,
    breaks = NULL,
    effect_size = NULL,
    icc = NULL
  )

  write(jsonlite::toJSON(test_json, pretty = TRUE, auto_unbox = TRUE, null = "null"), file = temp_file)

  power_params <- readPowerParams(temp_file)
  expect_s4_class(power_params, "PowerParams")
  expect_equal(power_params@target_columns, "experiment")
  expect_equal(power_params@levels, 1)
  expect_equal(power_params@power_curve, 1)
  expect_equal(power_params@nsimn, 1000)
  expect_equal(power_params@alpha, 0.05)

  test_json2 <- list(
    target_columns = c("experiment", "plate"),
    levels = c(1, 0),
    power_curve = 1,
    nsimn = 500,
    alpha = 0.01,
    max_size = c(10, 20),
    breaks = c(5, 10, 15),
    effect_size = 0.8,
    icc = c(0.1, 0.05)
  )

  write(jsonlite::toJSON(test_json2, pretty = TRUE, auto_unbox = TRUE, null = "null"), file = temp_file)

  power_params2 <- readPowerParams(temp_file)
  expect_equal(power_params2@target_columns, c("experiment", "plate"))
  expect_equal(power_params2@levels, c(1, 0))
  expect_equal(power_params2@max_size, c(10, 20))
  expect_equal(power_params2@effect_size, 0.8)

  unlink(temp_file)
})

test_that("getEstimatesOfInterest returns correct structure", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  model <- create_test_model("normal")

  result <- suppressWarnings(suppressMessages(
    getEstimatesOfInterest(data = test_data, design = design, model = model, print_plots = FALSE)
  ))

  expect_type(result, "list")
  expect_length(result, 2)
})

test_that("getEstimatesOfInterest handles covariates", {
  test_data <- create_test_data()
  design <- create_test_design("with_covariate")
  model <- create_test_model("normal")

  expect_no_error(suppressWarnings(suppressMessages(
    getEstimatesOfInterest(data = test_data, design = design, model = model, print_plots = FALSE)
  )))
})

test_that("getEstimatesOfInterest handles interactions", {
  test_data <- create_test_data()
  design <- create_test_design("with_covariate")
  design@include_interaction <- TRUE
  model <- create_test_model("normal")

  expect_no_error(suppressWarnings(suppressMessages(
    getEstimatesOfInterest(data = test_data, design = design, model = model, print_plots = FALSE)
  )))
})

test_that("diagnoseDataModel works correctly", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  model <- create_test_model("normal")

  expect_no_error(suppressWarnings(suppressMessages(
    diagnoseDataModel(data = test_data, design = design, model = model)
  )))
})

test_that("diagnoseDataModel handles different model types", {
  test_data <- create_count_data()
  design <- create_test_design("count_data")
  model <- create_test_model("poisson")

  expect_no_error(suppressWarnings(suppressMessages(
    diagnoseDataModel(data = test_data, design = design, model = model)
  )))
})

test_that("JSON reading functions handle missing files gracefully", {
  non_existent_file <- "/path/that/does/not/exist.json"

  expect_error(readDesign(non_existent_file))
  expect_error(readProbabilityModel(non_existent_file))
  expect_error(readPowerParams(non_existent_file))
})

test_that("JSON reading functions validate content", {
  temp_file <- tempfile(fileext = ".json")
  writeLines("{invalid json content", temp_file)

  expect_error(readDesign(temp_file))
  expect_error(readProbabilityModel(temp_file))
  expect_error(readPowerParams(temp_file))

  unlink(temp_file)
})

##' Pilot plate assay data
##'
##' Pilot dataset from plate-based assays used in the RMeDPower2
##' documentation and examples. The data represent repeated
##' measurements across plates and experimental units and are
##' intended for illustrating experimental design specification,
##' model diagnostics, and power calculations.
##'
##' @format A data frame with observations from a pilot plate
##'   assay. Column names and structure are documented in the
##'   package vignette and example code.
##'
##' @seealso \code{\link{plate_assay_pilot_data_wo_repeats}},
##'   \code{\link{plate_assay_full_data}}
##' @docType data
##' @keywords datasets
##' @name plate_assay_pilot_data
##' @usage data(plate_assay_pilot_data)
NULL

##' Pilot plate assay data without repeated measurements
##'
##' Version of \code{\link{plate_assay_pilot_data}} where repeated
##' measurements have been removed, suitable for power analyses
##' that assume a single observation per experimental unit at each
##' time point or condition.
##'
##' @format A data frame containing the pilot plate assay data
##'   without repeated measurements. See the vignette for details
##'   on columns and preprocessing.
##'
##' @seealso \code{\link{plate_assay_pilot_data}},
##'   \code{\link{plate_assay_full_data}}
##' @docType data
##' @keywords datasets
##' @name plate_assay_pilot_data_wo_repeats
##' @usage data(plate_assay_pilot_data_wo_repeats)
NULL

##' Full plate assay dataset
##'
##' Full plate assay dataset corresponding to the pilot data but
##' including the complete experimental run. This dataset is used
##' in examples demonstrating power calculations under more
##' realistic sample sizes and hierarchies.
##'
##' @format A data frame containing the full plate assay data.
##'   Column definitions follow those of
##'   \code{\link{plate_assay_pilot_data}}.
##'
##' @seealso \code{\link{plate_assay_pilot_data}},
##'   \code{\link{plate_assay_pilot_data_wo_repeats}}
##' @docType data
##' @keywords datasets
##' @name plate_assay_full_data
##' @usage data(plate_assay_full_data)
NULL

##' Single-nucleus RNA-seq cluster-level count data
##'
##' Example dataset containing cluster-level count summaries from
##' a single-nucleus RNA-seq experiment. The data are intended to
##' illustrate how RMeDPower2 can be applied to hierarchical
##' omics experiments with counts aggregated at the cluster
##' level.
##'
##' @format A data frame of cluster-level counts and associated
##'   annotations.
##'
##' @seealso \code{\link{snRNAseq_gene_count_data}}
##' @docType data
##' @keywords datasets
##' @name snRNAseq_cluster_count_data
##' @usage data(snRNAseq_cluster_count_data)
NULL

##' Single-nucleus RNA-seq gene-level count data
##'
##' Example dataset containing gene-level count summaries from a
##' single-nucleus RNA-seq experiment. This dataset can be used
##' to demonstrate power calculations for differential
##' expression-type analyses across experimental conditions.
##'
##' @format A data frame of gene-level counts and associated
##'   annotations.
##'
##' @seealso \code{\link{snRNAseq_cluster_count_data}}
##' @docType data
##' @keywords datasets
##' @name snRNAseq_gene_count_data
##' @usage data(snRNAseq_gene_count_data)
NULL

##' Mouse behavior data from a Morris Water Maze assay
##'
##' Example behavioral dataset containing measurements from a
##' mouse Morris Water Maze (MWM) assay. The data represent
##' repeated measures across trials and subjects and are suitable
##' for illustrating repeated measures power analysis.
##'
##' @format A data frame of behavioral measurements with
##'   information on mouse, trial, and experimental condition.
##'
##' @docType data
##' @keywords datasets
##' @name mouse_behavior_MWM_assay_data
##' @usage data(mouse_behavior_MWM_assay_data)
NULL

##' Mouse brain electrophysiology data
##'
##' Example dataset containing electrophysiological measurements
##' from mouse brain recordings. The data are used to demonstrate
##' power analyses in experiments with repeated measurements and
##' complex correlation structures.
##'
##' @format A data frame of electrophysiological measurements and
##'   associated experimental annotations.
##'
##' @docType data
##' @keywords datasets
##' @name mouse_brain_electro_physiology_data
##' @usage data(mouse_brain_electro_physiology_data)
NULL

#' @title get_model_and_data
#' @description This function performs a linear mixed model analysis using lmer.
#'
#'
#'
#' @param data Input data
#' @param condition_column Name of the condition variable (ex variable with values such as control/case). The input file has to have a corresponding column name
#' @param experimental_columns Name of variables related to experimental design such as "experiment", "plate", and "cell_line". They should be in order, for example, "experiment" should always come first .
#' @param response_column Name of the variable observed by performing the experiment. ex) intensity.
#' @param total_column Set this column only when family_p="binomial" and it is equal to the total number of observations (number of cases plus number of controls) for a given number of cases
#' @param condition_is_categorical Specify whether the condition variable is categorical. TRUE: Categorical, FALSE: Continuous.
#' @param covariate The name of the covariate to control in the regression model
#' @param crossed_columns Name of experimental variables that may appear repeatedly with the same ID. For example, cell_line C1 may appear in multiple experiments, but plate P1 cannot appear in more than one experiment
#' @param error_is_non_normal Default: the observed variable is continuous Categorical response variable will be implemented in the future. TRUE: Categorical , FALSE: Continuous (default).
#' @param family_p The type of distribution family to specify when the response is categorical. If family is "binary" then binary(link="log") is used, if family is "poisson" then poisson(link="logit") is used, if family is "poisson_log" then poisson(link=") log") is used.
#' @param na.action "complete": missing data is not allowed in all columns (default), "unique": missing data is not allowed only in condition, experimental, and response columns. Selecting "complete" removes an entire row when there is one or more missing values, which may affect the distribution of other features.
#' @param include_interaction Whether to include condition * covariate interaction
#' @param random_slope_variable Variable for random slopes (typically one of "condition_column" or "covariate" and assuming that they are numeric variables). A random slope term is added for each of the variables specified in the experimental columns in addition to their corresponding random intercept terms. The random slope and intercept terms for each experimental_columns variable are assumed to be uncorrelated.
#' @param covariate_is_categorical Specify whether the covariate variable is categorical. TRUE: Categorical, FALSE: Continuous.
#'
#' @return A list of the linear mixed model result, original data, experimental column names, and residual values
#'
#' @keywords internal
#' @noRd



get_model_and_data <- function(data, condition_column, experimental_columns, response_column, total_column = NULL, condition_is_categorical,  covariate=NULL,
                                     crossed_columns=NULL, error_is_non_normal=FALSE, family_p=NULL, na.action="complete",
                               include_interaction = FALSE,
                               random_slope_variable = NULL,
                               covariate_is_categorical = TRUE){



  ######input error handler
  if(!is.null(covariate) )
    if(!covariate%in%colnames(data))
      stop("covariate should be null or one of the column names")
  if(!condition_column%in%colnames(data)) stop("condition_column should be one of the column names")
  if(sum(experimental_columns%in%colnames(data))!=length(experimental_columns)) stop("experimental_columns must match column names")
  if(!response_column%in%colnames(data)) stop("response_column should be one of the column names")
  if(!is.na(condition_is_categorical) && !condition_is_categorical%in%c(TRUE,FALSE)) stop("condition_is_categorical must be TRUE or FALSE")
  if(!is.null(crossed_columns)){if(sum(crossed_columns%in%colnames(data))!=length(crossed_columns)) stop("crossed_columns must match column names")}
  if(!is.na(covariate_is_categorical) && !covariate_is_categorical%in%c(TRUE,FALSE)) stop("covariate_is_categorical must be TRUE or FALSE")

  # Validation for new parameters
  if(!is.na(include_interaction)) {
    if (include_interaction && is.null(covariate)) {
      stop("Cannot include interaction when covariate is NULL")
    }
  }

  if (!is.null(random_slope_variable) &&
      !random_slope_variable %in% c("condition_column", condition_column, "covariate", covariate)) {
    stop("random_slope_variable should be 'condition_column', 'covariate' or the actual condition column name or covariate column name")
  }

  if(error_is_non_normal==TRUE){
    if(family_p != "negative_binomial")
      family_p=switch(family_p,
                      "poisson" = poisson(link="log"),
                      "binomial" = binomial(link="logit"),
                      "bionomial_log" = binomial(link="log"),
                      "Gamma_log" = Gamma(link = "log"),
                      "Gamma" = Gamma(link = "inverse"))
    else{
      family_p = list(family = "negative_binomial")
    }
  }


  if(na.action=="complete"){

    notNAindex=which( rowSums(is.na(data)) == 0 )

  }else if(na.action=="unique"){

    if(is.null(covariate)) notNAindex=which( rowSums(is.na(data[,c(condition_column, experimental_columns, response_column, covariate)])) == 0 )
    else notNAindex=which( rowSums(is.na(data[,c(condition_column, experimental_columns, response_column)])) == 0 )


  }



  fixed_global_variable_data=data[notNAindex,]


  # cat("\n")
  # print("__________________________________________________________________Summary of data:")
  # print(summary(fixed_global_variable_data))
  # cat("\n")

  colnames_original=colnames(fixed_global_variable_data)
  experimental_columns_index=NULL
  ####### assign categorical variables
  if(condition_is_categorical==TRUE)
    fixed_global_variable_data[,condition_column]=as.factor(fixed_global_variable_data[,condition_column])
  else
    fixed_global_variable_data[,condition_column]=as.numeric(fixed_global_variable_data[,condition_column])

  if(!is.null(covariate)) {
    if(covariate_is_categorical==TRUE)
      fixed_global_variable_data[,covariate]=as.factor(fixed_global_variable_data[,covariate])
    else
      fixed_global_variable_data[,covariate]=as.numeric(fixed_global_variable_data[,covariate])
  }
  # random slope should be allowed only with a continuous variable
  if(!is.null(random_slope_variable)) {
    if(!inherits(fixed_global_variable_data[,random_slope_variable], "numeric")) {
      stop("random_slope_variable should be a numeric variable")
    }
  }

  #cat("\n")


  noncrossed_columns=NULL

  for(i in 1:length(experimental_columns)){
    fixed_global_variable_data[,experimental_columns[i]]=as.factor(fixed_global_variable_data[,experimental_columns[i]])
    experimental_columns_index=c(experimental_columns_index,which(colnames(fixed_global_variable_data)==experimental_columns[i]))
    colnames(fixed_global_variable_data)[experimental_columns_index[i]]=paste("experimental_column",i,sep="")

    if(i!=1&&!experimental_columns[i]%in%crossed_columns){
      noncrossed_columns=c(noncrossed_columns, paste("experimental_column",i,sep=""))
    }


    #cat("\n")
    # print(paste("_________________________________",experimental_columns[i]," is assigned to experimental_column",i,sep=""))
    #cat("\n")
  }



  if(length(experimental_columns)>=2){
      for(r in 2:length(experimental_columns)){
      if(colnames(fixed_global_variable_data)[experimental_columns_index[r]]%in%noncrossed_columns){
        fixed_global_variable_data[,experimental_columns_index[r]]=paste(fixed_global_variable_data[,experimental_columns_index[r-1]], fixed_global_variable_data[,experimental_columns_index[r]],sep="_")
        fixed_global_variable_data[,experimental_columns_index[r]]=as.factor(fixed_global_variable_data[,experimental_columns_index[r]])
      }
    }

  }


  # Rename columns to standard names
  colnames(fixed_global_variable_data)[which(colnames(fixed_global_variable_data)==condition_column)]="condition_column"
  colnames(fixed_global_variable_data)[which(colnames(fixed_global_variable_data)==response_column)]="response_column"
  if(!is.null(covariate)) colnames(fixed_global_variable_data)[which(colnames(fixed_global_variable_data)==covariate)]="covariate"
  if(!is.null(random_slope_variable)) {
    if(random_slope_variable == condition_column)
      random_slope_variable = "condition_column"
    else if(random_slope_variable == covariate)
      random_slope_variable = "covariate"
    else{
      stop("random_slope_variable can only be 'condition_column', 'covariate' or the actual condition column name or covariate column name")
    }

  }

  if(!is.null(total_column))
    colnames(fixed_global_variable_data)[which(colnames(fixed_global_variable_data)==total_column)]="total_column"

  # Build formula components
  fixed_formula <- build_fixed_formula(covariate, include_interaction)
  random_formula <- build_random_formula(experimental_columns, random_slope_variable)
  lmerFit <- generate_model_fit(data=fixed_global_variable_data,
                                fixed_formula,
                                random_formula,
                                error_is_non_normal,
                                family_p,
                                total_column)



  slmerFit <- summary(lmerFit)

  #cat("\n")
  # print("__________________________________________________________________Model statistics:")
  # print(slmerFit)
  #cat("\n")



  return(list(lmerFit, fixed_global_variable_data, colnames(fixed_global_variable_data)[experimental_columns_index], slmerFit$residuals))
}

#' @keywords internal
#' @noRd
build_fixed_formula <- function(covariate = NULL, include_interaction = FALSE) {
  if (is.null(covariate)) {
    return("condition_column")
  } else {
    if (include_interaction) {
      return("condition_column * covariate")  # This expands to: condition_column + covariate + condition_column:covariate
    } else {
      return("condition_column + covariate")
    }
  }
}

#' @keywords internal
#' @noRd
build_random_formula <- function(experimental_columns, random_slope_variable = NULL) {
  random_parts <- c()

  for (i in 1:length(experimental_columns)) {
    exp_col <- paste0("experimental_column", i)

    if (!is.null(random_slope_variable) && random_slope_variable == "condition_column") {
      # Add random slope for condition_column
      random_parts <- c(random_parts, paste0("(condition_column || ", exp_col, ")"))
    } else if (!is.null(random_slope_variable) && random_slope_variable == "covariate") {
      # Add random slope for condition_column
      random_parts <- c(random_parts, paste0("(covariate || ", exp_col, ")"))
    } else {
      # Standard random intercept only
      random_parts <- c(random_parts, paste0("(1 | ", exp_col, ")"))
    }
  }

  return(paste(random_parts, collapse = " + "))
}

#' @keywords internal
#' @noRd
build_random_formula0 <- function(experimental_columns) {
  random_parts <- c()

  for (i in 1:length(experimental_columns)) {
    exp_col <- paste0("experimental_column", i)
    # Standard random intercept only
    random_parts <- c(random_parts, paste0("(1 | ", exp_col, ")"))
  }

  return(paste(random_parts, collapse = " + "))
}


#' @keywords internal
#' @noRd
generate_model_fit <- function(data, fixed_formula, random_formula,
                                    error_is_non_normal = FALSE, family_p = NULL,
                                    total_column = NULL) {

  if (error_is_non_normal == FALSE) {
    # Linear mixed effects model
    formula_str <- as.formula(paste("response_column ~", fixed_formula, "+", random_formula))
    lmerFit <- lmerTest::lmer(formula_str, data = data)
  } else if (!is.null(family_p) && family_p$family == "binomial" && !is.null(total_column)) {
    # Binomial with total column
    formula_str <- as.formula(paste("cbind(response_column, (total_column - response_column)) ~",
                                     fixed_formula, "+", random_formula))
    lmerFit <- lme4::glmer(formula_str, data = data, family = family_p)
  } else if (!is.null(family_p) && family_p$family == "negative_binomial" && !is.null(total_column)) {
    # Negative binomial with offset
    formula_str <- as.formula(paste("response_column ~", fixed_formula, "+", random_formula,
                                     "+ offset(log(total_column))"))
    lmerFit <- lme4::glmer.nb(formula_str, data = data, family = family_p)
  } else if (!is.null(family_p) && family_p$family == "poisson" && !is.null(total_column)) {
    # poisson with offset
    formula_str <- as.formula(paste("response_column ~", fixed_formula, "+", random_formula,
                                     "+ offset(log(total_column))"))

    lmerFit <- lme4::glmer(formula_str, data = data, family = family_p)
  }
  else if (!is.null(family_p) && family_p$family == "negative_binomial" && is.null(total_column)) {
    # Negative binomial without offset
    formula_str <- as.formula(paste("response_column ~", fixed_formula, "+", random_formula))
    lmerFit <- lme4::glmer.nb(formula_str, data = data, family = family_p)
  }
  else {
    # Other GLMMs
    formula_str <- as.formula(paste("response_column ~", fixed_formula, "+", random_formula))
    lmerFit <- lme4::glmer(formula_str, data = data, family = family_p)
  }

  lmerFit@call$formula = formula_str
  lmerFit@call$data = as.name("data.update")
  return(lmerFit)
}

#' @keywords internal
#' @noRd
generate_model_fit0 <- function(data,  random_formula,
                               error_is_non_normal = FALSE, family_p = NULL,
                               total_column = NULL) {

  if (error_is_non_normal == FALSE) {
    # Linear mixed effects model
    formula_str <- paste("response_column ~",  random_formula)
    lmerFit <- lmerTest::lmer(as.formula(formula_str), data = data)
  } else if (!is.null(family_p) && family_p$family == "binomial" && !is.null(total_column)) {
    # Binomial with total column
    formula_str <- paste("cbind(response_column, (total_column - response_column)) ~", random_formula)
    lmerFit <- lme4::glmer(as.formula(formula_str), data = data, family = family_p)
  } else if (!is.null(family_p) && family_p$family == "negative_binomial" && !is.null(total_column)) {
    # Negative binomial with offset
    formula_str <- paste("response_column ~", random_formula, "+ offset(log(total_column))")
    lmerFit <- lme4::glmer.nb(as.formula(formula_str), data = data, family = family_p)
  } else if (!is.null(family_p) && family_p$family == "poisson" && !is.null(total_column)) {
    # Poisson with offset
    formula_str <- paste("response_column ~",  random_formula,"+ offset(log(total_column))")
    lmerFit <- lme4::glmer(as.formula(formula_str), data = data, family = family_p)
  }
  else if (!is.null(family_p) && family_p$family == "negative_binomial" && is.null(total_column)) {
    # Negative binomial without offset
    formula_str <- as.formula(paste("response_column ~",  random_formula))
    lmerFit <- lme4::glmer.nb(formula_str, data = data, family = family_p)
  }
  else {
    # Other GLMMs
    formula_str <- paste("response_column ~",  random_formula)
    lmerFit <- lme4::glmer(as.formula(formula_str), data = data, family = family_p)
  }


  lmerFit@call$formula = formula_str
  lmerFit@call$data = as.name("data.update")
  return(lmerFit)
}

#' @title readPowerParams
#'
#' @description This functions reads the underlying design for the data
#'
#'
#' @param jsonfile the jsonfile with the necessary parameters for statistical power estimation: target_columns, power_curve, nsimn, levels, max_size, alpha, breaks, effect_size, icc
#'
#' @return an object of class PowerParams
#' @export
#'
#' @examples
#' template_dir <- system.file("input_templates/cell_assay_data", package = "RMeDPower2")
#' power_param <- readPowerParams(file.path(template_dir,"power_param.json"))

readPowerParams <- function(jsonfile) {
  Power_Param_data <- jsonlite::fromJSON(jsonfile)

  power_param <- new("PowerParams")

  power_param@target_columns = Power_Param_data$target_columns
  power_param@power_curve = Power_Param_data$power_curve
  power_param@nsimn = Power_Param_data$nsimn
  power_param@levels = Power_Param_data$levels
  power_param@alpha = Power_Param_data$alpha
  if(!is.null(Power_Param_data$max_size)) power_param@max_size = Power_Param_data$max_size
  if(!is.null(Power_Param_data$breaks)) power_param@breaks = Power_Param_data$breaks
  if(!is.null(Power_Param_data$effect_size)) power_param@effect_size = Power_Param_data$effect_size
  if(!is.null(Power_Param_data$icc)) power_param@icc = Power_Param_data$icc
  return(power_param)
}

#' @title transform_data
#'
#' @description This function can be used to generate diagnostic QC plots for given model assumptions related to the input data, identify potential outlier observations and/or outlier experimental units
#'
#'
#' @param data Input data
#' @param condition_column Name of the condition variable (ex variable with values such as control/case). The input file has to have a corresponding column name
#' @param experimental_columns Name of the variable related to experimental design such as "experiment", "plate", and "cell_line". They should be in order, for example, "experiment" should always come first .
#' @param response_column Name of the variable observed by performing the experiment. ex) intensity.
#' @param total_column Set this column only when family_p="binomial" and it is equal to the total number of observations (number of cases plus number of controls) for a given number of cases
#' @param condition_is_categorical Specify whether the condition variable is categorical. TRUE: Categorical, FALSE: Continuous.
#' @param covariate The name of the covariate to control in the regression model
#' @param crossed_columns Name of experimental variables that may appear repeatedly with the same ID. For example, cell_line C1 may appear in multiple experiments, but plate P1 cannot appear in more than one experiment
#' @param error_is_non_normal Default: the observed variable is continuous Categorical response variable will be implemented in the future. TRUE: Categorical , FALSE: Continuous (default).
#' @param family_p The type of distribution family to specify when the response is categorical. If family is "binary" then binary(link="log") is used, if family is "poisson" then poisson(link="logit") is used, if family is "poisson_log" then poisson(link=") log") is used.
#' @param alpha numeric scalar between 0 and 1 indicating the Type I error associated with the test of outliers
#' @param na.action "complete": missing data is not allowed in all columns (default), "unique": missing data is not allowed only in condition, experimental, and response columns. Selecting "complete" removes an entire row when there is one or more missing values, which may affect the distribution of other features.
#' @param include_interaction logical - TRUE or FALSE - Whether to include condition * covariate interaction
#' @param random_slope_variable Variable for random slopes (typically one of "condition_column" or "covariate" and assuming that they are numeric variables). A random slope term is added for each of the variables specified in the experimental columns in addition to their corresponding random intercept terms. The random slope and intercept terms for each experimental_columns variable are assumed to be uncorrelated.
#' @param covariate_is_categorical Specify whether the covariate variable is categorical. TRUE: Categorical, FALSE: Continuous.
#'
#' @return A list with four elements. 1) models: representing the names of the models
#' evaluated based on differnt modifications of the response column.
#' The models would include one called natural_scale,
#' another model called natural_scale_wo_outliers if outliers had beeen identified,
#' another model called log_scale if the respose column is continuous
#' and the model on the log-transformed values of the responses are what was evaluated
#' and finally log_scale_wo_outliers model if there were outliers identified in the log_scale model.
#' 2) Data_updated representing the updated data frame with additional columns for the modified response column corresponding to each of the models evaluated.
#' 3) cooks_result: cooks distance of each of the experimental columns for each of the models evaluated.
#' For models based on the binomial probability distribution, cooks distance is only
#' reported for the first experimental column on account the increased computation time
#' for evaluating this metric for the other experimental columns.
#' 4) diagnostic_plots: is a list with two elements plots and captions. plots is a named list and captions is a character vector,
#' both of the same length as the number of models evaluated. Each element of the plots list is yet another
#' list of QC/diagnostic plots related to the corresponding model fit, while the captions is a vector of captions for each of the
#' QC plots output
#' 5) cooks_plots: is a list of plots is named list of the same length as the number of models evaluated. Each element of the cooks_plot is another list of bar plots of cooks distance plots for each of the experimental columns. The title of the plot indicate the model and experimental factor while the subtitle indicates the identified outliers if any using the 4/n threshold
#'
#' @keywords internal
#' @noRd

transform_data<-function(data, condition_column, experimental_columns, response_column, total_column=NULL, condition_is_categorical=TRUE, covariate=NULL,
                         crossed_columns = NULL, error_is_non_normal=FALSE, family_p=NULL, alpha=0.05, na.action="complete", include_interaction = NA, random_slope_variable = NULL, covariate_is_categorical = NA){



  if(na.action=="complete"){

    notNAindex=which( rowSums(is.na(data)) == 0 )

  }else if(na.action=="unique"){

    if(is.null(covariate)) notNAindex=which( rowSums(is.na(data[,c(condition_column, experimental_columns, response_column, covariate)])) == 0 )
    else notNAindex=which( rowSums(is.na(data[,c(condition_column, experimental_columns, response_column)])) == 0 )


  }



  data=data[notNAindex,]




  Data_updated=data

  plots_info <- list()
  cooks_result <- list()
  model_no <- 1
  models <- c("Natural scale")

  family_p_temp = family_p
  lms=get_model_and_data(data=data, condition_column=condition_column, experimental_columns=experimental_columns,
                         response_column=response_column, total_column = total_column, condition_is_categorical=condition_is_categorical, covariate=covariate,
                         crossed_columns=crossed_columns, error_is_non_normal=error_is_non_normal, family_p=family_p_temp, na.action=na.action,include_interaction=include_interaction, random_slope_variable=random_slope_variable, covariate_is_categorical = covariate_is_categorical)

  plots_info[[model_no]] <- generate_qc_plots(lms = lms,
                                           error_is_non_normal = error_is_non_normal,
                                           description_suffix = "(natural scale)",
                                           experimental_columns,
                                           alpha)
  names(plots_info)[model_no] <- "natural_scale"

  residual <- plots_info[[model_no]]$residual
  #
  cutoffs=rosner_test(trait=residual, response_column=response_column, alpha=alpha, hist_text = "raw residual")

  #run cook

  choose_cols <- vector(mode = "character")
  temp_count <- 0
  for(c in 1:length(lms[[3]])) {
    if(length(unique(lms[[2]][[lms[[3]][c]]])) > 2) {
      temp_count <- temp_count + 1
      choose_cols[temp_count] <- lms[[3]][c]
    }
  }

  if(temp_count > 0) {
    cooks_result[[model_no]]=cooks_test(lms[[1]], family_p, lms[[2]], choose_cols, response_column="response_column", hist_text="raw")
    names(cooks_result[[model_no]]) <- paste0(names(cooks_result[[model_no]]), "_raw")

  }
  else {
    message("Not enough grouping levels to perform the cook analyses on the experimental factors")
    return()
  }

  Data_noOutlier=Data_updated
  Data_noOutlier[is.infinite(residual),] <- NA
  if(sum(is.na(cutoffs))<2 | sum(is.infinite(residual)) > 0){

    model_no <- model_no + 1
    models[model_no] <- "Natural scale wo outliers"

    if(!is.na(cutoffs[1])){
      Data_noOutlier[residual<=cutoffs[1],]=NA
    }
    if(!is.na(cutoffs[2])){
      Data_noOutlier[residual>=cutoffs[2],]=NA
    }

    Data_updated=cbind(Data_noOutlier[,response_column], Data_updated)
    colnames(Data_updated)[1]=paste0(response_column,"_noOutlier")

    lms=get_model_and_data(data=Data_updated, condition_column=condition_column, experimental_columns=experimental_columns,
                           response_column=paste0(response_column,"_noOutlier"), total_column = total_column, condition_is_categorical=condition_is_categorical, covariate=covariate,
                           crossed_columns=crossed_columns, error_is_non_normal=error_is_non_normal, family_p=family_p_temp, na.action=na.action,include_interaction=include_interaction, random_slope_variable=random_slope_variable, covariate_is_categorical = covariate_is_categorical)

    plots_info[[model_no]] <- generate_qc_plots(lms = lms,
                                           error_is_non_normal = error_is_non_normal,
                                           description_suffix = " (natural scale wo outliers)",
                                           experimental_columns,
                                           alpha)
    names(plots_info)[model_no] <- "natural_scale_wo_outliers"

    #run cook


    if(length(choose_cols) > 0) {
      cooks_result[[model_no]]=cooks_test(lms[[1]], family_p, lms[[2]], choose_cols, response_column="response_column", hist_text="raw wo outliers")
      names(cooks_result[[model_no]]) <- paste0(names(cooks_result[[model_no]]), "_raw_wo_outliers")

    }
    else {
      message("Not enough grouping levels to perform the cook analyses on the experimental factors")
      return()
    }


  }


  if(error_is_non_normal==FALSE){

    ########################do the same using log transformed values

    ###log transform

    model_no <- model_no + 1
    models[model_no] <- "Log scale"
    Data_log=data
    temp1=Data_log[, response_column]


    #seperate zero and negative
    if(min(Data_log[,response_column])<0){

      temp2 = abs(min(temp1))/10 - min(temp1)
      Data_log[,response_column]=log(Data_log[,response_column]+temp2)

    }else if(min(Data_log[,response_column])==0){

      temp2 = min(temp1[temp1>0])/10
      Data_log[,response_column]=log(Data_log[,response_column]+temp2)

    }else{

      Data_log[,response_column]=log(Data_log[,response_column])

    }


    Data_updated=cbind(Data_log[,response_column], Data_updated)
    colnames(Data_updated)[1]=paste0(response_column,"_logTransformed")



    lms=get_model_and_data(data=Data_updated, condition_column=condition_column, experimental_columns=experimental_columns,
                           response_column=paste0(response_column,"_logTransformed"), condition_is_categorical=condition_is_categorical, covariate=covariate,
                           crossed_columns=crossed_columns, error_is_non_normal=error_is_non_normal, family_p=family_p_temp, na.action=na.action, include_interaction=include_interaction, random_slope_variable=random_slope_variable, covariate_is_categorical = covariate_is_categorical)

    plots_info[[model_no]] <- generate_qc_plots(lms = lms,
                                            error_is_non_normal = FALSE,
                                            description_suffix = "(log scale)",
                                            experimental_columns,
                                            alpha)
    names(plots_info)[model_no] <- "log_scale"

    #run cook


    if(length(choose_cols) > 0) {
      cooks_result[[model_no]]=cooks_test(lms[[1]], family_p, lms[[2]], choose_cols, response_column="response_column", hist_text="log transform")
      names(cooks_result[[model_no]]) <- paste0(names(cooks_result[[model_no]]), "_logTransformed")
    }
    else {
      message("Not enough grouping levels to perform the cook analyses on the experimental factors")
      return()
    }


    residual <- plots_info[[model_no]]$residual

    # trait = residual ###change the feature as you want
    #
    #
    #
    # #if(method=="rosner"){
    #
    cutoffs=rosner_test(trait=lms[[4]], response_column=response_column, alpha=alpha, hist_text = "log residual")

    if(sum(is.na(cutoffs))<2){

      model_no <- model_no + 1
      models[model_no] <- "Log scale wo outliers"
      Data_log_noOutlier=Data_log

      if(!is.na(cutoffs[1])){
        Data_log_noOutlier[residual<=cutoffs[1],]=NA
      }
      if(!is.na(cutoffs[2])){
        Data_log_noOutlier[residual>=cutoffs[2],]=NA
      }

      Data_updated=cbind(Data_log_noOutlier[,response_column], Data_updated)
      colnames(Data_updated)[1]=paste0(response_column,"_logTransformed_noOutlier")

      lms=get_model_and_data(data=Data_updated, condition_column=condition_column, experimental_columns=experimental_columns,
                             response_column=paste0(response_column,"_logTransformed_noOutlier"), condition_is_categorical=condition_is_categorical, covariate=covariate,
                             crossed_columns=crossed_columns, error_is_non_normal=error_is_non_normal, family_p=family_p_temp, na.action=na.action, include_interaction=include_interaction, random_slope_variable=random_slope_variable, covariate_is_categorical = covariate_is_categorical)

      plots_info[[model_no]] <- generate_qc_plots(lms = lms,
                                              error_is_non_normal = FALSE,
                                              description_suffix = "(log scale wo outlier)",
                                              experimental_columns,
                                              alpha)

      names(plots_info)[model_no] <- "log_scale_wo_outliers"

      #run cook


      if(length(choose_cols) > 0) {
        cooks_result[[model_no]]=cooks_test(lms[[1]], family_p, lms[[2]], choose_cols, response_column="response_column", hist_text="log transform wo outliers")
        names(cooks_result[[model_no]]) <- paste0(names(cooks_result[[model_no]]), "_logTransformed_wo_outliers")
      }
      else {
        message("Not enough grouping levels to perform the cook analyses on the experimental factors")
        return()
      }


    }


  }

  cooks_plots_plus_outliers <- generate_cooks_results_plots(cooks_result, names(plots_info), experimental_columns)

  result=list(models, plots_info, cooks_plots_plus_outliers$cooks_plots, cooks_plots_plus_outliers$inferred_outliers, cooks_result, Data_updated )
  names(result)=c("models", "diagnostic_plots", "cooks_plots", "inferred_outlier_groups","cooks_result", "Data_updated")

  return(result)
}

#' @keywords internal
#' @noRd
generate_cooks_results_plots <- function(cooks_result, models, experimental_columns) {
  cooks_plots <- list()
  inferred_outliers <- list()
  for(i in 1:length(models)) {
    cooks_plots[[i]] <- list()
    inferred_outliers[[i]] <- list()
    names(cooks_plots)[i] <- models[i]
    names(inferred_outliers)[i] <- models[i]
    #
    for(j in 1:length(cooks_result[[i]])){
      plot_title <- paste0("Cooks distance estimates for model ", models[i], " and experimental factor = ", experimental_columns[j])
      temp_data <- data.frame(cooks_distance = cooks_result[[i]][[j]])
      temp_data %<>% tibble::rownames_to_column("exp.factor")
      outliers <- temp_data %>% filter(cooks_distance > 4/nrow(.)) %>% .$exp.factor
      if(length(outliers) == 0)
        outliers <- "none"
      cooks_plots[[i]][[j]] <- ggplot(temp_data, aes(y=exp.factor, x=cooks_distance)) +
        geom_col() +
        theme_minimal() +
        geom_vline(xintercept = 4/nrow(temp_data), lty=2) +
        theme(axis.text.y = element_text(size = 8)) +
        ggtitle(plot_title) +
        labs(subtitle = paste0("Outliers are: ", paste(outliers, collapse = ","), " determined by the 4/n threshold as indicated by the vertical dashed line")) +
        theme(plot.title  = element_textbox_simple()) +
        theme(plot.subtitle  = element_textbox_simple())
      inferred_outliers[[i]][[j]] <- outliers
      names(inferred_outliers[[i]])[j] <- experimental_columns[j]
     }
  }

  return(list(cooks_plots=cooks_plots, inferred_outliers=inferred_outliers))
}

#' @keywords internal
#' @noRd
generate_qc_plots <-  function(lms,
                               error_is_non_normal,
                               description_suffix,
                               experimental_columns,
                               alpha) {

  plot_index <- 0
  plots <- list()
  captions <- vector(mode = "character")

  if(error_is_non_normal==FALSE){

    residual <- lms[[4]]
    model_data <- data.frame(fitted = predict(lms[[1]], type = "link"),
                             model_residuals = lms[[4]])
    model_data$sqrt_abs_residuals = sqrt(abs(model_data$model_residuals))


    plots[[plot_index+1]] <- ggplot(model_data, aes(sample = model_residuals)) +
      stat_qq() +
      stat_qq_line(color = "red") +
      labs(title = paste0("Q-Q Plot: Residuals", description_suffix),
           subtitle = paste0("Check normality of residuals", ". Expectation is that the black points lie on or close to the solid red diagonal line"),
           x = "Theoretical Quantiles",
           y = "Sample Quantiles") +
      theme_minimal() +
      theme(plot.title = element_text(size = 12, face = "bold")) +
      theme(plot.subtitle  = element_textbox_simple())

    names(plots)[plot_index+1] <- "residuals_QQ"

    captions[plot_index+1] <- paste0("Check normality of residuals ", description_suffix, ". Expectation is that the black points lie on or close to the solid red diagonal line. Check the Normal Q-Q plots for good and bad examples here: https://library.virginia.edu/data/articles/diagnostic-plots ")



    # 1a. Residuals vs Fitted Values
    plots[[plot_index+2]] <- ggplot(model_data, aes(x = fitted, y = model_residuals)) +
      geom_point(alpha = 0.6, size = 2) +
      geom_smooth(method = "loess", se = TRUE, color = "red") +
      geom_hline(yintercept = 0, linetype = "dashed", color = "blue") +
      labs(title = paste0("Residuals vs Fitted Values ", description_suffix),
           subtitle = paste0("Check for linearity", ". Expectation is that the best fit solid red line is horizontal or close to being horizontal"),
           x = "Fitted Values",
           y = "Residuals") +
      theme_minimal() +
      theme(plot.title = element_text(size = 12, face = "bold"))+
      theme(plot.subtitle  = element_textbox_simple())

    names(plots)[plot_index+2] <- "residuals_vs_fitted"
    captions[plot_index+2] <- paste0("Check for linearity ", description_suffix, ". Expectation is that the best fit solid red line is horizontal or close to being horizontal. Check the Residuals vs Fitted plots for good and bad examples here: https://library.virginia.edu/data/articles/diagnostic-plots")

    # 1b. Scale-Location Plot (Square root of standardized residuals vs fitted)
    plots[[plot_index+3]] <- ggplot(model_data, aes(x = fitted, y = sqrt_abs_residuals)) +
      geom_point(alpha = 0.6, size = 2) +
      geom_smooth(method = "loess", se = TRUE, color = "red") +
      labs(title = paste0("Scale-Location Plot ", description_suffix),
           subtitle = paste0("Check for homoscedasticity", ". Expectation is that the best fit solid red line is horizontal or close to being so"),
           x = "Fitted Values",
           y = "\u221a|Standardized Residuals|") +
      theme_minimal() +
      theme(plot.title = element_text(size = 12, face = "bold"))+
      theme(plot.subtitle  = element_textbox_simple())

    names(plots)[plot_index+3] <- "residuals_homoscedasticity"
    captions[plot_index+3] <- paste0("Check for homoscedasticity ",description_suffix, ". Expectation is that the best fit solid red line is horizontal or close to being so. Check the Scale-Location plots for good and bad examples here: https://library.virginia.edu/data/articles/diagnostic-plots")
    plot_index <- plot_index + 3

  }else{
    simulationOutput <- DHARMa::simulateResiduals(lms[[1]], plot = F)
    residual = residuals(simulationOutput, quantileFunction = qnorm)
    # plot(simulationOutput)
    # DHARMa::plotResiduals(simulationOutput, form =  lms[[2]]$condition_column)

    plots[[plot_index + 1]] <- ggplot_QQunif(simulationOutput, description_suffix = description_suffix)
    names(plots)[plot_index+1] <- "residuals_QQ"
    captions[plot_index + 1] <- paste0("Q-Q Plot for Uniform Distribution", ". Expectation is that the black points lie on or close to the solid red diagonal line. Check the Q-Q plots for good and bad examples here: https://library.virginia.edu/data/articles/diagnostic-plots ")

    plots[[plot_index + 2]] <- ggplot_residuals_vs_predictor(simulationOutput, description_suffix = description_suffix)
    names(plots)[plot_index+2] <- "residuals_vs_predicted"
    captions[plot_index + 2] <- paste0("Residuals vs Predicted", " Check the Residuals vs Fitted plots for good and bad examples here: https://library.virginia.edu/data/articles/diagnostic-plots ")

    plot_index <- plot_index + 2
  }


  random_effects <- lme4::ranef(lms[[1]])

  temp_count <- 1
  for(c in (1:length(random_effects))) {
    exp_factor <- experimental_columns[as.integer(gsub("experimental_column", "", names(random_effects)[c]))]
    for(j in 1:ncol(random_effects[[c]])) {

      model_data <- data.frame(model_residuals = random_effects[[c]][,j])
      plots[[plot_index+1]] <- ggplot(model_data, aes(sample = model_residuals)) +
        stat_qq() +
        stat_qq_line(color = "red") +
        labs(title = paste0("Q-Q Plot: Random effects for ", exp_factor),
             subtitle = paste0(description_suffix, ": Check normality of random effects for ", names(random_effects)[c], "_", colnames(random_effects[[c]][j]), " ", ": ", exp_factor, ". Expectation is that the black points lie on or close to the solid red diagonal line"),
             x = "Theoretical Quantiles",
             y = "Sample Quantiles") +
        theme_minimal() +
        theme(plot.title = element_text(size = 12, face = "bold"))+
        theme(plot.subtitle  = element_textbox_simple())

      names(plots)[plot_index+1] <- paste0("random_effects_QQ_",temp_count)
      captions[plot_index+1] <- paste0("Check normality of random effects for ", names(random_effects)[c], "_", colnames(random_effects[[c]][j]), " ", description_suffix, ": ", exp_factor, ". Expectation is that the black points lie on or close to the solid red diagonal line. Check the Normal Q-Q plots for good and bad examples here: https://library.virginia.edu/data/articles/diagnostic-plots ")

      plot_index <- plot_index + 1
      temp_count <- temp_count + 1
    }
  }

  cutoffs=rosner_test(trait=residual, response_column=response_column, alpha=alpha, hist_text = "raw residual")
  if(sum(is.na(cutoffs))<2 | sum(is.infinite(residual)) > 0){
    residuals_df <- data.frame(residual)
    plots[[plot_index+1]] <- ggplot(residuals_df, aes(x=residual)) +
      geom_histogram() +
      labs(title = "Histogram of Residuals Values",
           subtitle = "Histogram of residuals values. Red vertical lines (if present) are the location of the cutoffs to identify the outliers. No red vertical lines implies that the residual values that were estimated to be infinite were identified as outliers",
           x = "Residuals") +
      theme_minimal() +
      theme(plot.title = element_text(size = 12, face = "bold"))+
      theme(plot.subtitle  = element_textbox_simple())

    if(!is.na(cutoffs[1])){
      plots[[plot_index+1]] <- plots[[plot_index+1]] + geom_vline(xintercept = cutoffs[1], color = "red")
    }
    if(!is.na(cutoffs[2])){
      plots[[plot_index+1]] <- plots[[plot_index+1]] + geom_vline(xintercept = cutoffs[2], color = "red")
    }

    names(plots)[plot_index+1] = "residuals_histogram"
    captions[plot_index+1] = "Histogram of residuals values. Red vertical lines (if present) are the location of the cutoffs to identify the outliers. No red vertical lines implies that the residual values that were estimated to be infinite were identified as outliers"
    plot_index <- plot_index + 1
  }
  return(list(plots = plots, captions = captions, residual = residual))
}

#' @keywords internal
#' @noRd
rosner_test<- function (trait, response_column, alpha, hist_text) {

  cutoff1=NA
  cutoff2=NA


  upper_bound <- median(trait) + 3 * mad(trait)
  upper_bound

  lower_bound <- median(trait) - 3 * mad(trait)
  lower_bound

  outlierC=sum(trait>upper_bound)+sum(trait<lower_bound)
  outlierC

  if( outlierC >0 ){

    ###rosner's test
    suppressWarnings(
    test <- EnvStats::rosnerTest(trait,
                                 k = outlierC, alpha=alpha)
    )
    outliers=test$all.stats$Value[test$all.stats$Outlier]

    if(length(test$all.stats$Outlier) !=0  ){
      if(! ( length(test$all.stats$Outlier) ==1 &  sum(is.infinite(test$all.stats$Outlier)) ) ) {


        if(sum(outliers>median(trait)) >0) cutoff1=min( outliers[which(outliers>median(trait))] )
        if(sum(outliers<median(trait)) >0) cutoff2=max( outliers[which(outliers<median(trait))] )




        ############rosner's test end


      }



    }else{
      message("No outlier detected from the raw Data")
    }


  }else{
    message("No outlier detected from the raw Data")
  }

  return(c(cutoff2,cutoff1))
}

#' @keywords internal
#' @noRd
cooks_test<- function (model, family_p, fixed_global_variable_data, experimental_columns, response_column, hist_text) {

  n_cols_2_test <- length(experimental_columns)
  perform_test <- TRUE
  ##influence takes a very long time for testing influential observations higher than the first level
  if("glmerMod" %in% class(model)) {
    if(summary(model)$family == "binomial") {
      n_cols_2_test <- 1
      if("experimental_column1" %in% experimental_columns)
        experimental_columns = "experimental_column1"
      else{
        message("Not enough levels to perform cooks test for experimental_column1 with the assumed binomial distribution")
        cooks_result <- NA
        perform_test <- FALSE
      }
    }
  }


  if(perform_test) {
    cooks_result=lapply(1:n_cols_2_test,
                        function(i){

                          tb=table(fixed_global_variable_data[,"condition_column"], fixed_global_variable_data[,experimental_columns[i]])
                          tb=tb[rowSums(tb)>0,]
                          tb[tb>0]=1
                          condition_counts=rowSums(tb)


                          if(length(condition_counts)==2 & sum(condition_counts==1)==1){
                            not2exclude=fixed_global_variable_data[fixed_global_variable_data[,"condition_column"]==names(condition_counts[condition_counts==1]), experimental_columns[i]][1]
                            not2exclude=as.character(not2exclude)

                            cd=NULL

                            levels=setdiff(fixed_global_variable_data[,experimental_columns[i]],not2exclude)
                            for(level2test in levels ){
                              alt.est <- influence2(model, family_p, group=experimental_columns[i], select=level2test)
                              cd=rbind(cd, influence.ME::cooks.distance.estex(alt.est))
                            }
                            rownames(cd)=levels
                            cd

                          }else{
                            alt.est <- influence2(model, family_p, group=experimental_columns[i] )
                            influence.ME::cooks.distance.estex(alt.est)
                          }


                        }
    )


    names(cooks_result)=c(paste0("cooks_distance_",experimental_columns[1:n_cols_2_test]) )


  }

  return(cooks_result)

}

# Function to extract DHARMa residuals and create ggplot-ready data
#' @keywords internal
#' @noRd
extract_dharma_data <- function(dharma_obj) {
  data.frame(
    residuals = dharma_obj$scaledResiduals,
    predicted = dharma_obj$fittedPredictedResponse,
    observed = dharma_obj$observedResponse,
    outliers = as.logical(DHARMa::outliers(dharma_obj, return = "logical")),
    rank_predicted = rank(dharma_obj$fittedPredictedResponse),
    index = 1:length(dharma_obj$scaledResiduals)
  )
}

# ==============================================================================
# 1. GGPLOT VERSION OF plotQQunif()
# ==============================================================================

#' @keywords internal
#' @noRd
ggplot_QQunif <- function(dharma_obj,
                          title = "Q-Q Plot vs. Uniform Distribution",
                          test_uniformity = FALSE,
                          test_dispersion = FALSE,
                          test_outliers = FALSE,
                          description_suffix = "") {

  residuals <- dharma_obj$scaledResiduals
  n <- length(residuals)

  # Create theoretical quantiles for uniform distribution
  theoretical <- (1:n - 0.5) / n
  observed <- sort(residuals)

  # Create data frame
  qq_data <- data.frame(
    theoretical = theoretical,
    observed = observed
  )

  # Base plot
  p <- ggplot(qq_data, aes(x = theoretical, y = observed)) +
    geom_point(alpha = 0.6, size = 1.5) +
    geom_abline(intercept = 0, slope = 1, color = "red",  linewidth = 1) +
    labs(title = paste(title, description_suffix),
         subtitle = paste0("Q-Q Plot for Uniform Distribution", ". Expectation is that the black points lie on or close to the solid red diagonal line."),
         x = "Expected (Uniform)",
         y = "Observed Residuals") +
    theme_minimal() +
    theme(plot.title = element_text(size = 12, face = "bold")) +
    coord_cartesian(xlim = c(0, 1), ylim = c(0, 1)) +
    theme(plot.subtitle  = element_textbox_simple())

  # Add test results as subtitle if requested
  test_results <- character()

  if (test_uniformity) {
    ks_test <- DHARMa::testUniformity(dharma_obj, plot = FALSE)
    test_results <- c(test_results, paste("KS test: p =", round(ks_test$p.value, 4)))
  }

  if (test_dispersion) {
    disp_test <- DHARMa::testDispersion(dharma_obj, plot = FALSE)
    test_results <- c(test_results, paste("Dispersion test: p =", round(disp_test$p.value, 4)))
  }

  if (test_outliers) {
    outlier_test <- DHARMa::testOutliers(dharma_obj, plot = FALSE)
    test_results <- c(test_results, paste("Outlier test: p =", round(outlier_test$p.value, 4)))
  }

  if (length(test_results) > 0) {
    p <- p + labs(subtitle = paste(test_results, collapse = " | "))
  }

  return(p)
}


# Plot residuals against custom predictor with DHARMa-style formatting
#' @keywords internal
#' @noRd
ggplot_residuals_vs_predictor <- function(dharma_obj,
                                          predictor = NULL,
                                          predictor_name = "predictions",   #or "predictions"
                                          condition_is_categorical = TRUE,
                                          rank = TRUE,
                                          quantreg = TRUE,
                                          description_suffix = "") {

  plot_data <- extract_dharma_data(dharma_obj)
  if(predictor_name == "predictions")
    plot_data$predictor <- plot_data$rank_predicted
  if(predictor_name == "condition_column" & !is.null(predictor))
    plot_data$predictor <- predictor

  if(predictor_name == "condition_column" & condition_is_categorical)
    plot_data$predictor = as.factor(plot_data$predictor)

  if (rank & !is.factor(plot_data$predictor)) {
    plot_data$predictor <- rank(plot_data$predictor)
    plot_data$predictor <- plot_data$predictor/max(plot_data$predictor)
    x_label <- paste(predictor_name, "(Rank Transformed)")
  } else {
    x_label <- predictor_name
  }

  if(is.factor(plot_data$predictor) | (length(unique(plot_data$predictor)) < 10) & (length(plot_data$predictor) >= 10)){
    plot_data$predictor <- as.factor(plot_data$predictor)
    p <- ggplot(plot_data, aes(x = predictor, y = residuals)) +
      geom_boxplot() +
      theme_minimal() +
      geom_hline(yintercept = 0.5, linetype = "dashed", color = "black", alpha = 0.6) +
      labs(title = paste("DHARMa Residuals vs", predictor_name, description_suffix),
           subtitle = paste0("Residuals vs Predicted", ". Expectation is that the median of the boxplots are at or close to 0.5"),
           x = x_label,
           y = "DHARMa Residuals")
  }
  else{
    p <- ggplot(plot_data, aes(x = predictor, y = residuals)) +
      geom_point(color = "black", alpha = 0.6, size = 1.5) +
      theme_minimal()

    if (quantreg && nrow(plot_data) > 10) {
      # Add quantile regression lines
      quantiles <- c(0.25, 0.5, 0.75)
      for (q in quantiles) {
        p <- p + geom_smooth(formula = y ~ poly(x,2), method = quantreg::rq, method.args = list(tau = q),
                             se = FALSE, color = "red", linewidth = 0.8, alpha = 0.7)
      }
      # Add expected horizontal lines
      for (q in quantiles) {
        p <- p + geom_hline(yintercept = q, linetype = "dashed",
                            color = "black", alpha = 0.6)
      }

      p <- p + labs(title = paste("DHARMa Residuals vs", predictor_name, description_suffix),
             subtitle = paste0("Residuals vs Predicted", ". Expectation is that the best fit red lines at the three quartiles- 0.25, 0.50 and 0.75 - are close to the dashed horizontal lines at their respective quartiles"),
             x = x_label,
             y = "DHARMa Residuals")
    } else {
      p <- p + geom_smooth(method = "lm", se = TRUE, color = "red", alpha = 0.7) +
        geom_hline(yintercept = 0.5, linetype = "dashed", color = "black", alpha = 0.6) +
        labs(title = paste("DHARMa Residuals vs", predictor_name, description_suffix),
             subtitle = paste0("Residuals vs Predicted", ". Expectation is that the best fit solid red line(s) is horizontal or close to being so"),
             x = x_label,
             y = "DHARMa Residuals")
  }
}
  p <- p +
    theme_minimal() +
    theme(plot.title = element_text(size = 12, face = "bold")) +
    ylim(0, 1) +
    theme(plot.subtitle  = element_textbox_simple())

  return(p)
}

#' @keywords internal
#' @noRd
ensureDHARMa <- function(simulationOutput, convert = TRUE) {
  # Check if it's already a DHARMa object
  if (inherits(simulationOutput, "DHARMa")) {
    return(simulationOutput)
  }

  # Check if it's a numeric vector of residuals
  if (is.numeric(simulationOutput) && is.vector(simulationOutput)) {
    # Assume it's scaled residuals between 0 and 1
    if (any(simulationOutput < 0 | simulationOutput > 1, na.rm = TRUE)) {
      warning("Residuals not between 0 and 1. Are you sure these are DHARMa residuals?")
    }
    # Create a minimal DHARMa-like object
    out <- list(
      scaledResiduals = simulationOutput,
      fittedPredictedResponse = rep(0.5, length(simulationOutput))  # placeholder
    )
    class(out) <- "DHARMa"
    return(out)
  }

  # If convert is TRUE, try to extract from model object
  if (convert == TRUE || convert == "Model") {
    stop("DHARMa::plotResiduals > wrong argument to function, simulationOutput must be a DHARMa object or a numeric vector of quantile residuals! If you want to create DHARMa residuals from a model object, run simulateResiduals() first.")
  }

  stop("simulationOutput must be a DHARMa object, a numeric vector, or a supported model object")
}

# Helper function to ensure predictor
#' @keywords internal
#' @noRd
ensurePredictor <- function(simulationOutput, form = NULL) {
  if (is.null(form)) {
    # Use fitted predictions from DHARMa object
    if (!is.null(simulationOutput$fittedPredictedResponse)) {
      return(simulationOutput$fittedPredictedResponse)
    } else {
      stop("No predictor specified and no fitted predictions available in DHARMa object")
    }
  }

  # If form is provided directly as a vector
  if (is.numeric(form) || is.factor(form)) {
    if (length(form) != length(simulationOutput$scaledResiduals)) {
      stop("Length of form does not match number of residuals")
    }
    return(form)
  }

  # If form is a formula or expression, we can't evaluate it without the original data
  # This is a limitation of this standalone implementation
  stop("Form evaluation not supported in this implementation. Please provide predictor values directly as a numeric vector or factor.")
}

# Helper function to check dots (simplified version)
#' @keywords internal
#' @noRd
checkDots <- function(name, default, ...) {
  dots <- list(...)
  if (name %in% names(dots)) {
    return(dots[[name]])
  } else {
    return(default)
  }
}


## adapted from influence.ME to work with binomial glmer and avoid the need to declare global variables
## https://github.com/cran/influence.ME
#' @keywords internal
#' @noRd
influence2 <-
  function(model, family_p, group=NULL, select=NULL, obs=FALSE, gf="single", count = FALSE, delete=TRUE, ...)
  {

    fixef <- NA
    rm(fixef)

    ## Checks, errors, and warnings
    # obs=TRUE cannot be used with delete=FALSE, group, gf,  parameters

    if(is.null(group) & !obs)
    {
      stop("Please specify either the 'group' parameter, or specify 'obs=TRUE'")
    }

    if(!is.null(group) & obs)
    {
      stop("Either specify the 'group' parameter, or specify 'obs=TRUE', but not both.")
    }


    # Defining Internal Variables
    # Thanks to Kevin Darras for suggestion on how to extend functionality to binomial models and functions inside the model call

    ifelse(as.character(model@call)[3]=="data.update",
           data.adapted <- model.frame(model),
           data.adapted <- get(as.character(model@call)[3]))


    if("glmerMod" %in% class(model)) {
      if(summary(model)$family == "binomial") {
        success_failures <- data.adapted[,1]
        data.adapted[,1] <- success_failures[,1]
        colnames(data.adapted)[1] <- "response_column"
        data.adapted[,(ncol(data.adapted)+1)] <- success_failures[,1] + success_failures[,2]
        colnames(data.adapted)[ncol(data.adapted)] <- "total_column"
      }
      if((grepl("Negative",summary(model)$family) | grepl("poisson",summary(model)$family)) & any(grepl("offset", colnames(data.adapted)))) {
          offset_col <- grep("offset", colnames(data.adapted))
          data.adapted[, offset_col] <- exp(data.adapted[, offset_col])
          colnames(data.adapted)[offset_col] <- "total_column"
      }
      if(!grepl("Negative",summary(model)$family))
        family_p=switch(family_p,
                        "poisson" = poisson(link="log"),
                        "binomial" = binomial(link="logit"),
                        "bionomial_log" = binomial(link="log"),
                        "Gamma_log" = Gamma(link = "log"),
                        "Gamma" = Gamma(link = "inverse"))



    }


    original.no.estex <- which(substr(names(fixef(model)), 1,6) != "estex.")
    n.pred <- length(fixef(model)[original.no.estex])


    ####
    # Code kindly provided by Jennifer Bufford
    if("(weights)" %in% names(data.adapted)) {
      names(data.adapted)[names(data.adapted)=="(weights)"] <-
        as.character(model@call$weights)}
    if("(offset)" %in% names(data.adapted)) {
      names(data.adapted)[names(data.adapted)=="(offset)"] <-
        as.character(model@call$offset)}
    if(sum(grepl("offset", names(data.adapted)))>0) {
      names(data.adapted)[grep("offset", names(data.adapted))] <-
        gsub('offset\\(|\\)',"",names(data.adapted)[grep("offset", names(data.adapted))])}
    ####



    if(!obs)
    {
      grouping.names <- influence.ME::grouping.levels(model, group)
      n.groups <- length(grouping.names)
    }

    if(obs)
    {
      n.obs <- nrow(data.adapted)
    }

    ###
    # Defining and naming the output elements for the original models
    ###
    # These apply to all
    ###

    # Fixed Estimates of the original model
    or.fixed <- matrix(ncol = n.pred , nrow = 1, data = fixef(model)[original.no.estex])
    dimnames(or.fixed) <- list(NULL, names(fixef(model))[original.no.estex])

    # Standard Error of the original model
    or.se <- matrix(ncol = n.pred , nrow = 1, data = influence.ME::se.fixef(model)[original.no.estex])
    dimnames(or.se) <- list(NULL, names(fixef(model))[original.no.estex])

    # Variance / Covariance Matrix of the original model
    or.vcov <- as.matrix(vcov(model)[original.no.estex, original.no.estex])
    dimnames(or.vcov) <- list(
      names(fixef(model)[original.no.estex]),
      names(fixef(model)[original.no.estex]))

    # Test statistic of the original model
    or.test <- coef(summary(model))[original.no.estex,3]


    ###
    # Defining and naming the output elements for the adapted models
    ###
    # These procedures vary
    ###


    if(!obs)
    {

      if(is.null(select))
      {

        # Fixed Estimates of the modified model(s)
        alt.fixed <- matrix(ncol = n.pred, nrow = n.groups, data = NA)
        dimnames(alt.fixed) <- list(grouping.names, names(fixef(model))[original.no.estex])

        # Standard Error of the modified model(s)
        alt.se <- matrix(ncol = n.pred , nrow = n.groups, data = NA)
        dimnames(alt.se) <- list(grouping.names, names(fixef(model))[original.no.estex])

        # Variance / Covariance Matrix of the modified model(s)
        alt.vcov <- list()

        # Test statistic of the modified model(s)
        alt.test <- matrix(ncol = n.pred , nrow = n.groups, data = NA)
        dimnames(alt.test) <- list(grouping.names, names(fixef(model))[original.no.estex])

        for (i in 1:n.groups)
        {

          if(count == TRUE) {print(n.groups + 1 - i)}

          model.updated <- exclude.influence2(model=model, family_p, grouping=group, level=grouping.names[i], gf=gf, delete=delete)

          altered.no.estex <- which(substr(names(fixef(model.updated)), 1,6) != "estex.")

          alt.fixed[i,] 	<- as.matrix(fixef(model.updated)[altered.no.estex])
          alt.se[i,] 		<- as.matrix(influence.ME::se.fixef(model.updated)[altered.no.estex])
          alt.vcov[[i]] 	<- as.matrix(vcov(model.updated)[altered.no.estex, altered.no.estex])
          alt.test[i,] 		<- as.matrix(coef(summary(model.updated))[,3][altered.no.estex])
        }



      }

      if(!is.null(select))
      {

        model.updated <- exclude.influence2(model, family_p, group, select, gf=gf, delete=delete)
        altered.no.estex <- which(substr(names(fixef(model.updated)), 1,6) != "estex.")

        # Fixed Estimates of the modified model(s)
        alt.fixed <- matrix(ncol = n.pred, nrow = 1, data = fixef(model.updated)[altered.no.estex])
        dimnames(alt.fixed) <- list(
          "Altered model",
          names(fixef(model.updated))[altered.no.estex])

        # Standard Error of the modified model(s)
        alt.se <- matrix(ncol = n.pred , nrow = 1, data = influence.ME::se.fixef(model.updated)[altered.no.estex])
        dimnames(alt.se) <- list("Altered model", names(fixef(model.updated))[altered.no.estex])

        # Variance / Covariance Matrix of the modified model(s)
        alt.vcov <- list()
        alt.vcov[[1]] <- as.matrix(vcov(model.updated)[altered.no.estex, altered.no.estex])
        dimnames(alt.vcov[[1]]) <- list(
          names(fixef(model.updated)[altered.no.estex]),
          names(fixef(model.updated)[altered.no.estex]))

        # Test statistic of the modified model(s)
        alt.test <- matrix(ncol = n.pred , nrow = 1, data = coef(summary(model.updated))[,3][altered.no.estex])
        dimnames(alt.test) <- list("Altered model", names(fixef(model.updated))[altered.no.estex])

      }
    }


    if(obs)
    {

      if(is.null(select))
      {
        # Fixed Estimates of the modified model(s)
        alt.fixed <- matrix(ncol = n.pred, nrow = n.obs, data = NA)
        dimnames(alt.fixed) <- list(1:n.obs, names(fixef(model))[original.no.estex])

        # Standard Error of the modified model(s)
        alt.se <- matrix(ncol = n.pred , nrow = n.obs, data = NA)
        dimnames(alt.se) <- list(1:n.obs, names(fixef(model))[original.no.estex])

        # Variance / Covariance Matrix of the modified model(s)
        alt.vcov <- list()

        # Test statistic of the modified model(s)
        alt.test <- matrix(ncol = n.pred , nrow = n.obs, data = NA)
        dimnames(alt.test) <- list(1:n.obs, names(fixef(model))[original.no.estex])

        for (i in 1:n.obs)
        {

          if(count == TRUE) {print(n.obs + 1 - i)}

          model.updated <- exclude.influence2(model, family_p, obs=i)
          altered.no.estex <- which(substr(names(fixef(model.updated)), 1,6) != "estex.")

          alt.fixed[i,] 	<- as.matrix(fixef(model.updated)[altered.no.estex])
          alt.se[i,] 		<- as.matrix(influence.ME::se.fixef(model.updated)[altered.no.estex])
          alt.vcov[[i]] 	<- as.matrix(vcov(model.updated)[altered.no.estex, altered.no.estex])
          alt.test[i,]	 	<- as.matrix(coef(summary(model.updated))[,3][altered.no.estex])
        }


      }

      if(!is.null(select))
      {
        model.updated <- exclude.influence2(model, family_p, obs=select)
        altered.no.estex <- which(substr(names(fixef(model.updated)), 1,6) != "estex.")

        # Fixed Estimates of the modified model(s)
        alt.fixed <- matrix(ncol = n.pred, nrow = 1, data = fixef(model.updated)[altered.no.estex])
        dimnames(alt.fixed) <- list(
          "Altered model",
          names(fixef(model.updated))[altered.no.estex])

        # Standard Error of the modified model(s)
        alt.se <- matrix(ncol = n.pred , nrow = 1, data = influence.ME::se.fixef(model.updated)[altered.no.estex])
        dimnames(alt.se) <- list("Altered model", names(fixef(model.updated))[altered.no.estex])

        # Variance / Covariance Matrix of the modified model(s)
        alt.vcov <- list()
        alt.vcov[[1]] <- as.matrix(vcov(model.updated)[altered.no.estex, altered.no.estex])
        dimnames(alt.vcov[[1]]) <- list(
          names(fixef(model.updated)[altered.no.estex]),
          names(fixef(model.updated)[altered.no.estex]))

        # Test statistic of the modified model(s)
        alt.test <- matrix(ncol = n.pred , nrow = 1, data = coef(summary(model.updated))[,3][altered.no.estex])
        dimnames(alt.test) <- list("Altered model", names(fixef(model.updated))[altered.no.estex])

      }



    }


    estex <- list(
      or.fixed = or.fixed,
      or.se = or.se,
      or.vcov = or.vcov,
      or.test = or.test,
      alt.fixed = alt.fixed,
      alt.se = alt.se,
      alt.vcov = alt.vcov,
      alt.test = alt.test)

    class(estex) <- "estex"
    return(estex)

  }

## adapted from influence.ME to work with binomial glmer and avoid the need to declare global variables
## https://github.com/cran/influence.ME
#' @keywords internal
#' @noRd
exclude.influence2 <-
  function(model, family_p, grouping=NULL, level=NULL, obs=NULL, gf="single", delete=TRUE)
  {

    # Thanks to Kevin Darras for suggestion on how to extend functionality to binomial models and functions inside the model call

    ifelse(as.character(model@call)[3]=="data.update",
           data.adapted <- model.frame(model),
           data.adapted <- get(as.character(model@call)[3]))

    if("glmerMod" %in% class(model)) {
      if(summary(model)$family == "binomial") {
        success_failures <- data.adapted[,1]
        data.adapted[,1] <- success_failures[,1]
        colnames(data.adapted)[1] <- "response_column"
        data.adapted[,(ncol(data.adapted)+1)] <- success_failures[,1] + success_failures[,2]
        colnames(data.adapted)[ncol(data.adapted)] <- "total_column"
      }
      if((grepl("Negative",summary(model)$family) | grepl("poisson",summary(model)$family)) & any(grepl("offset", colnames(data.adapted)))) {
        offset_col <- grep("offset", colnames(data.adapted))
        data.adapted[, offset_col] <- exp(data.adapted[, offset_col])
        colnames(data.adapted)[offset_col] <- "total_column"
      }


    }

    added.variables <- character()
    ranef <- NA
    rm(ranef)

    ####
    # Code kindly provided by Jennifer Bufford
    if("(weights)" %in% names(data.adapted)) {
      names(data.adapted)[names(data.adapted)=="(weights)"] <-
        as.character(model@call$weights)}
    if("(offset)" %in% names(data.adapted)) {
      names(data.adapted)[names(data.adapted)=="(offset)"] <-
        as.character(model@call$offset)}
    if(sum(grepl("offset", names(data.adapted)))>0) {
      names(data.adapted)[grep("offset", names(data.adapted))] <-
        gsub('offset\\(|\\)',"",names(data.adapted)[grep("offset", names(data.adapted))])}
    ####


    if(!is.null(obs))
    {

      if(!is.null(grouping) | !is.null(level))
      {
        warning("Specification of the 'obs' parameter overrules specification of the 'grouping' and 'level' parameters.")
      }

      data.adapted <- data.adapted[-obs,]
      # For some reason, update() can not find the data.adapted within the function without the line below.
      data.update <- data.adapted
      # It can find data.update

      model.updated <- update(model, data=data.update)
      return(model.updated)
    }






    if(delete==TRUE)
    {

      ## Only works when length(level) == 1, this needs to be enhanced
      group.var <- which(names(data.adapted) == grouping)

      for (i in 1:length(level))
      {
        data.adapted <- subset(data.adapted, data.adapted[,group.var]!=level[i])
      }

      # For some reason, update() can not find the data.adapted within the function without the line below.
      data.update <- data.adapted
      # It can find data.update

      model.updated <- update(model, data=data.update)

      return(model.updated)

    }






    if(names(data.adapted)[2] != "intercept.alt")
    {

      data.adapted$intercept.alt <- ifelse(model@flist[[grouping]]==level[1], 0, 1)

      data.adapted[, ncol(data.adapted)+1] <-
        ifelse(model@flist[[grouping]]==level[1], 1, 0)

      added.variables <- make.names(paste("estex.", as.character(level[1]), sep=""))
      colnames(data.adapted)[ncol(data.adapted)] <- added.variables


      if(length(level) > 1)
      {
        for (i in 2:length(level))
        {

          data.adapted$intercept.alt[model@flist[[grouping]]==level[i]] <- 0

          data.adapted[, ncol(data.adapted)+1] <-
            ifelse(model@flist[[grouping]]==level[i], 1, 0)

          added.variables <- append(added.variables, values = make.names(paste("estex.", as.character(level[i]), sep="")))

          colnames(data.adapted)[ncol(data.adapted)] <- added.variables[length(added.variables)]

        }
      }

      if(gf=="single")
      {
        # grnr refers to "grouping number"
        grnr <- which(names(ranef(model))==grouping)

        if (length(names(ranef(model)[[grnr]])) == 1)
        {
          model.updated <- update(model,
                                  formula = as.formula(paste(". ~ 0 + intercept.alt +",
                                                             paste(added.variables, collapse="+"),
                                                             "+ .",
                                                             "- (1 |", grouping, ") + (0 + intercept.alt |", grouping, ")")),
                                  data = data.adapted)

        }

        if (length(names(ranef(model)[[grnr]])) > 1)
        {
          model.updated <- update(model,
                                  formula = as.formula(paste(". ~ 0 + intercept.alt + ",
                                                             paste(added.variables, collapse="+"),
                                                             " + .",
                                                             paste(" - (", paste(names(ranef(model)[[grnr]])[-1], collapse="+"), "|", grouping, ")"),
                                                             " + (0 + intercept.alt +", paste(names(ranef(model)[[grnr]])[-1], collapse="+"), "|", grouping, ")")),
                                  data = data.adapted)
        }
      }

      if(gf=="all")
      {
        delete.gf <- vector()
        for (i in 1:length(ranef(model)))
        {
          if(length(names(ranef(model)[[i]])) > 1)
          {
            delete.gf[i] <- paste(
              "- (",
              paste(names(ranef(model)[[i]][-1]), collapse="+"),
              "|",
              names(ranef(model))[i],
              ")")
          }

          if(length(names(ranef(model)[[i]])) == 1)
          {
            delete.gf[i] <- paste(
              "- ( 1 |",
              names(ranef(model))[i],
              ")")
          }
        }
        delete.gf <- paste(delete.gf, collapse=" ")

        new.gf <- vector()
        for (i in 1:length(ranef(model)))
        {
          if(length(names(ranef(model)[[i]])) > 1)
          {
            new.gf[i] <- paste(
              "+ (0 + intercept.alt +",
              paste(names(ranef(model)[[i]][-1]), collapse="+"),
              "|",
              names(ranef(model))[i],
              ")")
          }

          if(length(names(ranef(model)[[i]])) == 1)
          {
            new.gf[i] <- paste(
              "+ (0 + intercept.alt |",
              names(ranef(model))[i],
              ")")
          }
        }
        new.gf <- paste(new.gf, collapse=" ")

        model.updated <- update(model,
                                formula = as.formula(
                                  paste(
                                    ". ~ 0 + intercept.alt + ",
                                    paste(added.variables, collapse="+"),
                                    "+ . ",
                                    delete.gf,
                                    new.gf)),
                                data=data.adapted)

      }
    }


    if(names(data.adapted)[2] == "intercept.alt")
    {

      for (i in 1:length(level))
      {
        data.adapted$intercept.alt[model@flist[[grouping]]==level[i]] <- 0

        data.adapted[, ncol(data.adapted)+1] <-
          ifelse(model@flist[[grouping]]==level[i], 1, 0)

        added.variables <- append(added.variables, values = make.names(paste("estex.", as.character(level[i]), sep="")))

        colnames(data.adapted)[ncol(data.adapted)] <- added.variables[length(added.variables)]
      }

      model.updated <- update(model,
                              formula = as.formula(paste(
                                ". ~ 0 + intercept.alt + ",
                                paste(added.variables, collapse="+"),
                                "+ .")),
                              data = data.adapted)
    }

    return(model.updated)

  }

#' @title calculatePower
#' @description This functions makes statistical power estimates given the data, the underlying design for it and the assumed probability model of the error distribution
#'
#'
#' @param data Input data frame with columns having all the necessary information regarding the dependent and independent variables of interest
#' @param design an object of class RMeDesign with the necessary design information about the data
#' @param model an object of class ProbabilityModel giving the error distribution of the data
#' @param power_param an object of class PowerParams giving the target parameter of interest and the other necessary parameter to perform the power estimation
#'
#' @return A power curve as a ggplot object or a power calculation result printed in a text file
#'
#' @export
#'
#' @examples
#' \donttest{
#' template_dir <- system.file("input_templates/cell_assay_data", package = "RMeDPower2")
#' data <- plate_assay_pilot_data
#' design <- readDesign(file.path(template_dir,"design_cell_assay.json"))
#' model <- readProbabilityModel(file.path(template_dir,"prob_model.json"))
#' power_param <- readPowerParams(file.path(template_dir,"power_param.json"))
#' power_res <- calculatePower(data, design, model, power_param)
#' }

calculatePower <- function(data, design, model, power_param) {
  calculate_power(data = data,
                            condition_column = design@condition_column,
                            experimental_columns = design@experimental_columns,
                            response_column = design@response_column,
                            total_column = design@total_column,
                            target_columns = power_param@target_columns,
                            power_curve = power_param@power_curve,
                            condition_is_categorical = design@condition_is_categorical,
                            covariate=design@covariate,
                            crossed_columns = design@crossed_columns,
                            error_is_non_normal=model@error_is_non_normal,
                            nsimn=power_param@nsimn,
                            family_p=model@family_p,
                            levels=power_param@levels,
                            max_size=power_param@max_size,
                            breaks=power_param@breaks,
                            effect_size=power_param@effect_size,
                            ICC=power_param@icc,
                            na.action=design@na_action,
                            output=NULL,
                            alpha = power_param@alpha,
                            include_interaction = design@include_interaction,
                            random_slope_variable = design@random_slope_variable,
                            covariate_is_categorical = design@covariate_is_categorical
                  )

}

# Tests for input validation via public API

test_that("diagnoseDataModel catches missing condition column", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  design@condition_column <- "nonexistent_column"
  model <- create_test_model("normal")

  expect_error(suppressWarnings(suppressMessages(
    diagnoseDataModel(data = test_data, design = design, model = model)
  )))
})

test_that("diagnoseDataModel catches missing response column", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  design@response_column <- "nonexistent_response"
  model <- create_test_model("normal")

  expect_error(suppressWarnings(suppressMessages(
    diagnoseDataModel(data = test_data, design = design, model = model)
  )))
})

test_that("diagnoseDataModel catches missing experimental columns", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  design@experimental_columns <- c("experiment", "nonexistent_exp")
  model <- create_test_model("normal")

  expect_error(suppressWarnings(suppressMessages(
    diagnoseDataModel(data = test_data, design = design, model = model)
  )))
})

test_that("diagnoseDataModel catches missing covariate column", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  design@covariate <- "nonexistent_covariate"
  model <- create_test_model("normal")

  expect_error(suppressWarnings(suppressMessages(
    diagnoseDataModel(data = test_data, design = design, model = model)
  )))
})

test_that("diagnoseDataModel catches invalid crossed columns", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  design@crossed_columns <- "nonexistent_crossed"
  model <- create_test_model("normal")

  expect_error(suppressWarnings(suppressMessages(
    diagnoseDataModel(data = test_data, design = design, model = model)
  )))
})

test_that("diagnoseDataModel catches interaction without covariate", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  design@include_interaction <- TRUE
  model <- create_test_model("normal")

  expect_error(suppressWarnings(suppressMessages(
    diagnoseDataModel(data = test_data, design = design, model = model)
  )))
})

test_that("calculatePower catches missing condition column", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  design@condition_column <- "nonexistent_column"
  model <- create_test_model("normal")
  power_params <- create_test_power_params("basic")

  expect_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

test_that("calculatePower catches invalid nsimn", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  model <- create_test_model("normal")
  power_params <- create_test_power_params("basic")
  power_params@nsimn <- -10

  expect_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

test_that("calculatePower catches invalid levels", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  model <- create_test_model("normal")
  power_params <- create_test_power_params("basic")
  power_params@levels <- 2

  expect_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

test_that("calculatePower catches invalid power_curve", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  model <- create_test_model("normal")
  power_params <- create_test_power_params("basic")
  power_params@power_curve <- 2

  expect_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

test_that("calculatePower catches invalid target columns", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  model <- create_test_model("normal")
  power_params <- create_test_power_params("basic")
  power_params@target_columns <- "invalid_target"

  expect_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

# Tests for power calculation via calculatePower public API

test_that("calculatePower wrapper function works correctly", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  model <- create_test_model("normal")
  power_params <- create_test_power_params("basic")

  expect_no_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

test_that("calculatePower handles different target columns", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  model <- create_test_model("normal")
  power_params <- create_test_power_params("basic")
  power_params@target_columns <- "plate"

  expect_no_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

test_that("calculatePower handles covariates", {
  test_data <- create_test_data()
  design <- create_test_design("with_covariate")
  model <- create_test_model("normal")
  power_params <- create_test_power_params("basic")

  expect_no_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

test_that("calculatePower handles interactions", {
  test_data <- create_test_data()
  design <- create_test_design("with_covariate")
  design@include_interaction <- TRUE
  model <- create_test_model("normal")
  power_params <- create_test_power_params("basic")

  expect_no_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

test_that("calculatePower handles non-normal distributions", {
  test_data <- create_count_data()
  design <- create_test_design("count_data")
  model <- create_test_model("poisson")
  power_params <- create_test_power_params("basic")

  expect_no_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

test_that("calculatePower handles binomial data", {
  test_data <- create_count_data()
  design <- create_test_design("count_data")
  model <- create_test_model("binomial")
  power_params <- create_test_power_params("basic")

  expect_no_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

test_that("calculatePower handles multiple target columns", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  model <- create_test_model("normal")
  power_params <- create_test_power_params("multiple_targets")

  expect_no_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

test_that("calculatePower handles levels = 0 (within-group sample size)", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  model <- create_test_model("normal")
  power_params <- create_test_power_params("basic")
  power_params@levels <- 0

  expect_no_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

test_that("calculatePower handles power_curve = 0 (single calculation)", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  model <- create_test_model("normal")
  power_params <- create_test_power_params("basic")
  power_params@power_curve <- 0

  expect_no_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

test_that("calculatePower handles effect size specification", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  model <- create_test_model("normal")
  power_params <- create_test_power_params("basic")
  power_params@effect_size <- c(0.5, NA, NA)

  expect_no_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

test_that("calculatePower handles max_size parameter", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  model <- create_test_model("normal")
  power_params <- create_test_power_params("basic")
  power_params@max_size <- 5

  expect_no_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

test_that("calculatePower handles random slopes", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  design@condition_column <- "age"
  design@condition_is_categorical <- FALSE
  design@random_slope_variable <- "age"
  model <- create_test_model("normal")
  power_params <- create_test_power_params("basic")

  expect_no_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

test_that("calculatePower handles crossed experimental factors", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  design@crossed_columns <- "plate"
  model <- create_test_model("normal")
  power_params <- create_test_power_params("basic")

  expect_no_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

test_that("calculatePower integrates S4 classes correctly", {
  test_data <- create_test_data()

  design <- new("RMeDesign",
    response_column = "cell_size",
    condition_column = "treatment",
    condition_is_categorical = TRUE,
    experimental_columns = c("experiment", "plate"),
    covariate = "age",
    covariate_is_categorical = FALSE,
    crossed_columns = NULL,
    include_interaction = FALSE,
    random_slope_variable = NULL,
    total_column = NULL,
    na_action = "complete"
  )

  model <- new("ProbabilityModel",
    error_is_non_normal = FALSE,
    family_p = NULL
  )

  power_params <- new("PowerParams",
    target_columns = "experiment",
    levels = 1,
    power_curve = 1,
    nsimn = 5,
    alpha = 0.05,
    max_size = NULL,
    breaks = NULL,
    effect_size = NULL,
    icc = NULL
  )

  expect_no_error(suppressWarnings(suppressMessages(
    calculatePower(data = test_data, design = design, model = model, power_param = power_params)
  )))
})

#' @title readDesign
#'
#' @description This functions reads the underlying design for the data
#'
#'
#' @param jsonfile the jsonfile with the necessary design parameters: condition_column, experimental_columns, response_column, total_column, condition_is_categorical, covariate, method, crossed_columns, error_is_non_normal, family_p, outlier_alpha, na.action
#'
#' @return an object of class RMeDesign
#' @export
#'
#' @examples
#' template_dir <- system.file("input_templates/cell_assay_data", package = "RMeDPower2")
#' design <- readDesign(file.path(template_dir,"design_cell_assay.json"))

readDesign <- function(jsonfile) {
  design_data <- jsonlite::fromJSON(jsonfile)

  design <- new("RMeDesign")

  design@response_column = design_data$response_column

  design@condition_column = design_data$condition_column
  design@condition_is_categorical = design_data$condition_is_categorical
  design@experimental_columns = design_data$experimental_columns
  if("crossed_columns" %in% names(design_data) && !is.null(design_data$crossed_columns)) design@crossed_columns = design_data$crossed_columns
  if("total_column" %in% names(design_data) && isTRUE(nchar(design_data$total_column) > 0)) design@total_column = design_data$total_column
  if("covariate" %in% names(design_data) && isTRUE(nchar(design_data$covariate) > 0)) design@covariate = design_data$covariate
  if("covariate_is_categorical" %in% names(design_data)) design@covariate_is_categorical = design_data$covariate_is_categorical
  if("include_interaction" %in% names(design_data)) design@include_interaction = design_data$include_interaction
  if("random_slope_variable" %in% names(design_data) && isTRUE(nchar(design_data$random_slope_variable) > 0)) design@random_slope_variable = design_data$random_slope_variable
  if("outlier_alpha" %in% names(design_data) && is.numeric(design_data$outlier_alpha)) design@outlier_alpha = design_data$outlier_alpha
  return(design)
}

#' @title getEstimatesOfInterest
#'
#' @description This function performs the estimations of interest and also visualizes the resulting association
#'
#'
#' @param data Input data frame with columns having all the necessary information regarding the dependent and independent variables of interest
#' @param design an object of class RMeDesign with the necessary design information about the data
#' @param model an object of class ProbabilityModel giving the error distribution of the data
#' @param print_plots Whether or not to print the plots, irrespective of this argument ggplot versions of evaluated association between the response_column and the condition_column. TRUE - print the plot, FALSE - do not print the plot
#'
#' @return a list with two elements - 1. an object of class summary.merMod and
#' 2. the output from the get_residuals functions. This output consists of a list
#' with 3 elements. 1. The updated input data with an additional column with the model residuals of the individual observations. 2. A plot representing the purported association between the response column and the condition column. 3. The corresponding caption for this figure.
#'
#' @export
#'
#' @examples
#' \donttest{
#' template_dir <- system.file("input_templates/cell_assay_data", package = "RMeDPower2")
#' data <- plate_assay_pilot_data
#' design <- readDesign(file.path(template_dir,"design_cell_assay.json"))
#' model <- readProbabilityModel(file.path(template_dir,"prob_model.json"))
#' res <- getEstimatesOfInterest(data, design, model)
#' }

getEstimatesOfInterest <- function(data, design, model,print_plots=TRUE) {

  ##assign input arguments
  condition_column = design@condition_column
  experimental_columns = design@experimental_columns
  response_column = design@response_column
  total_column = design@total_column
  condition_is_categorical = design@condition_is_categorical
  covariate= design@covariate
  crossed_columns = design@crossed_columns
  error_is_non_normal=model@error_is_non_normal
  family_p=model@family_p
  alpha=design@outlier_alpha
  na.action=design@na_action

  # New enhanced parameters
  include_interaction = design@include_interaction
  random_slope_variable = design@random_slope_variable
  covariate_is_categorical = design@covariate_is_categorical

  # cat("\n=== Enhanced Estimates of Interest Analysis ===\n")
  # cat("Model specifications:\n")
  # cat("- Interaction effects:", ifelse(include_interaction, "Yes", "No"), "\n")
  # cat("- Random slopes:", ifelse(!is.null(random_slope_variable), "Yes", "No"), "\n")

  res <- list()
  ##get estimates
  res[[1]] <- calculate_lmer_estimates(data, condition_column, experimental_columns, response_column, total_column, condition_is_categorical, covariate,
                                                 crossed_columns, error_is_non_normal, family_p, na.action, include_interaction, random_slope_variable, covariate_is_categorical)
  ##visualize estimates
  res[[2]] <- get_residuals(data, condition_column, experimental_columns, response_column, condition_is_categorical, covariate,
                                                crossed_columns, total_column, error_is_non_normal, family_p, na.action, include_interaction, random_slope_variable, covariate_is_categorical, print_plots)

  return(res)

}

#' @keywords internal
#' @import ggplot2 dplyr magrittr methods
#' @importFrom grDevices png dev.off
#' @importFrom graphics mtext
#' @importFrom EnvStats rosnerTest
#' @importFrom jsonlite fromJSON
#' @importFrom stats poisson binomial Gamma as.formula median qqnorm qqline mad qnorm model.frame update vcov fitted predict residuals coef
#' @importFrom influence.ME cooks.distance.estex grouping.levels se.fixef
#' @importFrom lme4 fixef ranef VarCorr glmer glmer.nb getME
#' @importFrom lmerTest lmer
#' @importFrom simr "fixef<-" extend fcompare fixed makeLmer powerCurve powerSim
#' @importFrom ggtext element_textbox_simple
#' @importFrom tibble rownames_to_column
#' @importFrom DHARMa simulateResiduals outliers
NULL

utils::globalVariables(c(
  ".", "cooks_distance", "exp.factor", "experimental_column1",
  "lower", "med_residual1", "model_residuals", "residual",
  "response_column", "sqrt_abs_residuals", "upper"
))

#' @title calculate_power_covariate
#' @description This function uses simulation to perform power analysis. It is designed to explore the power of biological experiments and to suggest an optimal number of experimental variables with reasonable power. The backbone of the function is based on simr package, which fits a fixed effect or mixed effect model based on the observed data and simulates response variables. Users can test the power of different combinations of experimental variables and parameters.
#'
#'
#'
#'
#' @param data Input data
#' @param condition_column The name of the condition variable (ex a variable with values such as control/case). The input file has to have a corresponding column name
#' @param experimental_columns Names of variables related to the experimental design, such as "experiment", "plate", and "cell_line".vThey should be in order, for example, "experiment" should always come first .
#' @param response_column The name of the variable observed by performing the experiment. ex) intensity.
#' @param total_column Set this column only when family_p="binomial" and it is equal to the total number of observations (number of cases plus number of controls) for a given number of cases
#' @param power_curve 1: Power simulation over a range of sample sizes or levels. 0: Power calculation over a single sample size or a level.
#' @param condition_is_categorical Specify whether the condition variable is categorical. TRUE: Categorical, FALSE: Continuous.
#' @param covariate The name of the covariate to control in the regression model
#' @param crossed_columns Name of experimental variables that may appear repeatedly with the same ID. For example, cell_line C1 may appear in multiple experiments, but plate P1 cannot appear in more than one experiment
#' @param error_is_non_normal Default: Observed variable is continuous. Categorical response variable will be implemented in the future. TRUE: Categorical , FALSE: Continuous (default).
#' @param nsimn The number of simulations to run. Default=1000
#' @param family_p The type of distribution family to specify when the response is categorical. If family is "binary" then binary(link="log") is used, if family is "poisson" then poisson(link="logit") is used, if family is "poisson_log" then poisson(link=") log") is used.
#' @param target_columns Name of the experimental parameters to use for the power calculation.
#' @param levels 1: Amplify the number of corresponding target parameter. 0: Amplify the number of samples from the corresponding target parameter, ex) If target_columns = c("experiment","cell_line") and if you want to expand the number of experiment and sample more cells from each cell line, set levels = c(1,0).
#' @param max_size Maximum levels or sample sizes to test. Default: the current level or the current sample size x 5. ex) If max_levels = c(10,5), it will test upto 10 experiments and 5 cell lines.
#' @param breaks Levels /sample sizes of the variable to be specified along the power curve. Default: max(1, round( the number of current levels / 5 ))
#' @param na.action "complete": missing data is not allowed in all columns (default), "unique": missing data is not allowed only in condition, experimental, response, and target columns. Selecting "complete" removes an entire row when there is one or more missing values, which may affect the distribution of other features.
#' @param output Output file name
##### If variance estimates should be estimated from data
#' @param  effect_size A 3 dimensional numeric vector given the proposed effect sizes/parameter estimates for the condition_column, covariate and interaction term between these two variables, respectively in that order. Depending on the model not all elements of the effect_size would make sense. For example, in situations, where there is only the condition column, the second and third elements of effect_size should be equal to NA. If you know the effect size of your condition variable, the effect size can be provided as a parameter. If the effect size is not provided, it will be estimated from your data
##### If variance estimates are to be assigned by a user
#' @param  alpha Threshold for Type I error
#' @param  ICC Intra-Class Coefficients (ICC) for each parameter
#' @param include_interaction Whether to include condition * covariate interaction
#' @param random_slope_variable Variable for random slopes (typically one of "condition_column" or "covariate" and assuming that they are numeric variables). A random slope term is added for each of the variables specified in the experimental columns in addition to their corresponding random intercept terms. The random slope and intercept terms for each experimental_columns variable are assumed to be uncorrelated.
#' @param covariate_is_categorical Specify whether the covariate variable is categorical. TRUE: Categorical, FALSE: Continuous.
#'
#'
#' @return A power curve as a ggplot object or a power calculation result printed in a text file
#'
#' @keywords internal
#' @noRd



calculate_power <- function(data, condition_column, experimental_columns, response_column, total_column = NULL, target_columns, power_curve, condition_is_categorical, covariate=NULL,
                            crossed_columns = NULL, error_is_non_normal=FALSE, nsimn=1000, family_p=NULL,
                            levels=NULL, max_size=NULL, breaks=NULL, effect_size=NULL, ICC=NULL, na.action="complete", output=NULL, alpha =0.05,
                            include_interaction = NA,
                            random_slope_variable = NULL,
                            covariate_is_categorical = NA){






  ######input error handler
  if(length(levels)!=length(target_columns)) stop("User should specify levels of all target parameters")
  if(length(power_curve)==0 | !power_curve%in%c(0,1)) stop("power_curve must be 0 or 1")
  if(!condition_column %in% colnames(data)) stop("condition_column should be one of the column names")
  if(sum(experimental_columns%in%colnames(data))!=length(experimental_columns)) stop("experimental_columns must match column names")
  if(!is.null(crossed_columns)){if(sum(crossed_columns%in%colnames(data))!=length(crossed_columns)) stop("crossed_columns must match column names")}
  if(!response_column%in%colnames(data)) stop("response_column should be one of the column names")

  if(!is.na(condition_is_categorical) && !condition_is_categorical%in%c(TRUE,FALSE)) stop("condition_is_categorical must be TRUE or FALSE")
  if(!is.na(covariate_is_categorical) && !covariate_is_categorical%in%c(TRUE,FALSE)) stop("covariate_is_categorical must be TRUE or FALSE")

  if(!is.null(covariate)) {
    if(!covariate%in%colnames(data))
      stop("covariate should be NA or one of the column names")}
  if(! (is.numeric(nsimn)&&nsimn>0)) stop("nsimn should be a positive integer")
  if(sum(target_columns%in%colnames(data))!=length(target_columns)) stop("target_columns must match column names")
  if(sum(levels%in%c(0,1))!=length(levels)) stop("levels must be 0 or 1")
  if(!( is.null(max_size) | (is.numeric(max_size)&&sum(max_size>0)==length(max_size)) )) stop("max_size a positive integer")
  if(!( is.null(breaks) | (is.numeric(breaks)&&breaks>0) )) stop("breaks must be a positive integer")
  if(!(is.null(effect_size) | (is.numeric(effect_size) & length(effect_size) == 3))) stop("effect_size must be a 3 element numeric vector")
  if(!is.null(ICC) & error_is_non_normal==TRUE) stop("ICC-based simulations are not supported when the response is non-normal")
  if(!is.null(ICC)){if(length(ICC) != length(experimental_columns)) stop("The ICC vector should be of the same dimension as the number of experimental columns")}
  # Validation for new parameters
  if(!is.na(include_interaction)) {
    if (include_interaction && is.null(covariate)) {
      stop("Cannot include interaction when covariate is NULL")
    }
  }

  if (!is.null(random_slope_variable) &&
      !random_slope_variable %in% c("condition_column", condition_column, "covariate", covariate)) {
    stop("random_slope_variable should be 'condition_column', 'covariate' or the actual condition column name or covariate column name")
  }

  if(error_is_non_normal==TRUE){
    if(family_p != "negative_binomial")
      family_p=switch(family_p,
                      "poisson" = poisson(link="log"),
                      "binomial" = binomial(link="logit"),
                      "bionomial_log" = binomial(link="log"),
                      "Gamma_log" = Gamma(link = "log"),
                      "Gamma" = Gamma(link = "inverse"))
    else{
      family_p = list(family = "negative_binomial")
    }
  }


  if(na.action=="complete"){

    notNAindex=which( rowSums(is.na(data)) == 0 )

  }else if(na.action=="unique"){

    if(!is.null(covariate)) notNAindex=which( rowSums(is.na(data[,c(condition_column, experimental_columns, response_column, covariate)])) == 0 )
    else notNAindex=which( rowSums(is.na(data[,c(condition_column, experimental_columns, response_column)])) == 0 )


  }

  Data <- data[notNAindex,]


  # cat("\n")
  # print("__________________________________________________________________Summary of data:")
  # print(summary(Data))
  # cat("\n")

  colnames_original=colnames(Data)
  experimental_columns_index=NULL
  ####### assign categorical variables
  if(condition_is_categorical==TRUE) Data[,condition_column]=as.factor(Data[,condition_column])

  if(!is.null(covariate)) {
    if(covariate_is_categorical==TRUE)
      Data[,covariate]=as.factor(Data[,covariate])
    else
      Data[,covariate]=as.numeric(Data[,covariate])
  }

  # random slope should be allowed only with a continuous variable
  if(!is.null(random_slope_variable)) {
    if(!inherits(Data[,random_slope_variable], "numeric")) {
      stop("random_slope_variable should be a numeric variable")
    }
  }


  noncrossed_columns=NULL

  for(i in 1:length(experimental_columns)){
    Data[,experimental_columns[i]]=as.factor(Data[,experimental_columns[i]])
    experimental_columns_index=c(experimental_columns_index,which(colnames(Data)==experimental_columns[i]))
    colnames(Data)[experimental_columns_index[i]]=paste("experimental_column",i,sep="")

    if(i!=1&&!experimental_columns[i]%in%crossed_columns){
      noncrossed_columns=c(noncrossed_columns, paste("experimental_column",i,sep=""))
    }


    # cat("\n")
    # print(paste("_________________________________",experimental_columns[i]," is assigned to experimental_column",i,sep=""))
    # cat("\n")
  }

  if(length(experimental_columns)>=2){
    for(r in 2:length(experimental_columns)){
      if(colnames(Data)[experimental_columns_index[r]]%in%noncrossed_columns){
        Data[,experimental_columns_index[r]]=paste(Data[,experimental_columns_index[r-1]], Data[,experimental_columns_index[r]],sep="_")
        Data[,experimental_columns_index[r]]=as.factor(Data[,experimental_columns_index[r]])
      }
    }

  }

  colnames(Data)[which(colnames(Data)==condition_column)]="condition_column"
  colnames(Data)[which(colnames(Data)==response_column)]="response_column"
  if(!is.null(covariate)) colnames(Data)[which(colnames(Data)==covariate)]="covariate"

  if(!is.null(random_slope_variable)) {
    if(random_slope_variable == condition_column)
      random_slope_variable = "condition_column"
    else if(random_slope_variable == covariate)
      random_slope_variable = "covariate"
    else{
      stop("random_slope_variable can only be 'condition_column', 'covariate' or the actual condition column name or covariate column name")
    }

  }

  if(!is.null(total_column))
    colnames(Data)[which(colnames(Data)==total_column)]="total_column"


  ###### indices of target parameters in experimental variables
  target_i=NULL
  target_columns_renamed=NULL
  ###### match target parameters
  for(i in 1:length(target_columns)){
    cn=colnames(Data)[which(colnames_original==target_columns[i])]
    target_columns_renamed[i]=cn

    target_i=c(target_i,as.integer(substr(cn,nchar(cn),nchar(cn))))
  }


  multiple_levels_for_each_random_effect <- TRUE
  ####### Check to see if there is only one category
  for(i in 1:length(experimental_columns)){
    if(length(table(Data[,experimental_columns_index[i]]))==1 && multiple_levels_for_each_random_effect){
      multiple_levels_for_each_random_effect <- FALSE
    }
  }


  ####### run the formula
  if(multiple_levels_for_each_random_effect){


    # Build formula components
    fixed_formula <- build_fixed_formula(covariate, include_interaction)
    random_formula <- build_random_formula(experimental_columns, random_slope_variable)
    # lmerFit <- generate_model_fit_4_power_estimates(data=Data,
    #                               fixed_formula,
    #                               random_formula,
    #                               error_is_non_normal,
    #                               family_p,
    #                               total_column)


    if (error_is_non_normal == FALSE) {
      # Linear mixed effects model
      formula_str <- as.formula(paste("response_column ~", fixed_formula, "+", random_formula))
      lmerFit <- lmer(formula_str, data = Data)
      ##base model fixed formula when there is an interaction term
      if(!is.null(covariate)){
        formula_str0 <- as.formula(paste0("response_column ~ covariate"))
      }
    } else if (!is.null(family_p) && family_p$family == "binomial" && !is.null(total_column)) {
      # Binomial with total column
      formula_str <- as.formula(paste("cbind(response_column, (total_column - response_column)) ~",
                                       fixed_formula, "+", random_formula))
      lmerFit <- lme4::glmer(formula_str, data = Data, family = family_p)
      ##base model fixed formula when there is an interaction term
      if(!is.null(covariate)){
        formula_str0 <- as.formula(paste0("cbind(response_column, (total_column - response_column)) ~ covariate"))
      }

    } else if (!is.null(family_p) && family_p$family == "negative_binomial" && !is.null(total_column)) {
      # Negative binomial with offset
      formula_str <- as.formula(paste("response_column ~", fixed_formula, "+", random_formula,
                                       "+ offset(log(total_column))"))

      lmerFit <- lme4::glmer.nb(formula_str, data = Data, family = family_p)
      ##base model fixed formula when there is an interaction term
      if(!is.null(covariate)){
        formula_str0 <- as.formula(paste0("response_column ~ covariate"))

      }
    } else if (!is.null(family_p) && family_p$family == "poisson" && !is.null(total_column)) {
      # Poisson with offset
      formula_str <- as.formula(paste("response_column ~", fixed_formula, "+", random_formula,
                                       "+ offset(log(total_column))"))

      lmerFit <- lme4::glmer(formula_str, data = Data, family = family_p)
      ##base model fixed formula when there is an interaction term
      if(!is.null(covariate)){
        formula_str0 <- as.formula(paste0("response_column ~ covariate"))

      }
    }
    else if (!is.null(family_p) && family_p$family == "negative_binomial" && is.null(total_column)) {
      # Negative binomial with offset
      formula_str <- as.formula(paste("response_column ~", fixed_formula, "+", random_formula))
      lmerFit <- lme4::glmer.nb(formula_str, data = Data, family = family_p)
      ##base model fixed formula when there is an interaction term
      if(!is.null(covariate)){
        formula_str0 <- as.formula(paste0("response_column ~ covariate"))
      }
    }
    else {
      # Other GLMMs
      formula_str <- as.formula(paste("response_column ~", fixed_formula, "+", random_formula))
      lmerFit <- lme4::glmer(formula_str, data = Data, family = family_p)
      ##base model fixed formula when there is an interaction term
      if(!is.null(covariate)){
        formula_str0 <- as.formula(paste0("response_column ~ covariate"))
      }
    }

  }else if(!multiple_levels_for_each_random_effect && !error_is_non_normal){

      if(is.null(covariate)){
        lmerFit=stats::lm(response_column ~ condition_column, data=Data)
      }else if(!include_interaction){
        lmerFit=stats::lm(response_column ~ condition_column + covariate, data=Data)
      }else{
        lmerFit=stats::lm(response_column ~ condition_column*covariate, data=Data)
      }




  }


  if(error_is_non_normal == FALSE & multiple_levels_for_each_random_effect){
    lmerFit <- as(lmerFit, "lmerMod")
    lmerFit@call[[2]] <- formula_str
  }
  if(error_is_non_normal == TRUE) {
    if(family_p$family != "negative_binomial")
      lmerFit@call[[4]] <- family_p
  }
  slmerFit <- summary(lmerFit)
  # cat("\n")
  # print("__________________________________________________________________Model statistics:")
  # print(slmerFit)
  # cat("\n")

  fixed_effects=slmerFit$coefficients[,1]







  ##### ICC based variance estimation
  if(length(ICC)>0 && !error_is_non_normal){

    ####### If there is only one category, add one more
    for(i in 1:length(experimental_columns)){
      if(length(table(Data[,experimental_columns_index[i]]))==1){
        categroy_temp=paste0(Data[,experimental_columns_index[i]][1],"_2")
        levels(Data[,experimental_columns_index[i]])=c(levels(Data[,experimental_columns_index[i]]),categroy_temp)
        Data[,experimental_columns_index[i]][sample(1:length(Data[,1]),round(length(Data[,1])/2))]=rep(categroy_temp,round(length(Data[,1])/2))
      }
    }


    ##estimate varEs
    a = matrix(NA, length(experimental_columns), length(experimental_columns))
    b = matrix(NA, length(experimental_columns), 1)

    for(i in 1:length(experimental_columns)) {
      for(j in 1:length(experimental_columns)) {
        a[j,i] = -ICC[j]
      }
      a[i,i] = 1 - ICC[i]
      b[i,1] = ICC[i]*(slmerFit$sigma)^2
    }
    varEs=solve(a,b)

    # Build formula components
    fixed_formula <- build_fixed_formula(covariate, include_interaction)
    random_formula <- build_random_formula(experimental_columns, random_slope_variable)
    formula_str <- as.formula(paste("response_column ~", fixed_formula, "+", random_formula))

    artificial_lmer=simr::makeLmer(formula = formula_str,
                                   data=Data,
                                   VarCorr = as.list(varEs), sigma = slmerFit$sigma,
                                   fixef=fixed_effects )
    lmerFit=artificial_lmer

    ##base model fixed formula when there is an interaction term
    if(!is.null(covariate)){
      formula_str0 <- as.formula(paste0("response_column ~ covariate"))
    }


  }





  ####### print observed levels and sample sizes
  maxs=NULL
  mins=NULL
  lens=NULL
  for(i in 1:length(experimental_columns)){
    # cat("\n")
    # print(paste("__________________________________________________________________Levels and sample sizes of",experimental_columns[i]))
    # cat("\n")
    xtabs_s=stats::xtabs(~Data[,paste0("experimental_column",i)])


    # print(xtabs_s)
    maxs=c(maxs,max(xtabs_s))
    if(xtabs_s[1]==0){
      mins=c(mins,min(xtabs_s[-1]))
    }else{
      mins=c(mins,min(xtabs_s))
    }

    lens=c(lens,length(xtabs_s))
    # cat("\n")
    # print(paste("_________________________________Max sample size:",maxs[i]))
    # print(paste("_________________________________Min sample size:",mins[i]))
    # print(paste("_________________________________Count of levels:",lens[i]))
    # cat("\n")
  }







  ##### Assign known effect sizes
  if(length(effect_size)>0){

    if(!is.na(effect_size[1])){
      fixef(lmerFit)[2] <- effect_size[1]
      message("Effect size of the condition_column is now ", effect_size[1])
    }
    if(!is.na(effect_size[2]) & !is.null(covariate)){
      fixef(lmerFit)[3] <- effect_size[2]
      message("Effect size of the covariate is now ", effect_size[2])
    }
    if(!is.null(include_interaction)) {
      if(!is.na(effect_size[3]) & include_interaction){
        fixef(lmerFit)[4] <- effect_size[3]
        message("Effect size of the interaction term is now ", effect_size[3])
      }
    }

  }




  if(power_curve==0){




    ####### extend parameter levels and sample sizes


    if(length(max_size)==0){max_size=rep(0, length(target_columns))}
    if(length(breaks)==0){
      breaks=rep(0,length(target_columns))
    }


    for(i in 1:length(target_columns)){#target_columns_renamed, levels and max_size follow user input order but lens, mins, maxs don't
      if(levels[i]==1){ # increase levels

        if(max_size[i]==0){
          max_size[i]=lens[target_i[i]]*5
        }else if(max_size[i]<lens[target_i[i]]){
          # cat("\n")
          # print(paste("_________________________________Max size is set to ",max_size[i]," which is smaller than the observed max size ",lens[target_i[i]],". The observed max size will be used instead.",sep=""))
          # cat("\n")
          max_size[i]=lens[target_i[i]]
        }
        if(breaks[i]==0) breaks[i] = max(1, round( lens[target_i[i]] / 5 ))
        if(i==1){
          extended_target_columns=simr::extend(lmerFit,along=target_columns_renamed[i],n=max_size[i])
        }else{
          extended_target_columns=simr::extend(extended_target_columns,along=target_columns_renamed[i],n=max_size[i])
        }

      }else{ # increase sample sizes
        if(max_size[i]==0){
          max_size[i]=maxs[target_i[i]]*5
        }else if(max_size[i]<maxs[target_i[i]]){
          # cat("\n")
          # print(paste("_________________________________Max size is set to ",max_size[i]," which is smaller than the observed max size ",maxs[target_i[i]],". The observed max size will be used instead.",sep=""))
          # cat("\n")
          max_size[i]=maxs[target_i[i]]
        }
        if(breaks[i]==0) breaks[i] = max(1, round( maxs[target_i[i]] / 5 ))
        if(i==1){
          extended_target_columns=simr::extend(lmerFit,within=target_columns_renamed[i],n=max_size[i])
        }else{
          extended_target_columns=simr::extend(extended_target_columns,within=target_columns_renamed[i],n=max_size[i])
        }
      }
      # cat("\n")
      # print(paste("Input max size of",target_columns[i]))
      # print(max_size[i])
      # cat("\n")

    }

    #cat("\n")
    # print("__________________________________________________________________Extended parameters:")
    # for(i in 1:length(target_columns)){
    #   if(target_columns_renamed[i]=="experimental_column1"){
    #     # print(xtabs(~experimental_column1,data=attributes(extended_target_columns)$newData))
    #     cat("\n")
    #   }
    #   if(target_columns_renamed[i]=="experimental_column2"){
    #     # print(xtabs(~experimental_column2,data=attributes(extended_target_columns)$newData))
    #     cat("\n")
    #   }
    #   if(target_columns_renamed[i]=="experimental_column3"){
    #     # print(xtabs(~experimental_column3,data=attributes(extended_target_columns)$newData))
    #     cat("\n")
    #   }
    #   if(target_columns_renamed[i]=="experimental_column4"){
    #     # print(xtabs(~experimental_column4,data=attributes(extended_target_columns)$newData))
    #     cat("\n")
    #   }
    #   if(target_columns_renamed[i]=="experimental_column5"){
    #     # print(xtabs(~experimental_column5,data=attributes(extended_target_columns)$newData))
    #     cat("\n")
    #   }
    #
    # }

    if(length(experimental_columns)>=2){
            for(r in 2:length(experimental_columns)){
        if(colnames(Data)[experimental_columns_index[r]]%in%noncrossed_columns){
          attributes(extended_target_columns)$newData[,experimental_columns_index[r]]=paste(attributes(extended_target_columns)$newData[,experimental_columns_index[r-1]],attributes(extended_target_columns)$newData[,experimental_columns_index[r]],sep="_")
        }
      }
      # print(attributes(extended_target_columns)$newData)
    }



    ###### power simulation

    if(!is.null(covariate))
      ps=simr::powerSim(extended_target_columns, test=simr::fcompare(formula_str0), nsim=nsimn, progress = FALSE)
    else
      ps=simr::powerSim(extended_target_columns, test=simr::fixed("condition_column"), nsim=nsimn, progress = FALSE)
    # cat("\n")
    # print("__________________________________________________________________Power simulation result:")
    # print(ps)
    # cat("\n")
    if(length(output)!=0){
      sink(paste0(output,".txt"))
       print(ps)
      sink()
    }else{
       print(ps)
    }







  }else{#power curve =1


    ####### extend parameter levels and sample sizes
    extended_target_columns=list()
    if(length(max_size)==0){max_size=rep(0,length(target_columns))}
    if(length(breaks)==0){
      breaks=rep(0,length(target_columns))
    }





    for(i in 1:length(target_columns)){
      if(levels[i]==1){ # increase levels
        if(max_size[i]==0){
          max_size[i]=lens[target_i[i]]*5
        }else if(max_size[i]<lens[target_i[i]]){
          # cat("\n")
          # print(paste("_________________________________Max size is set to ",max_size[i]," which is smaller than the observed max size ",lens[target_i[i]],". The observed max size will be used instead.",sep=""))
          # cat("\n")
          max_size[i]=lens[target_i[i]]
        }
        if(breaks[i]==0) breaks[i] = max(1, round( lens[target_i[i]] / 5 ))
        extended_target_columns=c(extended_target_columns,list(simr::extend(lmerFit,along=target_columns_renamed[i],n=max_size[i])))
      }else{ # increase sample sizes
        if(max_size[i]==0){
          max_size[i]=maxs[target_i[i]]*5
        }
        if(breaks[i]==0) breaks[i] = max(1, round( maxs[target_i[i]] / 5 ))
        extended_target_columns=c(extended_target_columns,list(simr::extend(lmerFit,within=target_columns_renamed[i],n=max_size[i])))
      }
      # cat("\n")
      # print(paste("_________________________________Power simulation will be performed based on the max size of",target_columns[i],":"))
      # print(max_size[i])
      # cat("\n")

    }


    # cat("\n")
    # print("__________________________________________________________________Extended parameters:")
    # for(i in 1:length(target_columns)){
    #   if(target_columns_renamed[i]=="experimental_column1"){
    #     # print(xtabs(~experimental_column1,data=attributes(extended_target_columns[[i]])$newData))
    #     cat("\n")
    #   }
    #   if(target_columns_renamed[i]=="experimental_column2"){
    #     # print(xtabs(~experimental_column2,data=attributes(extended_target_columns[[i]])$newData))
    #     cat("\n")
    #   }
    #   if(target_columns_renamed[i]=="experimental_column3"){
    #     # print(xtabs(~experimental_column3,data=attributes(extended_target_columns[[i]])$newData))
    #     cat("\n")
    #   }
    #   if(target_columns_renamed[i]=="experimental_column4"){
    #     # print(xtabs(~experimental_column4,data=attributes(extended_target_columns[[i]])$newData))
    #     cat("\n")
    #   }
    #   if(target_columns_renamed[i]=="experimental_column5"){
    #     # print(xtabs(~experimental_column5,data=attributes(extended_target_columns[[i]])$newData))
    #     cat("\n")
    #   }
    #
    # }

    if(length(experimental_columns)>=2){
            for(r in 2:length(experimental_columns)){
        if(experimental_columns[r]%in%noncrossed_columns){
          attributes(extended_target_columns)$newData[noncrossed_columns[r]]=paste(attributes(extended_target_columns)$newData[noncrossed_columns[r-1]],
                                                                                      attributes(extended_target_columns)$newData[noncrossed_columns[r]],sep="_")
        }
      }
      # print(attributes(extended_target_columns)$newData)
    }


    plots <- list()
    captions <- vector(mode = "character")
    ###### power curve simulation
    for(i in 1:length(target_columns)){

      if(levels[i]==1){

          # pc=simr::powerCurve(extended_target_columns[[i]], test=simr::fixed("condition_column"),along=target_columns_renamed[i], nsim=nsimn,
          #                   breaks=seq(1,max_size[i],breaks[i]), progress = FALSE   )
          if(!is.null(covariate))
            pc=simr::powerCurve(extended_target_columns[[i]], test=simr::fcompare(formula_str0),along=target_columns_renamed[i], nsim=nsimn, breaks=seq(1,max_size[i],breaks[i]), progress = FALSE)
          else
            pc=simr::powerCurve(extended_target_columns[[i]], test=simr::fixed("condition_column"), along=target_columns_renamed[i], nsim=nsimn, breaks=seq(1,max_size[i],breaks[i]), progress = FALSE)

      }else{

          if(!is.null(covariate))
            pc=simr::powerCurve(extended_target_columns[[i]], test=simr::fcompare(formula_str0),within=target_columns_renamed[i], nsim=nsimn, breaks=seq(1,max_size[i],breaks[i]), progress = FALSE)
          else
            pc=simr::powerCurve(extended_target_columns[[i]], test=simr::fixed("condition_column"), within=target_columns_renamed[i], nsim=nsimn, breaks=seq(1,max_size[i],breaks[i]), progress = FALSE)

      }





      if(length(output)==0){
        #plot(pc)
        plot_data <- do.call("rbind", lapply(pc$ps, summary))
        plot_data <- plot_data[,-(1:2)]
        ##convert to percentage
        plot_data <- 100*plot_data
        plot_data <- data.frame(levels = pc$nlevels, plot_data)
        plots[[i]] <- ggplot(plot_data, aes(x=levels, y=mean)) +
          geom_point() +
          geom_line() +
          geom_errorbar(aes(ymin = lower, ymax = upper)) +
          geom_hline(yintercept = 80, lty=2) +
          labs(title = "Sample size calculations",
               subtitle = paste0("Statistical power estimates expressed as a function of different numbers of ", target_columns[i]),
               x = paste0("Number of ", target_columns[i]),
               y = "Statistical power") +
          theme_minimal() +
          theme(plot.title = element_text(size = 12, face = "bold"))+
          theme(plot.subtitle  = element_textbox_simple())

        captions[i] <- paste0("Statistical power estimates expressed as a function of different numbers of ", target_columns[i])
      }else{
        png(paste0(output[i],".png"))
        print(plots[[i]] + ggtitle(captions[i]) + theme(plot.title = element_textbox_simple()))
        print(mtext(response_column))
        dev.off()
      }



    }
    return(list(plots = plots, captions = captions))
  }


}

#' @title diagnoseDataModel
#'
#' @description This function can be used to generate diagnostic QC plots for given model assumptions related to the input data, identify potential outlier observations and/or outlier experimental units
#'
#'
#' @param data Input data frame with columns having all the necessary information regarding the dependent and independent variables of interest
#' @param design an object of class RMeDesign with the necessary design information about the data
#' @param model an object of class ProbabilityModel giving the error distribution of the data

#' @return A list with four elements. 1) models: representing the names of the models
#' evaluated based on differnt modifications of the response column.
#' The models would include one called natural_scale,
#' another model called natural_scale_wo_outliers if outliers had beeen identified,
#' another model called log_scale if the respose column is continuous
#' and the model on the log-transformed values of the responses are what was evaluated
#' and finally log_scale_wo_outliers model if there were outliers identified in the log_scale model.
#' 2) Data_updated representing the updated data frame with additional columns for the modified response column corresponding to each of the models evaluated.
#' 3) cooks_result: cooks distance of each of the experimental columns for each of the models evaluated.
#' For models based on the binomial probability distribution, cooks distance is only
#' reported for the first experimental column on account the increased computation time
#' for evaluating this metric for the other experimental columns.
#' 4) plots_info: is a list with two elements plots and captions. plots is a named list and captions is a character vector,
#' both of the same length as the number of models evaluated. Each element of the plots list is yet another
#' list of QC/diagnostic plots related to the corresponding model fit, while the captions is a vector of captions for each of the
#' QC plots output
#'
#' @export
#'
#' @examples
#' \donttest{
#' template_dir <- system.file("input_templates/cell_assay_data", package = "RMeDPower2")
#' data <- plate_assay_pilot_data
#' design <- readDesign(file.path(template_dir,"design_cell_assay.json"))
#' model <- readProbabilityModel(file.path(template_dir,"prob_model.json"))
#' diagnose_res <- diagnoseDataModel(data, design, model)
#' }

diagnoseDataModel <- function(data, design, model) {
  transform_data(data,
                  condition_column = design@condition_column,
                  experimental_columns = design@experimental_columns,
                  response_column = design@response_column,
                  total_column = design@total_column,
                  condition_is_categorical = design@condition_is_categorical,
                  covariate= design@covariate,
                  crossed_columns = design@crossed_columns,
                  error_is_non_normal=model@error_is_non_normal,
                  family_p=model@family_p,
                  alpha=design@outlier_alpha,
                  na.action=design@na_action,
                 include_interaction = design@include_interaction,
                 random_slope_variable = design@random_slope_variable,
                 covariate_is_categorical = design@covariate_is_categorical
                 )

}

# Tests for model building via diagnoseDataModel public API

test_that("diagnoseDataModel returns correct structure", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  model <- create_test_model("normal")

  result <- suppressWarnings(suppressMessages(
    diagnoseDataModel(data = test_data, design = design, model = model)
  ))

  expect_type(result, "list")
  expect_true("models" %in% names(result))
  expect_true("Data_updated" %in% names(result))
  expect_true("diagnostic_plots" %in% names(result))
  expect_true("cooks_result" %in% names(result))
})

test_that("diagnoseDataModel returns updated data frame", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  model <- create_test_model("normal")

  result <- suppressWarnings(suppressMessages(
    diagnoseDataModel(data = test_data, design = design, model = model)
  ))

  expect_true(is.data.frame(result$Data_updated))
  expect_true(nrow(result$Data_updated) > 0)
})

test_that("diagnoseDataModel handles covariates correctly", {
  test_data <- create_test_data()
  design <- create_test_design("with_covariate")
  model <- create_test_model("normal")

  result <- suppressWarnings(suppressMessages(
    diagnoseDataModel(data = test_data, design = design, model = model)
  ))

  expect_type(result, "list")
  expect_true("Data_updated" %in% names(result))
})

test_that("diagnoseDataModel handles interactions correctly", {
  test_data <- create_test_data()
  design <- create_test_design("with_covariate")
  design@include_interaction <- TRUE
  model <- create_test_model("normal")

  expect_no_error(suppressWarnings(suppressMessages(
    diagnoseDataModel(data = test_data, design = design, model = model)
  )))
})

test_that("diagnoseDataModel handles non-normal distributions", {
  test_data <- create_count_data()
  design <- create_test_design("count_data")
  model <- create_test_model("poisson")

  result <- suppressWarnings(suppressMessages(
    diagnoseDataModel(data = test_data, design = design, model = model)
  ))

  expect_type(result, "list")
  expect_true("models" %in% names(result))
})

test_that("diagnoseDataModel handles binomial data with totals", {
  test_data <- create_count_data()
  design <- create_test_design("count_data")
  model <- create_test_model("binomial")

  result <- suppressWarnings(suppressMessages(
    diagnoseDataModel(data = test_data, design = design, model = model)
  ))

  expect_type(result, "list")
  expect_true("Data_updated" %in% names(result))
})

test_that("diagnoseDataModel handles missing data correctly", {
  test_data_na <- create_test_data_with_na()
  design <- create_test_design("basic")
  model <- create_test_model("normal")

  result <- suppressWarnings(suppressMessages(
    diagnoseDataModel(data = test_data_na, design = design, model = model)
  ))

  expect_true(is.data.frame(result$Data_updated))
  expect_true(all(complete.cases(result$Data_updated[, c(design@condition_column, design@response_column)])))
})

test_that("diagnoseDataModel preserves data structure", {
  test_data <- create_test_data()
  design <- create_test_design("basic")
  model <- create_test_model("normal")

  result <- suppressWarnings(suppressMessages(
    diagnoseDataModel(data = test_data, design = design, model = model)
  ))

  updated_data <- result$Data_updated
  expect_true(design@condition_column %in% names(updated_data))
  expect_true(design@response_column %in% names(updated_data))
})

# Tests for S4 class definitions in RMeDPower2

test_that("RMeDesign class can be created with valid parameters", {
  # Test basic construction
  design <- new("RMeDesign",
    response_column = "cell_size",
    condition_column = "treatment",
    condition_is_categorical = TRUE,
    experimental_columns = c("experiment", "plate"),
    covariate = NULL,
    covariate_is_categorical = NA,
    crossed_columns = NULL,
    include_interaction = NA,
    random_slope_variable = NULL,
    total_column = NULL,
    na_action = "complete"
  )

  expect_s4_class(design, "RMeDesign")
  expect_equal(design@response_column, "cell_size")
  expect_equal(design@condition_column, "treatment")
  expect_true(design@condition_is_categorical)
  expect_equal(design@experimental_columns, c("experiment", "plate"))
  expect_null(design@covariate)
  expect_true(is.na(design@covariate_is_categorical))
  expect_true(is.na(design@include_interaction))
})

test_that("RMeDesign class handles covariate parameters correctly", {
  design <- new("RMeDesign",
    response_column = "outcome",
    condition_column = "treatment",
    condition_is_categorical = TRUE,
    experimental_columns = c("batch", "subject"),
    covariate = "age",
    covariate_is_categorical = FALSE,
    crossed_columns = NULL,
    include_interaction = TRUE,
    random_slope_variable = "condition_column",
    total_column = NULL,
    na_action = "unique"
  )

  expect_equal(design@covariate, "age")
  expect_false(design@covariate_is_categorical)
  expect_true(design@include_interaction)
  expect_equal(design@random_slope_variable, "condition_column")
  expect_equal(design@na_action, "unique")
})

test_that("RMeDesign class handles crossed columns correctly", {
  design <- new("RMeDesign",
    response_column = "expression",
    condition_column = "genotype",
    condition_is_categorical = TRUE,
    experimental_columns = c("experiment", "cell_line"),
    covariate = NULL,
    covariate_is_categorical = NA,
    crossed_columns = "cell_line",
    include_interaction = FALSE,
    random_slope_variable = NULL,
    total_column = NULL,
    na_action = "complete"
  )

  expect_equal(design@crossed_columns, "cell_line")
})

test_that("ProbabilityModel class can be created for normal distribution", {
  model <- new("ProbabilityModel",
    error_is_non_normal = FALSE,
    family_p = NULL
  )

  expect_s4_class(model, "ProbabilityModel")
  expect_false(model@error_is_non_normal)
  expect_null(model@family_p)
})

test_that("ProbabilityModel class can be created for non-normal distributions", {
  # Test Poisson
  model_poisson <- new("ProbabilityModel",
    error_is_non_normal = TRUE,
    family_p = "poisson"
  )

  expect_true(model_poisson@error_is_non_normal)
  expect_equal(model_poisson@family_p, "poisson")

  # Test binomial
  model_binomial <- new("ProbabilityModel",
    error_is_non_normal = TRUE,
    family_p = "binomial"
  )

  expect_equal(model_binomial@family_p, "binomial")

  # Test negative binomial
  model_nb <- new("ProbabilityModel",
    error_is_non_normal = TRUE,
    family_p = "negative_binomial"
  )

  expect_equal(model_nb@family_p, "negative_binomial")
})

test_that("PowerParams class can be created with basic parameters", {
  power_params <- new("PowerParams",
    target_columns = "experiment",
    levels = 1,
    power_curve = 1,
    nsimn = 1000,
    alpha = 0.05,
    max_size = NULL,
    breaks = NULL,
    effect_size = NULL,
    icc = NULL
  )

  expect_s4_class(power_params, "PowerParams")
  expect_equal(power_params@target_columns, "experiment")
  expect_equal(power_params@levels, 1)
  expect_equal(power_params@power_curve, 1)
  expect_equal(power_params@nsimn, 1000)
  expect_equal(power_params@alpha, 0.05)
  expect_null(power_params@max_size)
})

test_that("PowerParams class can be created with multiple target columns", {
  power_params <- new("PowerParams",
    target_columns = c("experiment", "plate"),
    levels = c(1, 0),
    power_curve = 1,
    nsimn = 500,
    alpha = 0.01,
    max_size = c(10, 20),
    breaks = c(5, 10, 15),
    effect_size = c(0.5,NA,NA),
    icc = c(0.1, 0.05)
  )

  expect_equal(power_params@target_columns, c("experiment", "plate"))
  expect_equal(power_params@levels, c(1, 0))
  expect_equal(power_params@nsimn, 500)
  expect_equal(power_params@alpha, 0.01)
  expect_equal(power_params@max_size, c(10, 20))
  expect_equal(power_params@breaks, c(5, 10, 15))
  expect_equal(power_params@effect_size[1], 0.5)
  expect_equal(power_params@icc, c(0.1, 0.05))
})

test_that("PowerParams class handles single values correctly", {
  power_params <- new("PowerParams",
    target_columns = "subject",
    levels = 0,
    power_curve = 0,
    nsimn = 100,
    alpha = 0.1,
    max_size = 50,
    breaks = NULL,
    effect_size = c(1.2,NA,NA),
    icc = 0.2
  )

  expect_equal(power_params@target_columns, "subject")
  expect_equal(power_params@levels, 0)
  expect_equal(power_params@power_curve, 0)
  expect_equal(power_params@max_size, 50)
  expect_equal(power_params@effect_size[1], 1.2)
  expect_equal(power_params@icc, 0.2)
})

test_that("Class slots have correct types", {
  design <- create_test_design("basic")
  model <- create_test_model("normal")
  power_params <- create_test_power_params("basic")

  # Test RMeDesign slot types
  expect_type(design@response_column, "character")
  expect_type(design@condition_column, "character")
  expect_type(design@condition_is_categorical, "logical")
  expect_type(design@experimental_columns, "character")
  expect_type(design@covariate_is_categorical, "logical")
  expect_type(design@include_interaction, "logical")
  expect_type(design@na_action, "character")

  # Test ProbabilityModel slot types
  expect_type(model@error_is_non_normal, "logical")

  # Test PowerParams slot types
  expect_type(power_params@target_columns, "character")
  expect_type(power_params@levels, "double")
  expect_type(power_params@power_curve, "double")
  expect_type(power_params@nsimn, "double")
  expect_type(power_params@alpha, "double")
})

# Helper functions and test data for RMeDPower2 tests

# Create simple test dataset
create_test_data <- function(n_experiments = 2, n_plates = 3, n_wells = 10) {
  set.seed(123)  # For reproducible tests

  n_total <- n_experiments * n_plates * n_wells

  data.frame(
    experiment = rep(1:n_experiments, each = n_plates * n_wells),
    plate = rep(rep(1:n_plates, each = n_wells), n_experiments),
    well = rep(1:n_wells, n_experiments * n_plates),
    treatment = rep(c("control", "treated"), length.out = n_total),
    cell_size = rnorm(n_total, mean = 100, sd = 15) +
                rep(c(0, 10), length.out = n_total), # treatment effect
    batch = rep(c("A", "B"), length.out = n_total),
    age = runif(n_total, min = 20, max = 60),
    stringsAsFactors = FALSE
  )
}

# Create test data with missing values
create_test_data_with_na <- function() {
  data <- create_test_data(n_experiments = 2, n_plates = 2, n_wells = 5)
  # Introduce some missing values
  data$cell_size[c(5, 10, 15)] <- NA
  data$treatment[20] <- NA
  data$age[c(3, 8)] <- NA
  return(data)
}

# Create count data for testing non-normal distributions
create_count_data <- function(n_experiments = 2, n_plates = 3, n_wells = 10) {
  set.seed(123)

  n_total <- n_experiments * n_plates * n_wells

  data.frame(
    experiment = rep(1:n_experiments, each = n_plates * n_wells),
    plate = rep(rep(1:n_plates, each = n_wells), n_experiments),
    well = rep(1:n_wells, n_experiments * n_plates),
    treatment = rep(c("control", "treated"), length.out = n_total),
    count = rpois(n_total, lambda = rep(c(5, 8), length.out = n_total)),
    total_count = rep(100, n_total),
    stringsAsFactors = FALSE
  )
}

# Create valid design objects for testing
create_test_design <- function(type = "basic") {
  switch(type,
    "basic" = new("RMeDesign",
      response_column = "cell_size",
      condition_column = "treatment",
      condition_is_categorical = TRUE,
      experimental_columns = c("experiment", "plate"),
      covariate = NULL,
      covariate_is_categorical = NA,
      crossed_columns = NULL,
      include_interaction = FALSE,
      random_slope_variable = NULL,
      total_column = NULL,
      na_action = "complete"
    ),
    "with_covariate" = new("RMeDesign",
      response_column = "cell_size",
      condition_column = "treatment",
      condition_is_categorical = TRUE,
      experimental_columns = c("experiment", "plate"),
      covariate = "age",
      covariate_is_categorical = FALSE,
      crossed_columns = NULL,
      include_interaction = FALSE,
      random_slope_variable = NULL,
      total_column = NULL,
      na_action = "complete"
    ),
    "count_data" = new("RMeDesign",
      response_column = "count",
      condition_column = "treatment",
      condition_is_categorical = TRUE,
      experimental_columns = c("experiment", "plate"),
      covariate = NULL,
      covariate_is_categorical = TRUE,
      crossed_columns = NULL,
      include_interaction = FALSE,
      random_slope_variable = NULL,
      total_column = "total_count",
      na_action = "complete"
    )
  )
}

# Create valid probability model objects
create_test_model <- function(type = "normal") {
  switch(type,
    "normal" = new("ProbabilityModel",
      error_is_non_normal = FALSE,
      family_p = NULL
    ),
    "poisson" = new("ProbabilityModel",
      error_is_non_normal = TRUE,
      family_p = "poisson"
    ),
    "binomial" = new("ProbabilityModel",
      error_is_non_normal = TRUE,
      family_p = "binomial"
    )
  )
}

# Create valid power parameter objects
create_test_power_params <- function(type = "basic") {
  switch(type,
    "basic" = new("PowerParams",
      target_columns = "experiment",
      levels = 1,
      power_curve = 1,
      nsimn = 10,  # Small number for fast testing
      alpha = 0.05,
      max_size = 5,
      breaks = NULL,
      effect_size = NULL,
      icc = NULL
    ),
    "multiple_targets" = new("PowerParams",
      target_columns = c("experiment", "plate"),
      levels = c(1, 1),
      power_curve = 1,
      nsimn = 10,
      alpha = 0.05,
      max_size = c(5, 10),
      breaks = NULL,
      effect_size = NULL,
      icc = NULL
    )
  )
}

# Helper function to check if an object is approximately equal (for numeric comparisons)
expect_approximately_equal <- function(object, expected, tolerance = 1e-6) {
  expect_true(abs(object - expected) < tolerance)
}

# Helper function to suppress warnings for known issues during testing
suppress_known_warnings <- function(expr) {
  suppressWarnings({
    # Suppress known warnings about lme4::getData vs simr::getData masking
    expr
  })
}

# Basic functionality tests to verify package loading

test_that("Package loads correctly", {
  expect_true("RMeDPower2" %in% loadedNamespaces())
})

test_that("Main functions are available", {
  expect_true(exists("calculatePower"))
  expect_true(exists("getEstimatesOfInterest"))
  expect_true(exists("diagnoseDataModel"))
})

test_that("S4 classes are defined", {
  expect_no_error(getClass("RMeDesign"))
  expect_no_error(getClass("ProbabilityModel"))
  expect_no_error(getClass("PowerParams"))
})

test_that("Example data is available", {
  expect_true(exists("plate_assay_pilot_data"))
  data("plate_assay_pilot_data", envir = environment())
  expect_true(is.data.frame(plate_assay_pilot_data))
  expect_true(nrow(plate_assay_pilot_data) > 0)
})

test_that("Basic S4 class creation works", {
  # Create a minimal RMeDesign object using helper function
  design <- create_test_design("basic")

  expect_s4_class(design, "RMeDesign")
  expect_equal(design@response_column, "cell_size")
  expect_equal(design@condition_column, "treatment")
  expect_true(design@condition_is_categorical)
})

---
title: '*RMeDPower2*: helping design and analyses of data from repeated measures experiments'
author:
- name: "Min-Gyoung Shin"
  affiliation: Gladstone Institutes
  email: mingyoung.shin@gladstone.ucsf.edu
- name: Reuben Thomas
  affiliation: Gladstone Institutes
  email: reuben.thomas@gladstone.ucsf.edu
package: RMeDPower2
output:
  html_document:
    toc: true
    toc_float: true
    number_sections: true
  pdf_document:
    latex_engine: xelatex
    toc: true
    number_sections: true
vignette: >
  %\VignetteIndexEntry{RMeDPower2 Tutorial}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
abstract: "Biomedical research very often involves data generated from repeated measures
  experiments. RMeDPower2 is an R package that provides complete functionality to
  analyse data coming from repeated measures experiments, i.e., where one has repeated
  measures from the same biological/independent units or samples. RMeDPower2 helps
  test the modeling assumptions one makes, identify outlier observations, outlier
  units at different levels of the design, estimates statistical power or perform
  sample size calculations, estimate parameters of interest and also to visualize
  the association being tested. The functionality is limited to testing associations
  of one predictor (continuous or categorical, e.g., disease status or brain pathology)
  along with one another covariate (e.g., gender status) in the context of hierarchical
  or crossed experimental designs. This vignette illustrates the use of this package
  in multiple contexts relevant to biomedical research. \n\nRMeDPower2 defines the
  experimental design for the data, probability model of the data generating distribution
  and necessary parameters required for sample size calculation using convenient S4
  class objects. It uses these objects in one framework that brings together the functionality
  implemented in multiple R packages - lme4 (implementation of linear mixed effects
  models), influence.ME (identification of outlier units), EnvStats (identification
  of outlier observations), DHARMa (testing of modeling assumptions for non-normal
  distributions), simr (sample-size calculations) and tidyverse (data manipulation
  and visualization).\n"
---
```{r setup, include=FALSE}
knitr::opts_chunk$set(warning = FALSE, message = FALSE)
```

# Getting started
```{r echo=FALSE}
template_dir <- system.file("input_templates/cell_assay_data",
                            package = "RMeDPower2")
```
## Installing `RMeDPower2`
`RMeDPower2` can be installed using the following commands
```{r eval=FALSE}
install.packages("devtools")
library(devtools)
install_github('gladstone-institutes/RMeDPower2', build_vignettes=TRUE, force = TRUE)
```

## Quick tutorial
The workflow can be accomplished in 5 main steps as described in the sections below (1.2.1-.2.5).

### Load libraries and read input data
The first step is to load RMeDPower2 and read-in the input data as a data frame. This is accomplished by the following commands:

```{r results='hide'}
suppressPackageStartupMessages(library(RMeDPower2))
suppressPackageStartupMessages(library(ggplot2))
suppressPackageStartupMessages(library(dplyr))
suppressPackageStartupMessages(library(magrittr))

data <- plate_assay_pilot_data
```

Example data has experiment, line as experience columns, classification as the condition column, and cell size as the response column.

```{r}
knitr::kable(data[1:5,] )
```


### Specify design, model and power parameters

The following set of commands allows users to specify parameters for experimental design, assumed probability models and distributions, and power simulations. 

1. Use a text editor to modify the template jsonfile capturing the design for the data and read in this file.

```{r}
design <- readDesign(file.path(template_dir,"design_cell_assay.json"))
```

2. Use a text editor to modify the template jsonfile capturing the error probability model and read in this file.

```{r}
model <- readProbabilityModel(file.path(template_dir,"prob_model.json"))
```

3. Use a text editor to modify the template jsonfile capturing the power parameters and read in this file.

```{r}
power_param <- readPowerParams(file.path(template_dir,"power_param.json"))
```


### Diagnose data and model

The next set of commands will allow the user to evaluate the datatype, examine if the model is appropriate and identity outliers. 

1. To diagnose the data and model, test model assumptions, identify outlier observations, outlier biological/independent units.

```{r eval=FALSE}
diagnose_res <- diagnoseDataModel(data, design, model)
data_update = diagnose_res$Data_updated
```

2. Modify design objects (e.g., remove outliers), model objects (e.g., change probability distribution assumption) if needed based on output of step 1 above.

```{r eval=FALSE}
diagnose_res_update <- diagnoseDataModel(data_update, design_update, model_update)
```

3. Repeat steps 1 and 2 until the models assumptions are met. For example, users can choose to remove outlier-experiments so that the distribution is close to normal.


### Statistical power estimation

This section of code will allow the user to run sample-size calculations from the pilot input data. 

```{r eval=FALSE}
power_res <- calculatePower(data, design, model, power_param)
```

### Estimate parameters of interest

Once we have run the models, the user can estimate and visualize parameters of interest in order to investigate the association between response variable and condition variables.

```{r eval=FALSE}
estimate_res <- getEstimatesOfInterest(data, design, model, print_plots = F)
```


## Input data

Users will have to ensure that the input data is organized in a format that will allow RMedPower2 to read in the data. The data should be structured in a table with rows representing the  individual observations or responses and columns representing not only the corresponding explanatory variables but also other variables that capture the hierarchical or crossed design of how the data was generated. It is important that the columns have names or headers - these column names will be used to define the relevant S4 class objects. 

For example, the next below illustrates the input data from a cellular assay where each of the 2,588 observations represents a measurement of size (`cell_size2`) of a single cell, from a given cell line (`line`) that is either from a control or disease condition (`classification`) and is measured in a given experimental batch (`experiment`).

```{r echo=FALSE}
suppressPackageStartupMessages(require(dplyr))
suppressPackageStartupMessages(library(RMeDPower2))
template_dir <- system.file("input_templates/cell_assay_data",
                            package = "RMeDPower2")

data <- plate_assay_pilot_data %>%
  select(experiment, line, classification, cell_size2) %>%
  mutate(experiment = as.factor(experiment), line = as.factor(line), classification = as.factor(classification))

str(data)
```


## Three key classes of `RMeDPower2`

Here, users will learn how to set up parameters for experimental designs, probability and distributional assumptions, and power simulations.


### `RMeDesign` S4 class

The underlying design for the data will be stored in an object (R data structure with variables) of this class. The slots of this class are: 

1. `response_column`: character that is the column name in the input data table that captures the responses or individual observations. This has to be set by the user.

2. `covariate`: character that is the column name in the input data table that captures the covariate information. This will be used as a fixed effect in the mixed effects model of the data. This slot is set to `NULL` if there are no covariates.

3. `condition_column`: character that is the column name in the input data table that captures the predictor/independent variable information. This will be used as a fixed effect in the mixed effects model of the data.This has to be set by the user.

4. `experimental_columns`: character vector that represents the column names in the input data table that captures the experimental variables of the design. These will be used as random effects in the mixed effects model of the data. The order of the names in this character vector is important. The first element captures the top hierarchical level of the design, the second element the next level of the hierarchy and so on. The hierarchy is explicitly modeled in the specification of the mixed effects models. The hierarchy will not be modeled for variables specified in the `crossed_columns` slot (see below). This has to be set by the user. 

5. `condition_is_categorical`: logical value equal to TRUE if the variable in the `condition_column` is to be considered as a categorical variable and is equal to FALSE otherwise. Defaults to TRUE.

6. `crossed_columns`: character vector that represents the column names in the input data table that are a subset of the values in `experimental_columns` representing crossed factors. Defaults to `NULL`.

7. `total_column`: character that is the column name in the input table that will be used to offset values in the mixed effects models. Useful for modeling count data. Defaults to `NULL`.
  
8. `outlier_alpha`: numeric value that is the p-value threshold used to identify outlier observations. Defaults to 0.05. 

9. `na_action`: character equal to _complete_ or _unique_. To be used in the context where the input data has multiple responses that will each be sequentially modeled. For example, from a single set of data, a user is gathering numerous observations.  Here we refer to _complete_ in the scenario  where the observed outliers that identified for one response will also be identified as outliers for all the other responses while  _unique_ refers to the situation where the outlier observations identified for one response will only be used for the model of that particular response. Defaults to _complete_. 

10. `include_interaction`: Whether to include condition * covariate interaction

11. `random_slope_variable`: Variable for random slopes (typically "condition_column")

12.  `covariate_is_categorical`: Specify whether the covariate variable is categorical. TRUE: Categorical, FALSE: Continuous.


To create an object of class RMeDesign, the following code is used:

```{r}
design <- new("RMeDesign")
print(design)
```

In practice, for a given data set the design can be input from a jsonfile. The user can use a text editor to modify the `design_template.json` file available with the package to input the relevant information based on the column names of the input data. For example, the design for the cell size data referred to above can be read using the function `readDesign`.

```{r echo=FALSE}
template_dir <- system.file("input_templates/cell_assay_data",
                            package = "RMeDPower2")
```
```{r}
design <- readDesign(file.path(template_dir, "design_cell_assay.json"))
print(design)
```

### `ProbabilityModel` S4 class
The error probability distribution is specified by two slots in objects of the `ProbabilityModel` class.

1. `error_is_non_normal`: Is the logical value that is TRUE if the underlying probability distribution is not assumed to be Normal and is FALSE otherwise.

2. `family_p`: This character value specifies the probability distribution. Accepted values are _poisson_(`=poisson(link="log")`), _binomial_(`=binomial(link="logit")`), _bionomial_log_(`=binomial(link="log")`), _Gamma_log_(`=Gamma(link="log")`), _Gamma_(`=Gamma(link="inverse")`), _negative_binomial_. Defaults to `NULL`.

```{r}
model <- readProbabilityModel(file.path(template_dir, "prob_model.json"))
print(model)
```

### `PowerParams` S4 class

The parameters described below are necessary for sample-size estimation and are stored in the following slots of the `PowerParams` class.

1. `target_columns`: This describes the character vector with column names of experimental variables for which the sample-size estimation is to be performed
2. `power_curve`: This is numeric 1 or 0. 1: Power simulation over a range of sample sizes or levels. 0: Power calculation over a single sample size or a level. Defaults to 1.
3. `nsimn`: The number of simulations to estimate power. Defaults to 10.
4. `levels`: numeric 1 or 0. 1: Amplify the number of corresponding target parameter. 0: Amplify the number of samples from the corresponding target parameter, ex) If target_columns = c("experiment","cell_line") and if you want to expand the number of experiment and sample more cells from each cell line, set levels = c(1,0).
5. max_size: Maximum levels or sample sizes to test. Default if set to `NULL` equals the current level or the current sample size x 5. ex) If max_levels = c(10,5), it will test upto 10 experiments and 5 cell lines.
6. `alpha`: Type I error for sample-size estimation. Defaults to 0.05.
7. `breaks`: Levels /sample sizes of the variable to be specified along the power curve. Default if set to NULL equals max (1, round (the number of current levels / 5)).
8. `effect_size`: A 3 dimensional numeric vector given the proposed effect sizes/parameter estimates for the condition_column, covariate and interaction term between these two variables, respectively in that order. Depending on the model not all elements of the effect_size would make sense. For example, in situations, where there is only the condition column, the second and third elements of effect_size should be equal to NA. If you know the effect size of your condition variable, the effect size can be provided as a parameter. Default set to `NULL`,i.e., if the effect size is not provided then it will be estimated from your data.
9. `icc`: Intra-class coefficients for each of the experimental variables. Used only in the situation when `error_is_non_normal=FALSE` and when the data does not allow variance component estimation.

```{r}
power_param <- readPowerParams(file.path(template_dir, "power_param.json"))
print(power_param)
```

## Three key functions of `RMeDPower2`

The following are functions for users to manipulate data, estimate power, and perform association analysis.

### `diagnoseDataModel(data, design, model)` function

This set of code, tests the model assumptions by using quantile-quantile, residuals vs fitted and residuals versus predicted plots. If needed, the code also transforms the responses if feasible and identifies outlier observations and also outlier experimental units.

### `calculatePower(data, design, model, power_param)` function

This set of code performs sample-size calculations for given experimental design/target variable.

### `getEstimatesOfInterest(data, design, model)` function
This string estimates parameter of interest and also plots the resulting association with predictor of interest using residuals from the model that removes the effects of the modeled experimental variables.

An overview of the anticipated usage of the functionality of this package is shown in the figure below.

```{r flowchart, echo=FALSE, fig.align = "center", fig.cap="Flowchart illustrating the functionality of the package", fig.small=TRUE, echo=FALSE}
knitr::include_graphics(system.file("figures/package_structure.png", package = "RMeDPower2"))
```

# Application examples
Here we illustrate the application of RMeDPower2 using data derived from in vitro assays. We further illustrate the RMeDPower2 across the other domains in biomedical research on our website: https://gladstone-institutes.github.io/RMeDPower2/index.html .
```{r applications, echo=FALSE, fig.align = "center", fig.cap="Flowchart illustrating the functionality of the package", fig.small=TRUE, echo=FALSE}
knitr::include_graphics(system.file("figures/application_domains.png", package = "RMeDPower2"))
```


## Plate assays

iPSC lines from control and patient derived iPSCs from sALS patients were obtained through the Answer ALS (AALS) consortium36 . AALS is the largest repository of ALS patient samples, and contains publicly available patient-specific iPSCs and multi-OMIC data from motor neurons derived from those iPSCs—including RNA Seq, proteomics and whole genome sequence data— all of which is matched with longitudinal clinical data (https://dataportal.answerals.org/home)36. 


We used 6 control lines and 4 sALS lines and differentiated each line into motor neurons (iMNs). iMNs were  transduced with a morphology marker and imaged on day ~25 using RM37-45 as previously
described36. Images were processed and the soma size of neurons were captured using contour ellipse46 function adapted for use in python (https://www.python.org/).

The next step is to read the data and perform data distribution diagnostics, association analysis, and power simulation.

First, we will read the data from the package and json files with specified parameters:

```{r}
template_dir <- system.file("input_templates/cell_assay_data", package = "RMeDPower2")
data <- plate_assay_pilot_data
design <- readDesign(file.path(template_dir,"design_cell_assay.json"))
model <- readProbabilityModel(file.path(template_dir,"prob_model.json"))
power_param <- readPowerParams(file.path(template_dir,"power_param.json"))
```

A visualization of the distribution of cell_size2 across experiments and cell-lines as box-plots and also in terms of empirical cumulative distribution plots suggests the observations have an extremely long tail. We will therefore filter out the top 5 percent of the observations to ensure the results are not skewed by these extreme observations. We also see that the observations in experiment 3 are in general lower than those in the other two experiments.
```{r echo=FALSE}
print(ggplot(data, aes(x=experiment, y=cell_size2, color = line)) +
  geom_boxplot() +
  theme_bw() +
  theme(text = element_text(size = 16)))

print(ggplot(data, aes(x=cell_size2, color = experiment)) +
  stat_ecdf() +
  theme_bw() +
  theme(text = element_text(size = 16)))
```

```{r}
data %<>% filter(cell_size2 < quantile(cell_size2, 0.95))
```

We would like to estimate the difference in the mean cell-sizes across all the observations derived from all experimental batches between the ALS disease lines and the control cell lines (modeled by the _classification_ variable). The first step is run diagnosis on the data and model using the 'diagnoseDataModel' function. The Q-Q plot of the residuals of the LMM model fit to the log-transformed cell-sizes suggests a better fit than the one fit to the cell sizes in the natural scale , assuming that the data generating distribution is based on the Normal distribution. Therefore, we will hereafter model the log-transformed cell sizes. The reasonably horizontal best fit lines for the residuals from this versus the fitted values,  and the square-root of the absolute values of the residuals versus the fitted values, Figure 7e suggest both the LMM model assumptions of linearity and homoscedasticity are reasonable. There were no identified individual outlier observations based on the application of the Rosner’s test to the distribution of the residuals in.


```{r}
diagnose_res <- diagnoseDataModel(data, design, model)
```
The *diagnoseDataModel* function fits  at least one model representing one in the natural scale of the response of interest. Additionally, if individual observations are inferred as outliers using the rosner's test applied to the residuals of the fitted model then another model is fit without these outlier observations. In addition, if the assumed probability distribution of the residuals of the model is normal then a further additional model are fit using logarithm (log) transformed responses and another one if there are outliers in the log-transformed model fit.

To identify all the models fit to the data
```{r}
print(diagnose_res$models)
```
We see that two models were fit to this data - one in the natural scale of cell size and the other in the logarithmic scale. No individual observations were identified as outliers in each of these models.

Let us look the basic QC plots for the models fit in the natural scale of the responses.
```{r output-plots1, results='asis'}
plot_no <- 1
plots <- diagnose_res$diagnostic_plots$natural_scale$plots
captions <- diagnose_res$diagnostic_plots$natural_scale$captions
for (i in seq_along(plots)) {
  print(plots[[i]])
  cat(sprintf("\n\n\n**Figure %d:** %s. %s\n\n", plot_no,names(plots)[i], captions[i]))
  plot_no <- plot_no + 1
}

cat("\n\n\n")
cooks_plots <- diagnose_res$cooks_plots$natural_scale
for(i in 1:length(cooks_plots)) {
  print(cooks_plots[[i]])
  cat(sprintf("\n\n\n**Figure %d:**\n\n", plot_no))
  plot_no <- plot_no + 1
}

print("Inferred outlier groups are")
print(diagnose_res$inferred_outlier_groups$natural_scale)
```

Now, let us look at the same set of QC plots for models on the log-transformed responses.
```{r output-plots2, results='asis'}
plots <- diagnose_res$diagnostic_plots$log_scale$plots
captions <- diagnose_res$diagnostic_plots$log_scale$captions
for (i in seq_along(plots)) {
  print(plots[[i]])
  cat(sprintf("\n\n\n**Figure %d:** %s. %s\n\n", plot_no,names(plots)[i], captions[i]))
  plot_no <- plot_no + 1
}


cooks_plots <- diagnose_res$cooks_plots$log_scale

cat("\n\n\n")
for(i in 1:length(cooks_plots)) {
  print(cooks_plots[[i]])
  cat(sprintf("\n\n\n**Figure %d:**\n\n", plot_no))
  plot_no <- plot_no + 1
}

print("Inferred outlier groups are")
print(diagnose_res$inferred_outlier_groups$natural_scale)
```

We see normality assumption is better met with the log-transformed model (Figure 6) compared to the model in the natural scale (Figure 1).

The code also outputs Cook's distance measures for each experiment and cell-line that is intended to capture how influential each of these were in determining the final parameter estimates in the LMM model.


The observations in experimental batch 3 and those for cell lines 4 and 9 were identified as outliers using a 4/n (where n is the number of replicates, n=3 for experimental batches and n=10 for cell-lines) threshold applied to the estimates of Cook’s distance for the estimated model parameters. This warranted further investigation on whether or not there was something unusual with the observations made in experimental batch 3 (example: the images from the microscope displayed low signal to noise and may have skewed the measurements) and also for cell line 4# and 9# (example: differentiation runs, genetic background was different from the other lines) that would suggest observations linked to this experiment and cell lines may need to be removed from the analysis. Our investigation did not indicate anything untoward and therefore we ran a sensitivity analysis where we dropped all observations linked to either experiment 3, cell line 4# or cell line 9#, refit the model and visualized the resulting change in ALS status associated mean soma perimeter value. There were only relatively slight changes in the estimated changes in the mean, so that the conclusions we would draw of an increase in the soma cell perimeter in ALS lines remains irrespective of whether or not we drop observations from the experimental batch or the two cell lines.

 We will use the log transformed cell_size2 data for parameter estimation using the following commands. 

```{r}
design_update <- design
data_update <- diagnose_res$Data_updated
print(colnames(data_update))
design_update@response_column <-  "cell_size2_logTransformed"
estimate_res <- getEstimatesOfInterest(data_update, design_update, model,print_plots = FALSE)
```

```{r, results="asis", echo = FALSE}
print(estimate_res[[2]]$plots)
knitr::asis_output(sprintf("\n\n\n**Figure %d:**\n\n", plot_no))
plot_no <- plot_no + 1

```

The median residual boxplot shows the differences in the distribution of the response variable for the two conditions.

```{r}
##print model summary output
print(estimate_res[[1]])
```

As a result of linear mixed model-based association analysis, the response variable was found to be significantly associated with the condition variable with an estimate of 0.03.

We can also estimate the statistical needed to estimate the observed differences in mean perimeter across 15 experimental batches

```{r}
power_param@max_size <- 15
power_res <- calculatePower(data_update, design_update, model, power_param)
```

```{r,results="asis", echo = FALSE}
print(power_res$plots)
knitr::asis_output(sprintf("\n\n\n**Figure %d:**\n\n", plot_no))
plot_no <- plot_no + 1
```

The power curve shows that at least eight or more experimental batches are required to achieve a power of 80% or greater. The points in the plot represent the mean estimate of power over the simulations while the error bars represent the 95% confidence intervals.

Accordingly, we will now load data from 8 experimental batches and estimate our parameter of interest.

```{r}
full_data <- plate_assay_full_data

##filter the top 5% of the observations
full_data %<>% filter(perim_2th_effect < quantile(perim_2th_effect, 0.95))

design@response_column <- "perim_2th_effect"
design@condition_column <- "classif"
full_diag_res <- diagnoseDataModel(full_data, design, model)
full_data_update <- full_diag_res$Data_updated
full_design <- design
full_design@response_column <- "perim_2th_effect_logTransformed"
```

Note now, none of the experimental batches or the 13 cell-lines are considered outliers or unduly influencing the final parameter estimates.

```{r, results='asis'}
cooks_plots <- full_diag_res$cooks_plots$log_scale
cat("\n\n\n")
for(i in 1:length(cooks_plots)) {
  print(cooks_plots[[i]])
  cat(sprintf("\n\n\n**Figure %d:**\n\n", plot_no))
  plot_no <- plot_no + 1
}

print("Inferred outlier groups are")
print(diagnose_res$inferred_outlier_groups$natural_scale)

```

We derive an estimate of 0.027 as the mean difference in perimeter with an associated significance of 0.006 using the data from 8 experimental batches.

```{r}
full_res <- getEstimatesOfInterest(full_data_update, full_design, model, print_plots = FALSE)
print(full_res[[1]])
```

```{r, results="asis", echo = FALSE}
print(full_res[[2]]$plots)
knitr::asis_output(sprintf("\n\n\n**Figure %d:**\n\n", plot_no))
plot_no <- plot_no + 1
cat("\n\n\n")
```

# Website

Please check https://gladstone-institutes.github.io/RMeDPower2/index.html for more detailed tutorials in other applications involving single cell RNA, mouse behavior and electrophysiology experiments. These will include illustration of the application of RMeDPower2 to non-normal count data and an explanation of the simulation setup for sample size calculations. There is also an guide to defining and explaining the required classes by the package. The guide include an Rshiny app to help an user appropriately define the classes appropriate for their experimental design and question of interest.



# Session info

```{r sessionInfo, echo=FALSE}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readDesign.R
\name{readDesign}
\alias{readDesign}
\title{readDesign}
\usage{
readDesign(jsonfile)
}
\arguments{
\item{jsonfile}{the jsonfile with the necessary design parameters: condition_column, experimental_columns, response_column, total_column, condition_is_categorical, covariate, method, crossed_columns, error_is_non_normal, family_p, outlier_alpha, na.action}
}
\value{
an object of class RMeDesign
}
\description{
This functions reads the underlying design for the data
}
\examples{
template_dir <- system.file("input_templates/cell_assay_data", package = "RMeDPower2")
design <- readDesign(file.path(template_dir,"design_cell_assay.json"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main_classes.R
\docType{class}
\name{PowerParams-class}
\alias{PowerParams-class}
\title{PowerParams-class}
\arguments{
\item{power_curve}{1: Power simulation over a range of sample sizes or levels. 0: Power calculation over a single sample size or a level.}

\item{nsimn}{The number of simulations to run. Default=1000}

\item{target_columns}{Name of the experimental parameters to use for the power calculation.}

\item{levels}{1: Amplify the number of corresponding target parameter. 0: Amplify the number of samples from the corresponding target parameter, ex) If target_columns = c("experiment","cell_line") and if you want to expand the number of experiment and sample more cells from each cell line, set levels = c(1,0).}

\item{max_size}{Maximum levels or sample sizes to test. Default: the current level or the current sample size x 5. ex) If max_levels = c(10,5), it will test upto 10 experiments and 5 cell lines.}

\item{breaks}{Levels /sample sizes of the variable to be specified along the power curve. Default: max(1, round( the number of current levels / 5 ))}

\item{effect_size}{If you know the effect size of your condition variable, the effect size can be provided as a parameter. If the effect size is not provided, it will be estimated from your data}

\item{alpha}{Threshold for Type I error}

\item{ICC}{Intra-Class Coefficients (ICC) for each parameter}
}
\value{
an object of class ProbabilityModel
}
\description{
Objects of PowerParams class store information required for sample size estimation for given data
}
\examples{
power_param=new("PowerParams")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-documentation.R
\docType{data}
\name{snRNAseq_gene_count_data}
\alias{snRNAseq_gene_count_data}
\title{Single-nucleus RNA-seq gene-level count data}
\format{
A data frame of gene-level counts and associated
  annotations.
}
\usage{
data(snRNAseq_gene_count_data)
}
\description{
Example dataset containing gene-level count summaries from a
single-nucleus RNA-seq experiment. This dataset can be used
to demonstrate power calculations for differential
expression-type analyses across experimental conditions.
}
\seealso{
\code{\link{snRNAseq_cluster_count_data}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readPowerParams.R
\name{readPowerParams}
\alias{readPowerParams}
\title{readPowerParams}
\usage{
readPowerParams(jsonfile)
}
\arguments{
\item{jsonfile}{the jsonfile with the necessary parameters for statistical power estimation: target_columns, power_curve, nsimn, levels, max_size, alpha, breaks, effect_size, icc}
}
\value{
an object of class PowerParams
}
\description{
This functions reads the underlying design for the data
}
\examples{
template_dir <- system.file("input_templates/cell_assay_data", package = "RMeDPower2")
power_param <- readPowerParams(file.path(template_dir,"power_param.json"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-documentation.R
\docType{data}
\name{mouse_brain_electro_physiology_data}
\alias{mouse_brain_electro_physiology_data}
\title{Mouse brain electrophysiology data}
\format{
A data frame of electrophysiological measurements and
  associated experimental annotations.
}
\usage{
data(mouse_brain_electro_physiology_data)
}
\description{
Example dataset containing electrophysiological measurements
from mouse brain recordings. The data are used to demonstrate
power analyses in experiments with repeated measurements and
complex correlation structures.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readProbabilityModel.R
\name{readProbabilityModel}
\alias{readProbabilityModel}
\title{readProbabilityModel}
\usage{
readProbabilityModel(jsonfile)
}
\arguments{
\item{jsonfile}{the jsonfile with the necessary parameters for probability model: error_is_non_normal, family_p}
}
\value{
an object of class ProbabilityModel
}
\description{
This functions reads the underlying design for the data
}
\examples{
template_dir <- system.file("input_templates/cell_assay_data", package = "RMeDPower2")
model <- readProbabilityModel(file.path(template_dir,"prob_model.json"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-documentation.R
\docType{data}
\name{plate_assay_full_data}
\alias{plate_assay_full_data}
\title{Full plate assay dataset}
\format{
A data frame containing the full plate assay data.
  Column definitions follow those of
  \code{\link{plate_assay_pilot_data}}.
}
\usage{
data(plate_assay_full_data)
}
\description{
Full plate assay dataset corresponding to the pilot data but
including the complete experimental run. This dataset is used
in examples demonstrating power calculations under more
realistic sample sizes and hierarchies.
}
\seealso{
\code{\link{plate_assay_pilot_data}},
  \code{\link{plate_assay_pilot_data_wo_repeats}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-documentation.R
\docType{data}
\name{snRNAseq_cluster_count_data}
\alias{snRNAseq_cluster_count_data}
\title{Single-nucleus RNA-seq cluster-level count data}
\format{
A data frame of cluster-level counts and associated
  annotations.
}
\usage{
data(snRNAseq_cluster_count_data)
}
\description{
Example dataset containing cluster-level count summaries from
a single-nucleus RNA-seq experiment. The data are intended to
illustrate how RMeDPower2 can be applied to hierarchical
omics experiments with counts aggregated at the cluster
level.
}
\seealso{
\code{\link{snRNAseq_gene_count_data}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-documentation.R
\docType{data}
\name{plate_assay_pilot_data}
\alias{plate_assay_pilot_data}
\title{Pilot plate assay data}
\format{
A data frame with observations from a pilot plate
  assay. Column names and structure are documented in the
  package vignette and example code.
}
\usage{
data(plate_assay_pilot_data)
}
\description{
Pilot dataset from plate-based assays used in the RMeDPower2
documentation and examples. The data represent repeated
measurements across plates and experimental units and are
intended for illustrating experimental design specification,
model diagnostics, and power calculations.
}
\seealso{
\code{\link{plate_assay_pilot_data_wo_repeats}},
  \code{\link{plate_assay_full_data}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getEstimatesOfInterest.R
\name{getEstimatesOfInterest}
\alias{getEstimatesOfInterest}
\title{getEstimatesOfInterest}
\usage{
getEstimatesOfInterest(data, design, model, print_plots = TRUE)
}
\arguments{
\item{data}{Input data frame with columns having all the necessary information regarding the dependent and independent variables of interest}

\item{design}{an object of class RMeDesign with the necessary design information about the data}

\item{model}{an object of class ProbabilityModel giving the error distribution of the data}

\item{print_plots}{Whether or not to print the plots, irrespective of this argument ggplot versions of evaluated association between the response_column and the condition_column. TRUE - print the plot, FALSE - do not print the plot}
}
\value{
a list with two elements - 1. an object of class summary.merMod and
2. the output from the get_residuals functions. This output consists of a list
with 3 elements. 1. The updated input data with an additional column with the model residuals of the individual observations. 2. A plot representing the purported association between the response column and the condition column. 3. The corresponding caption for this figure.
}
\description{
This function performs the estimations of interest and also visualizes the resulting association
}
\examples{
\donttest{
template_dir <- system.file("input_templates/cell_assay_data", package = "RMeDPower2")
data <- plate_assay_pilot_data
design <- readDesign(file.path(template_dir,"design_cell_assay.json"))
model <- readProbabilityModel(file.path(template_dir,"prob_model.json"))
res <- getEstimatesOfInterest(data, design, model)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculatePower.R
\name{calculatePower}
\alias{calculatePower}
\title{calculatePower}
\usage{
calculatePower(data, design, model, power_param)
}
\arguments{
\item{data}{Input data frame with columns having all the necessary information regarding the dependent and independent variables of interest}

\item{design}{an object of class RMeDesign with the necessary design information about the data}

\item{model}{an object of class ProbabilityModel giving the error distribution of the data}

\item{power_param}{an object of class PowerParams giving the target parameter of interest and the other necessary parameter to perform the power estimation}
}
\value{
A power curve as a ggplot object or a power calculation result printed in a text file
}
\description{
This functions makes statistical power estimates given the data, the underlying design for it and the assumed probability model of the error distribution
}
\examples{
\donttest{
template_dir <- system.file("input_templates/cell_assay_data", package = "RMeDPower2")
data <- plate_assay_pilot_data
design <- readDesign(file.path(template_dir,"design_cell_assay.json"))
model <- readProbabilityModel(file.path(template_dir,"prob_model.json"))
power_param <- readPowerParams(file.path(template_dir,"power_param.json"))
power_res <- calculatePower(data, design, model, power_param)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main_classes.R
\docType{class}
\name{ProbabilityModel-class}
\alias{ProbabilityModel-class}
\title{ProbabilityModel-class}
\arguments{
\item{error_is_non_normal}{Default: the observed variable is continuous Categorical response variable will be implemented in the future. TRUE: Categorical , FALSE: Continuous (default).}

\item{family_p}{The type of distribution family to specify when the response is categorical. If family is "binary" then binary(link="log") is used, if family is "poisson" then poisson(link="logit") is used, if family is "poisson_log" then poisson(link=") log") is used.}
}
\value{
an object of class ProbabilityModel
}
\description{
Objects of ProbabilityModel class store information on the assumed probability distribution for the model
}
\examples{
model=new("ProbabilityModel")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main_classes.R
\docType{class}
\name{RMeDesign-class}
\alias{RMeDesign-class}
\title{RMeDesign-class}
\arguments{
\item{data}{Input data}

\item{condition_column}{Name of the condition variable (ex variable with values such as control/case). The input file has to have a corresponding column name}

\item{experimental_columns}{Name of the variable related to experimental design such as "experiment", "plate", and "cell_line". They should be in order, for example, "experiment" should always come first .}

\item{response_column}{Name of the variable observed by performing the experiment. ex) intensity.}

\item{total_column}{Set this column only when family_p="binomial" and it is equal to the total number of observations (number of cases plus number of controls) for a given number of cases}

\item{outlier_alpha}{numeric scalar between 0 and 1 indicating the Type I error associated with the test of outliers}

\item{condition_is_categorical}{Specify whether the condition variable is categorical. TRUE: Categorical, FALSE: Continuous.}

\item{covariate}{The name of the covariate to control in the regression model}

\item{method}{The method used to detect outliers. "rosner" (default) runs Rosner's test and "cook" runs Cook's distance.}

\item{crossed_columns}{Name of experimental variables that may appear repeatedly with the same ID. For example, cell_line C1 may appear in multiple experiments, but plate P1 cannot appear in more than one experiment}

\item{error_is_non_normal}{Default: the observed variable is continuous Categorical response variable will be implemented in the future. TRUE: Categorical , FALSE: Continuous (default).}

\item{family_p}{The type of distribution family to specify when the response is categorical. If family is "binary" then binary(link="log") is used, if family is "poisson" then poisson(link="logit") is used, if family is "poisson_log" then poisson(link=") log") is used.}

\item{na.action}{"complete": missing data is not allowed in all columns (default), "unique": missing data is not allowed only in condition, experimental, and response columns. Selecting "complete" removes an entire row when there is one or more missing values, which may affect the distribution of other features.}

\item{include_interaction}{logical - TRUE or FALSE - Whether to include condition * covariate interaction}

\item{random_slope_variable}{Variable for random slopes (typically one of "condition_column" or "covariate" and assuming that they are numeric variables). A random slope term is added for each of the variables specified in the experimental columns in addition to their corresponding random intercept terms. The random slope and intercept terms for each experimental_columns variable are assumed to be uncorrelated.}

\item{covariate_is_categorical}{Specify whether the covariate variable is categorical. TRUE: Categorical, FALSE: Continuous.}
}
\value{
an object of class RMeDesign
}
\description{
Objects of RMeDesign class store information on the relevant repeated measures design for the given data
}
\examples{
design=new("RMeDesign")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-documentation.R
\docType{data}
\name{mouse_behavior_MWM_assay_data}
\alias{mouse_behavior_MWM_assay_data}
\title{Mouse behavior data from a Morris Water Maze assay}
\format{
A data frame of behavioral measurements with
  information on mouse, trial, and experimental condition.
}
\usage{
data(mouse_behavior_MWM_assay_data)
}
\description{
Example behavioral dataset containing measurements from a
mouse Morris Water Maze (MWM) assay. The data represent
repeated measures across trials and subjects and are suitable
for illustrating repeated measures power analysis.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-documentation.R
\docType{data}
\name{plate_assay_pilot_data_wo_repeats}
\alias{plate_assay_pilot_data_wo_repeats}
\title{Pilot plate assay data without repeated measurements}
\format{
A data frame containing the pilot plate assay data
  without repeated measurements. See the vignette for details
  on columns and preprocessing.
}
\usage{
data(plate_assay_pilot_data_wo_repeats)
}
\description{
Version of \code{\link{plate_assay_pilot_data}} where repeated
measurements have been removed, suitable for power analyses
that assume a single observation per experimental unit at each
time point or condition.
}
\seealso{
\code{\link{plate_assay_pilot_data}},
  \code{\link{plate_assay_full_data}}
}
\keyword{datasets}
